(*  Copyright (c) 2002, the CodeLens project
  All rights reserved.

  Redistribution and use in source and binary forms,
  with or without modification, are permitted provided
  that the following conditions are met:

  Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.

  Redistributions in binary form must reproduce the
  above copyright notice, this list of conditions
  and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
  Neither the name of the CodeLens project nor the names
  of its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS
  AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. *)

unit frmRefactorNodeTreeView;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  SynEdit,
  ExtCtrls,
  ComCtrls,
  ActnList,
  Menus,
  SynEditHighlighter,
  SynHighlighterPas,
  ucwRefactoring,
  ucwRefactorNodes,
  ucwClasses;

type
  TRefactorNodeTreeViewRefactoring = class(TcwRefactoring)
  protected
    function GetCanExecute: Boolean; override;
    function GetMenuCaption: string; override;
    function GetName: string; override;
    procedure Execute; override;
  end;

  TRefactorNodeTreeViewForm = class(TForm)
    RefactorTreeView: TTreeView;
    Splitter1: TSplitter;
    RefactorSynEdit: TSynEdit;
    RefactorStatusBar: TStatusBar;
    SynPasSyn1: TSynPasSyn;
    MainMenu1: TMainMenu;
    ActionList1: TActionList;
    GenerateAction: TAction;
    File1: TMenuItem;
    Generate1: TMenuItem;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure RefactorTreeViewChange(Sender: TObject; Node: TTreeNode);
    procedure GenerateActionExecute(Sender: TObject);
  private
    FModuleNode: TcwModuleNode;
    procedure LoadTreeView;
  public
    constructor Create(const AModuleNode: TcwModuleNode); reintroduce;
  end;

implementation

{$R *.dfm}

uses
  TypInfo,
  mwPasLexTypes;

{ TRefactorNodeTreeViewRefactoring }

procedure TRefactorNodeTreeViewRefactoring.Execute;
begin
  TRefactorNodeTreeViewForm.Create(TryCreateModuleNode(gcwRefactoringHost.SelectedUnit)).ShowModal;
end;

function TRefactorNodeTreeViewRefactoring.GetCanExecute: Boolean;
begin
  Result := Assigned(gcwRefactoringHost.SelectedUnit);
end;

function TRefactorNodeTreeViewRefactoring.GetMenuCaption: string;
begin
  Result := 'Parse Tree...';
end;

function TRefactorNodeTreeViewRefactoring.GetName: string;
begin
  Result := 'ParseTree';
end;

{ TRefactorNodeTreeViewForm }

constructor TRefactorNodeTreeViewForm.Create(const AModuleNode: TcwModuleNode);
begin
  FModuleNode := AModuleNode;
  inherited Create(nil);
  LoadTreeView;
end;

procedure TRefactorNodeTreeViewForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TRefactorNodeTreeViewForm.LoadTreeView;

  procedure AddNode(AParentNode: TTreeNode; ANode: TcwNode);
  var
    Index: Integer;
    ATreeNode: TTreeNode;
    ACaption: string;
    ATokenNode: TcwTokenNode;
  begin
    if ANode is TcwTokenNode then
    begin
      ATokenNode := ANode as TcwTokenNode;
      ACaption := GetEnumName(TypeInfo(TptTokenKind), Ord(ATokenNode.TokenKind));
    end else
      ACaption := ANode.ClassName;
    ATreeNode := RefactorTreeView.Items.AddChildObject(AParentNode, ACaption, ANode);
    for Index := 0 to ANode.Count - 1 do
      AddNode(ATreeNode, ANode[Index]);
  end;

begin
  RefactorTreeView.Items.BeginUpdate;
  try
    RefactorTreeView.Items.Clear;
    AddNode(nil, FModuleNode);
  finally
    RefactorTreeView.Items.EndUpdate;
  end;
end;

var
  ARefactorNodeTreeViewRefactoring: IcwRefactoring;

procedure TRefactorNodeTreeViewForm.RefactorTreeViewChange(Sender: TObject; Node: TTreeNode);
var
  ANode: TcwNode;
  ACodeNode: TcwCodeNode;
begin
  ANode := TObject(Node.Data) as TcwNode;
  if ANode is TcwCodeNode then
    RefactorSynEdit.Lines.Text := (ANode as TcwCodeNode).Code
  else
    RefactorSynEdit.Lines.Clear;

  if ANode is TcwCodeNode then
  begin
    ACodeNode := ANode as TcwCodeNode;
    RefactorStatusBar.SimpleText := Format('%d:  %d', [ACodeNode.Point.Y, ACodeNode.Point.X]);
  end;
end;

procedure TRefactorNodeTreeViewForm.GenerateActionExecute(Sender: TObject);
var
  AFileName: string;
begin
  AFileName := '';
  if InputQuery('File', 'File', AFileName) then
    FModuleNode.Generate(AFileName, True);
end;

initialization

  ARefactorNodeTreeViewRefactoring := TRefactorNodeTreeViewRefactoring.Create;
  gcwRefactoringHost.RegisterRefactoring(ARefactorNodeTreeViewRefactoring);

finalization

  gcwRefactoringHost.UnregisterRefactoring(ARefactorNodeTreeViewRefactoring);

end.
