(*  Copyright (c) 2002, the CodeLens project
  All rights reserved.

  Redistribution and use in source and binary forms,
  with or without modification, are permitted provided
  that the following conditions are met:

  Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.

  Redistributions in binary form must reproduce the
  above copyright notice, this list of conditions
  and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
  Neither the name of the CodeLens project nor the names
  of its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS
  AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. *)

unit uWithStatementRefactoring;

interface

uses
  ucwClasses,
  ucwRefactoring,
  ucwRefactorNodes,
  ucwMessage;

type
  TAbstractAddWithVariableToLocalVarRefactoring = class(TcwRefactoring)
  private
    FDidRefactor: Boolean;
    FAddToUnit: TcwUnit;
  protected
    procedure AddLocalVarsCallback(const ANode: TcwNode);
    procedure DoAddLocalVars(const AUnit: TcwUnit; ABaseFolderName: string);
  end;

  TAddWithVariableToLocalVarUnitRefactoring = class(TAbstractAddWithVariableToLocalVarRefactoring)
  protected
    procedure Execute; override;
    function GetCanExecute: Boolean; override;
    function GetMenuCaption: string; override;
    function GetName: string; override;
  end;

  TAddWithVariableToLocalVarAllUnitsRefactoring = class(TAbstractAddWithVariableToLocalVarRefactoring)
  protected
    procedure Execute; override;
    function GetCanExecute: Boolean; override;
    function GetMenuCaption: string; override;
    function GetName: string; override;
  end;

  TAbstractQualifyWithStatementRefactoring = class(TcwRefactoring)
  private
    FDidRefactor: Boolean;
    FQualifyUnit: TcwUnit;
    FWithMembersOnly: Boolean;
  protected
    procedure QualifyWithStatementCallback(const ANode: TcwNode);
    procedure DoQualifyWithStatements(const AUnit: TcwUnit; ABaseFolderName: string;
      AWithMembersOnly: Boolean);
  end;

  TQualifyWithMembersUnitRefactoring = class(TAbstractQualifyWithStatementRefactoring)
  protected
    procedure Execute; override;
    function GetCanExecute: Boolean; override;
    function GetMenuCaption: string; override;
    function GetName: string; override;
  end;

  TQualifyWithMembersAllUnitsRefactoring = class(TAbstractQualifyWithStatementRefactoring)
  protected
    procedure Execute; override;
    function GetCanExecute: Boolean; override;
    function GetMenuCaption: string; override;
    function GetName: string; override;
  end;

  TQualifyAllUnitRefactoring = class(TAbstractQualifyWithStatementRefactoring)
  protected
    procedure Execute; override;
    function GetCanExecute: Boolean; override;
    function GetMenuCaption: string; override;
    function GetName: string; override;
  end;

  TQualifyAllAllUnitsRefactoring = class(TAbstractQualifyWithStatementRefactoring)
  protected
    procedure Execute; override;
    function GetCanExecute: Boolean; override;
    function GetMenuCaption: string; override;
    function GetName: string; override;
  end;

  TAbstractRemoveWithStatementRefactoring = class(TcwRefactoring)
  private
    FWithUnit: TcwUnit;
    FDidRefactor: Boolean;
  protected
    procedure DoRemoveWithStatementCallback(const ANode: TcwNode);
    procedure DoRemoveWithStatement(const AUnit: TcwUnit; ABaseFolderName: string);
  end;

  TRemoveWithStatementUnitRefactoring = class(TAbstractRemoveWithStatementRefactoring)
  protected
    procedure Execute; override;
    function GetCanExecute: Boolean; override;
    function GetMenuCaption: string; override;
    function GetName: string; override;
  end;

  TRemoveWithStatementAllUnitsRefactoring = class(TAbstractRemoveWithStatementRefactoring)
  protected
    procedure Execute; override;
    function GetCanExecute: Boolean; override;
    function GetMenuCaption: string; override;
    function GetName: string; override;
  end;

implementation

uses
  SysUtils,
  Types,
  mwPasLexTypes;

function InWithVariableList(ANode: TcwCodeNode): Boolean;
var
  ATempNode: TcwNode;
  AStatementNode: TcwStatementNode;
  Index: Integer;
begin
  Result := False;
  if Assigned(ANode) then
  begin
    // walk up the parentnodes looking for the first with statement...
    AStatementNode := nil;
    ATempNode := ANode;
    repeat
      if (ATempNode is TcwStatementNode) and
        ((ATempNode as TcwStatementNode).GetStatementKind = ptWith) then
        AStatementNode := ATempNode as TcwStatementNode;
      ATempNode := ATempNode.ParentNode;
    until not Assigned(ATempNode) or Assigned(AStatementNode);

    // if with statement found, go to each VariableReferenceNode, and see if ANode exists in
    // its child hierarchy...
    if Assigned(AStatementNode) then
      for Index := 0 to AStatementNode.Count - 1 do
        if AStatementNode.Nodes[Index] is TcwVariableReferenceNode then
        begin
          Result := AStatementNode.Nodes[Index].IsParentOf(ANode);
          if Result then
            Break;
        end;
  end;
end;

{ TAbstractAddWithVariableToLocalVarRefactoring }

procedure TAbstractAddWithVariableToLocalVarRefactoring.AddLocalVarsCallback(const ANode: TcwNode);

  procedure ProcessWithStatmentNode(AStatementNode: TcwStatementNode);

    procedure AddWithMemberAsLocalVariable(AWithStatementNode: TcwStatementNode;
      AUsedByLocation: TcwUsedByLocation; AVariableReferenceNode: TcwVariableReferenceNode;
      AType: TcwSymbol);
    var
      ABlockNode: TcwBlockNode;
      AWithJunkNode: TcwJunkNode;
      AWithIndex: Integer;
      AFound: Boolean;
      Index: Integer;
      AStatementNode: TcwStatementNode;
      AVarName: string;
      AVariableDeclarationListNode: TcwVariableDeclarationListNode;
    begin
      // see if the with member is a complicated expression, which requires a local var to be addded...
      AFound := False;
      for Index := 0 to AVariableReferenceNode.Count - 1 do
        if AVariableReferenceNode[Index] is TcwVariableReferenceNode then
        begin
          AFound := True;
          Break;
        end;
      if AFound then
      begin
        AVarName := AType.Name;
        Assert(AVarName <> '');
        if AVarName[1] = 'T' then
          AVarName := 'A' + Copy(AVarName, 2, Length(AVarName))
        else
          AVarName := 'A' + AVarName;

        AWithIndex := AWithStatementNode.GetIndex;
        if (AWithIndex > 0) and (AWithStatementNode.ParentNode[AWithIndex - 1] is TcwJunkNode) then
          AWithJunkNode := AWithStatementNode.ParentNode[AWithIndex - 1] as TcwJunkNode
        else
          AWithJunkNode := nil;

        AStatementNode := TcwStatementNode.CreateCode(AWithStatementNode.ParentNode,
          Format('%s := %s;', [AVarName, AVariableReferenceNode.GetCode(False)]), AWithIndex);

        AVariableReferenceNode.Code := AVarName;
        if AVariableReferenceNode[AVariableReferenceNode.Count - 1] is TcwJunkNode then
          AVariableReferenceNode.Code := AVariableReferenceNode.Code +
            (AVariableReferenceNode[AVariableReferenceNode.Count - 1] as TcwJunkNode).Code;

        if Assigned(AWithJunkNode) then
          TcwJunkNode.CreateCode(AWithStatementNode.ParentNode, AWithJunkNode.Code, AWithIndex + 1);

        ABlockNode := AStatementNode.GetBlockNode;
        Assert(Assigned(ABlockNode));
        AVariableDeclarationListNode := ABlockNode.GetInnerMostVariableSection;
        if not Assigned(AVariableDeclarationListNode) then
          AVariableDeclarationListNode := ABlockNode.AddVariableSectionCode;
        AVariableDeclarationListNode.AddVariableCode(AVarName, AType.Name);

        FDidRefactor := True;
      end;
    end;

  var
    AType: TcwSymbol;
    AContainer: TcwSymbolContainer;
    AAsOperator: Boolean;
    AUsedByLocation: TcwUsedByLocation;
    ASymbol: TcwSymbol;
    ACodeLocation: TcwCodeLocation;
    Index: Integer;
    AIdentifierNode: TcwIdentifierNode;
    AVariableReferenceNode: TcwVariableReferenceNode;
  begin
    for Index := AStatementNode.Count - 1 downto 0 do
    begin
      if AStatementNode.Nodes[Index] is TcwVariableReferenceNode then
      begin
        AVariableReferenceNode := AStatementNode.Nodes[Index] as TcwVariableReferenceNode;
        AIdentifierNode := AVariableReferenceNode.GetContainerNode(AAsOperator);
        if Assigned(AIdentifierNode) then
        begin
          ACodeLocation := FindCodeLocationForNode(FAddToUnit, AIdentifierNode);
          if Assigned(ACodeLocation) and
            (ACodeLocation is TcwUsedByLocation) then
          begin
            AUsedByLocation := ACodeLocation as TcwUsedByLocation;
            ASymbol := AUsedByLocation.UsedByList.Symbol;

            if (ASymbol is TcwRoutine) and
              ((ASymbol as TcwRoutine).RoutineKind in [rkFunction, rkConstructor]) then
            begin
              gcwRefactoringHost.AcceptRefactoringMessage(Self, TcwMessage.Create(
                Format('Symbol, %s, is a constructor or a function', [ASymbol.Name]),
                FAddToUnit.FileIndex, AUsedByLocation.X, AUsedByLocation.Y, False));
              Break;
            end else
            begin
              AType := ASymbol.TypeIntf;
              AContainer := ASymbol.FindContainer;
              if (AContainer is TcwClass) or (AContainer is TcwRecord) then
                AddWithMemberAsLocalVariable(AStatementNode, AUsedByLocation,
                  AVariableReferenceNode, AType)
              else
              begin
                gcwRefactoringHost.AcceptRefactoringMessage(Self, TcwMessage.Create(
                  Format('Type, %s is neither a class nor a record', [ASymbol.Name]),
                  FAddToUnit.FileIndex, AUsedByLocation.X, AUsedByLocation.Y, False));
                Break;
              end;
            end;
          end else
          begin
            gcwRefactoringHost.AcceptRefactoringMessage(Self, TcwMessage.Create(
              Format('Identifier, %s has no location in CodeLens', [AIdentifierNode.Code]),
              FAddToUnit.FileIndex, AIdentifierNode.Point.X, AIdentifierNode.Point.Y, False));
            Break;
          end;
        end else
        begin
          gcwRefactoringHost.AcceptRefactoringMessage(Self, TcwMessage.Create(
            Format('Member, %s, in with statement not found?', [AVariableReferenceNode.Code]),
            FAddToUnit.FileIndex, AVariableReferenceNode.Point.X, AVariableReferenceNode.Point.Y,
              False));
          Break;
        end;
      end;
    end;
  end;

var
  AStatementNode: TcwStatementNode;
begin
  if (ANode is TcwStatementNode) then
  begin
    AStatementNode := ANode as TcwStatementNode;
    if AStatementNode.GetStatementKind = ptWith then
      ProcessWithStatmentNode(AStatementNode);
  end;
end;

procedure TAbstractAddWithVariableToLocalVarRefactoring.DoAddLocalVars(const AUnit: TcwUnit;
  ABaseFolderName: string);
begin
  FAddToUnit := AUnit;
  FDidRefactor := False;
  BeginRefactoring(ABaseFolderName, FAddToUnit);
  try
    ModuleNode[FAddToUnit].IterateNodes(True, AddLocalVarsCallback);
  finally
    if FDidRefactor then
      EndRefactoring(ABaseFolderName, FAddToUnit)
    else
      CancelRefactoring(FAddToUnit);
  end;
end;

{ TAddWithVariableToLocalVarUnitRefactoring }

procedure TAddWithVariableToLocalVarUnitRefactoring.Execute;
var
  ABaseFolderName: string;
begin
  ABaseFolderName := GetBaseFolderName;
  DoAddLocalVars(gcwRefactoringHost.SelectedUnit, ABaseFolderName);
  ViewRefactoring(ABaseFolderName);
end;

function TAddWithVariableToLocalVarUnitRefactoring.GetCanExecute: Boolean;
begin
  Result := Assigned(gcwRefactoringHost.SelectedUnit);
end;

function TAddWithVariableToLocalVarUnitRefactoring.GetMenuCaption: string;
begin
  Result := 'Selected Unit';
end;

function TAddWithVariableToLocalVarUnitRefactoring.GetName: string;
begin
  Result := 'AddWithLocalVarUnit';
end;

{ TAddWithVariableToLocalVarAllUnitsRefactoring }

procedure TAddWithVariableToLocalVarAllUnitsRefactoring.Execute;
var
  ABaseFolderName: string;
  Index: Integer;
begin
  ABaseFolderName := GetBaseFolderName;
  for Index := 0 to gcwProject.AllUnits.Count - 1 do
    DoAddLocalVars(gcwProject.AllUnits[Index], ABaseFolderName);
  ViewRefactoring(ABaseFolderName);
end;

function TAddWithVariableToLocalVarAllUnitsRefactoring.GetCanExecute: Boolean;
begin
  Result := True;
end;

function TAddWithVariableToLocalVarAllUnitsRefactoring.GetMenuCaption: string;
begin
  Result := 'All Units';
end;

function TAddWithVariableToLocalVarAllUnitsRefactoring.GetName: string;
begin
  Result := 'AddWithLocalVarAllUnits';
end;

{ TAbstractQualifyWithStatementRefactoring }

procedure TAbstractQualifyWithStatementRefactoring.QualifyWithStatementCallback(
  const ANode: TcwNode);

  procedure ProcessWithStatmentNode(AStatementNode: TcwStatementNode);

    procedure ProcessSymbol(AStatementNode: TcwStatementNode; ASymbolContainer: TcwSymbolContainer;
      ASymbol: TcwSymbol; AVariableReferenceNode: TcwVariableReferenceNode; AAsOperator: Boolean);
    var
      ATokenNode: TcwTokenNode;
      AIdentifierNode: TcwIdentifierNode;
      AUsedByIndex: Integer;
      AUsedByLocation: TcwUsedByLocation;
      ANode: TcwCodeNode;
      ACode: string;
    begin
      for AUsedByIndex := 0 to ASymbol.UsedBy.Count - 1 do
      begin
        AUsedByLocation := ASymbol.UsedBy[AUsedByIndex];
        if AUsedByLocation.UnitIntf.FullFileName = FQualifyUnit.FullFileName then
        begin
          ANode := FindNodeForLocation(AStatementNode, AUsedByLocation);
          if Assigned(ANode) and ANode.ParentedBy(AStatementNode) then
          begin
            if not FWithMembersOnly or InWithVariableList(ANode) then
            begin
              Assert(ANode is TcwIdentifierNode);
              AIdentifierNode := ANode as TcwIdentifierNode;
              if not AIdentifierNode.IsQualified and not ANode.CodeModified then
              begin
                FDidRefactor := True;
                ACode := AVariableReferenceNode.GetCode(False);
                if AAsOperator then
                begin
                  ATokenNode := AVariableReferenceNode.GetFirstTokenNode;
                  if (ATokenNode.GetIndex = 0) or
                    (ATokenNode.TokenKind <> ptRoundOpen) then
                    ACode := '(' + ACode + ')';
                end;
                ANode.Code := ACode + '.' + ANode.Code;
              end;
            end;
          end;
        end;
      end;
    end;

    procedure ProcessClass(AStatementNode: TcwStatementNode;
      AClass: TcwClass;
      AVariableReferenceNode: TcwVariableReferenceNode;
      FWithMembersOnly: Boolean; AAsOperator: Boolean; out AMembersFound: Boolean);

      procedure DoProcessClassSymbol(ASymbol: TcwSymbol);
      begin
        ProcessSymbol(AStatementNode, AClass, ASymbol, AVariableReferenceNode, AAsOperator);
        AMembersFound := True;
      end;

    var
      AAncestorMembersFound: Boolean;
      Index: Integer;
    begin
      AMembersFound := False;
      for Index := 0 to AClass.Fields.Count - 1 do
        DoProcessClassSymbol(AClass.Fields[Index]);
      for Index := 0 to AClass.Properties.Count - 1 do
        DoProcessClassSymbol(AClass.Properties[Index]);
      for Index := 0 to AClass.Routines.Count - 1 do
        DoProcessClassSymbol(AClass.Routines[Index]);
      AAncestorMembersFound := False;
      if Assigned(AClass.InheritsFromClass) then
        ProcessClass(AStatementNode, AClass.InheritsFromClass, AVariableReferenceNode,
          FWithMembersOnly, AAsOperator, AAncestorMembersFound);
      AMembersFound := AMembersFound or AAncestorMembersFound;
    end;

    procedure ProcessRecord(AStatementNode: TcwStatementNode;
      ARecord: TcwRecord;
      AVariableReferenceNode: TcwVariableReferenceNode;
      FWithMembersOnly: Boolean; AAsOperator: Boolean; out AMembersFound: Boolean);
    var
      Index: Integer;
    begin
      AMembersFound := False;
      for Index := 0 to ARecord.Fields.Count - 1 do
      begin
        ProcessSymbol(AStatementNode, ARecord, ARecord.Fields[Index], AVariableReferenceNode,
          AAsOperator);
        AMembersFound := True;
      end;
    end;

  var
    AContainer: TcwSymbolContainer;
    AAsOperator: Boolean;
    AUsedByLocation: TcwUsedByLocation;
    ASymbol: TcwSymbol;
    ACodeLocation: TcwCodeLocation;
    Index: Integer;
    AIdentifierNode: TcwIdentifierNode;
    AVariableReferenceNode: TcwVariableReferenceNode;
    AMembersFound: Boolean;
  begin
    for Index := AStatementNode.Count - 1 downto 0 do
    begin
      if AStatementNode.Nodes[Index] is TcwVariableReferenceNode then
      begin
        AVariableReferenceNode := AStatementNode.Nodes[Index] as TcwVariableReferenceNode;
        AIdentifierNode := AVariableReferenceNode.GetContainerNode(AAsOperator);
        if Assigned(AIdentifierNode) then
        begin
          ACodeLocation := FindCodeLocationForNode(FQualifyUnit, AIdentifierNode);
          if Assigned(ACodeLocation) and
            (ACodeLocation is TcwUsedByLocation) then
          begin
            AUsedByLocation := ACodeLocation as TcwUsedByLocation;
            ASymbol := AUsedByLocation.UsedByList.Symbol;

            if (ASymbol is TcwRoutine) and ((ASymbol as TcwRoutine).RoutineKind in
              [rkFunction, rkConstructor]) then
            begin
              gcwRefactoringHost.AcceptRefactoringMessage(Self, TcwMessage.Create(
                Format('Symbol, %s, is a constructor or a function', [ASymbol.Name]),
                FQualifyUnit.FileIndex, AUsedByLocation.X, AUsedByLocation.Y, False));
              Break;
            end else
            begin
              AContainer := ASymbol.FindContainer;
              if (AContainer is TcwClass) or (AContainer is TcwRecord) then
              begin
                AMembersFound := False;

                if AContainer is TcwClass then
                  ProcessClass(AStatementNode, AContainer as TcwClass, AVariableReferenceNode,
                    FWithMembersOnly, AAsOperator, AMembersFound)
                else
                  if AContainer is TcwRecord then
                    ProcessRecord(AStatementNode, AContainer as TcwRecord, AVariableReferenceNode,
                      FWithMembersOnly, AAsOperator, AMembersFound);

                if not AMembersFound then
                begin
                  gcwRefactoringHost.AcceptRefactoringMessage(Self, TcwMessage.Create(
                    Format('Symbol, %s, has no members', [AContainer.Name]),
                    FQualifyUnit.FileIndex, AUsedByLocation.X, AUsedByLocation.Y, False));
                  Break;
                end;
              end else
              begin
                gcwRefactoringHost.AcceptRefactoringMessage(Self, TcwMessage.Create(
                  Format('Type, %s is neither a class nor a record', [ASymbol.Name]),
                  FQualifyUnit.FileIndex, AUsedByLocation.X, AUsedByLocation.Y, False));
                Break;
              end;
            end;
          end else
          begin
            gcwRefactoringHost.AcceptRefactoringMessage(Self, TcwMessage.Create(
              Format('Identifier, %s has no location', [AIdentifierNode.Code]),
              FQualifyUnit.FileIndex, AIdentifierNode.Point.X, AIdentifierNode.Point.Y, False));
            Break;
          end;
        end else
        begin
          gcwRefactoringHost.AcceptRefactoringMessage(Self, TcwMessage.Create(
            Format('Member, %s, in with statement not found?', [AVariableReferenceNode.Code]),
            FQualifyUnit.FileIndex, AVariableReferenceNode.Point.X, AVariableReferenceNode.Point.Y,
              False));
          Break;
        end;
      end;
    end;
  end;

var
  AStatementNode: TcwStatementNode;
begin
  if (ANode is TcwStatementNode) then
  begin
    AStatementNode := ANode as TcwStatementNode;
    if AStatementNode.GetStatementKind = ptWith then
      ProcessWithStatmentNode(AStatementNode);
  end;
end;

procedure TAbstractQualifyWithStatementRefactoring.DoQualifyWithStatements(const AUnit: TcwUnit;
  ABaseFolderName: string; AWithMembersOnly: Boolean);
begin
  FDidRefactor := False;
  FQualifyUnit := AUnit;
  FWithMembersOnly := AWithMembersOnly;
  BeginRefactoring(ABaseFolderName, FQualifyUnit);
  try
    ModuleNode[FQualifyUnit].IterateNodes(True, QualifyWithStatementCallback);
  finally
    if FDidRefactor then
      EndRefactoring(ABaseFolderName, FQualifyUnit)
    else
      CancelRefactoring(FQualifyUnit);
  end;
end;

{ TQualifyWithMembersUnitRefactoring }

procedure TQualifyWithMembersUnitRefactoring.Execute;
var
  ABaseFolderName: string;
begin
  ABaseFolderName := GetBaseFolderName;
  DoQualifyWithStatements(gcwRefactoringHost.SelectedUnit, ABaseFolderName, True);
  ViewRefactoring(ABaseFolderName);
end;

function TQualifyWithMembersUnitRefactoring.GetCanExecute: Boolean;
begin
  Result := Assigned(gcwRefactoringHost.SelectedUnit);
end;

function TQualifyWithMembersUnitRefactoring.GetMenuCaption: string;
begin
  Result := 'Selected Unit';
end;

function TQualifyWithMembersUnitRefactoring.GetName: string;
begin
  Result := 'QualifyWithMembersUnit';
end;

{ TQualifyWithMembersAllUnitsRefactoring }

procedure TQualifyWithMembersAllUnitsRefactoring.Execute;
var
  ABaseFolderName: string;
  Index: Integer;
begin
  ABaseFolderName := GetBaseFolderName;
  for Index := 0 to gcwProject.AllUnits.Count - 1 do
    DoQualifyWithStatements(gcwProject.AllUnits[Index], ABaseFolderName, True);
  ViewRefactoring(ABaseFolderName);
end;

function TQualifyWithMembersAllUnitsRefactoring.GetCanExecute: Boolean;
begin
  Result := True;
end;

function TQualifyWithMembersAllUnitsRefactoring.GetMenuCaption: string;
begin
  Result := 'All Units';
end;

function TQualifyWithMembersAllUnitsRefactoring.GetName: string;
begin
  Result := 'QualifyWithMembersAllUnits';
end;

{ TQualifyAllUnitRefactoring }

procedure TQualifyAllUnitRefactoring.Execute;
var
  ABaseFolderName: string;
begin
  ABaseFolderName := GetBaseFolderName;
  DoQualifyWithStatements(gcwRefactoringHost.SelectedUnit, ABaseFolderName, False);
  ViewRefactoring(ABaseFolderName);
end;

function TQualifyAllUnitRefactoring.GetCanExecute: Boolean;
begin
  Result := Assigned(gcwRefactoringHost.SelectedUnit);
end;

function TQualifyAllUnitRefactoring.GetMenuCaption: string;
begin
  Result := 'Selected Unit';
end;

function TQualifyAllUnitRefactoring.GetName: string;
begin
  Result := 'QualifyAllUnit';
end;

{ TQualifyAllAllUnitsRefactoring }

procedure TQualifyAllAllUnitsRefactoring.Execute;
var
  ABaseFolderName: string;
  Index: Integer;
begin
  ABaseFolderName := GetBaseFolderName;
  for Index := 0 to gcwProject.AllUnits.Count - 1 do
    DoQualifyWithStatements(gcwProject.AllUnits[Index], ABaseFolderName, False);
  ViewRefactoring(ABaseFolderName);
end;

function TQualifyAllAllUnitsRefactoring.GetCanExecute: Boolean;
begin
  Result := True;
end;

function TQualifyAllAllUnitsRefactoring.GetMenuCaption: string;
begin
  Result := 'All Units';
end;

function TQualifyAllAllUnitsRefactoring.GetName: string;
begin
  Result := 'QualifyAllAllUnits';
end;

{ TAbstractRemoveWithStatementRefactoring }

procedure TAbstractRemoveWithStatementRefactoring.DoRemoveWithStatementCallback(
  const ANode: TcwNode);

  procedure DeleteWithStatement(var AStatementNode: TcwStatementNode);

    procedure DecreaseIndent(ANode: TcwNode);
    var
      ACode: string;
      AJunkNode: TcwJunkNode;
      Index: Integer;
    begin
      for Index := 0 to ANode.Count - 1 do
        if ANode[Index] is TcwJunkNode then
        begin
          AJunkNode := ANode[Index] as TcwJunkNode;
          ACode := AJunkNode.Code;
          if (Pos(#13#10, ACode) > 0) and (Copy(ACode, Length(ACode) - 1, 2) = '  ') then
          begin
            SetLength(ACode, Length(ACode) - 2); // decrease indent
            AJunkNode.Code := ACode;
          end;
        end;
      for Index := 0 to ANode.Count - 1 do
        // Recurse!
        DecreaseIndent(ANode[Index]);
    end;

  var
    AChildNode: TcwCodeNode;
    AChildIndex: Integer;
    AChildStatementNode: TcwStatementNode;
    ATokenNode: TcwTokenNode;
    AIndex: Integer;
  begin
    FDidRefactor := True;

    Assert(AStatementNode.Count > 0);

    DecreaseIndent(AStatementNode);

    if AStatementNode[AStatementNode.Count - 1] is TcwStatementNode then
    begin
      AChildStatementNode := AStatementNode[AStatementNode.Count - 1] as TcwStatementNode;
      AIndex := AChildStatementNode.GetIndex;
      // junk before this statement might be a comment, so preserve that too...
      if (AIndex > 0) and (AStatementNode[AIndex - 1] is TcwJunkNode) and
        not (AStatementNode[AIndex - 1] as TcwJunkNode).IsWhiteSpaceOnly then
        AStatementNode[AIndex - 1].SetParentNode(AStatementNode.ParentNode,
          AStatementNode.GetIndex);
      AChildStatementNode.SetParentNode(AStatementNode.ParentNode, AStatementNode.GetIndex);
      FreeAndNil(AStatementNode);
      ATokenNode := AChildStatementNode.GetFirstTokenNode;
      // does with-statement have begin-end pair?
      if Assigned(ATokenNode) and (ATokenNode.TokenKind = ptBegin) then
      begin
        AChildIndex := AChildStatementNode.GetIndex;
        Assert(AChildIndex >= 0);
        if AChildIndex < AChildStatementNode.ParentNode.Count - 1 then
        begin
          if (AChildStatementNode.ParentNode[AChildIndex + 1] is TcwTokenNode) and
            ((AChildStatementNode.ParentNode[AChildIndex + 1] as TcwTokenNode).
             TokenKind = ptSemiColon) then
            AChildStatementNode.ParentNode[AChildIndex + 1].Free;
          AIndex := ATokenNode.GetIndex;
          Inc(AIndex);
          // dont move the junk after the 'begin' node if its only whitespace...
          if (AChildStatementNode[AIndex] is TcwJunkNode) and
            (AChildStatementNode[AIndex] as TcwJunkNode).IsWhiteSpaceOnly then
            Inc(AIndex);
          if AIndex < AChildStatementNode.Count - 1 then
          begin
            while (AIndex <= AChildStatementNode.Count - 1) do
            begin
              AChildNode := AChildStatementNode[AIndex] as TcwCodeNode;
              // stop moving nodes if we've reached the junk before the end node, and its whitespace only...
              if (AChildNode is TcwJunkNode) and
                (AChildNode as TcwJunkNode).IsWhiteSpaceOnly and
                (AIndex < AChildStatementNode.Count - 1) and
                (AChildNode.ParentNode[AIndex + 1] is TcwTokenNode) and
                ((AChildNode.ParentNode[AIndex + 1] as TcwTokenNode).TokenKind = ptEnd) then
                Break;
              // stop moving nodes if we've reached the 'end' node
              if (AChildNode is TcwTokenNode) and
                ((AChildNode as TcwTokenNode).TokenKind = ptEnd) then
                Break;
              AChildNode.SetParentNode(AChildStatementNode.ParentNode, AChildIndex);
              Inc(AChildIndex);
            end;
            AChildStatementNode.Free; // free statement node and ptBegin/ptEnd child nodes
          end;
        end;
      end;
    end else
      gcwRefactoringHost.AcceptRefactoringMessage(Self, TcwMessage.Create(
        'With statement has no child statements', FWithUnit.FileIndex, AStatementNode.Point.X,
          AStatementNode.Point.Y, False));
  end;

var
  AStatementNode: TcwStatementNode;
begin
  if (ANode is TcwStatementNode) then
  begin
    AStatementNode := ANode as TcwStatementNode;
    if AStatementNode.GetStatementKind = ptWith then
      DeleteWithStatement(AStatementNode);
  end;
end;

procedure TAbstractRemoveWithStatementRefactoring.DoRemoveWithStatement(const AUnit: TcwUnit;
  ABaseFolderName: string);
begin
  FDidRefactor := False;
  FWithUnit := AUnit;
  BeginRefactoring(ABaseFolderName, AUnit);
  try
    ModuleNode[FWithUnit].IterateNodes(True, DoRemoveWithStatementCallback);
  finally
    if FDidRefactor then
      EndRefactoring(ABaseFolderName, FWithUnit)
    else
      CancelRefactoring(FWithUnit);
  end;
end;

{ TRemoveWithStatementUnitRefactoring }

procedure TRemoveWithStatementUnitRefactoring.Execute;
var
  ABaseFolderName: string;
begin
  ABaseFolderName := GetBaseFolderName;
  DoRemoveWithStatement(gcwRefactoringHost.SelectedUnit, ABaseFolderName);
  ViewRefactoring(ABaseFolderName);
end;

function TRemoveWithStatementUnitRefactoring.GetCanExecute: Boolean;
begin
  Result := Assigned(gcwRefactoringHost.SelectedUnit);
end;

function TRemoveWithStatementUnitRefactoring.GetMenuCaption: string;
begin
  Result := 'Selected Unit';
end;

function TRemoveWithStatementUnitRefactoring.GetName: string;
begin
  Result := 'RemoveWithStatementUnit';
end;

{ TRemoveWithStatementAllUnitsRefactoring }

procedure TRemoveWithStatementAllUnitsRefactoring.Execute;
var
  Index: Integer;
  ABaseFolderName: string;
begin
  ABaseFolderName := GetBaseFolderName;
  for Index := 0 to gcwProject.AllUnits.Count - 1 do
    DoRemoveWithStatement(gcwProject.AllUnits[Index], ABaseFolderName);
  ViewRefactoring(ABaseFolderName);
end;

function TRemoveWithStatementAllUnitsRefactoring.GetCanExecute: Boolean;
begin
  Result := True;
end;

function TRemoveWithStatementAllUnitsRefactoring.GetMenuCaption: string;
begin
  Result := 'All Units';
end;

function TRemoveWithStatementAllUnitsRefactoring.GetName: string;
begin
  Result := 'RemoveWithStatementAllUnits';
end;

var
  AAddWithVariableToLocalVarUnitRefactoring: TcwRefactoring;
  AAddWithVariableToLocalVarAllUnitsRefactoring: TcwRefactoring;
  AQualifyWithMembersUnitRefactoring: TcwRefactoring;
  AQualifyWithMembersAllUnitsRefactoring: TcwRefactoring;
  AQualifyAllUnitRefactoring: TcwRefactoring;
  AQualifyAllAllUnitsRefactoring: TcwRefactoring;
  ARemoveWithStatementUnitRefactoring: TcwRefactoring;
  ARemoveWithStatementAllUnitsRefactoring: TcwRefactoring;

initialization

  AAddWithVariableToLocalVarUnitRefactoring := TAddWithVariableToLocalVarUnitRefactoring.Create;
  gcwRefactoringHost.AddGroupName('WithStatement', 'With Statement');
  gcwRefactoringHost.AddGroupName('AddLocalVars', 'Add Local Vars', 'WithStatement');
  gcwRefactoringHost.RegisterRefactoring(AAddWithVariableToLocalVarUnitRefactoring, 'AddLocalVars');
  AAddWithVariableToLocalVarAllUnitsRefactoring := TAddWithVariableToLocalVarAllUnitsRefactoring.
    Create;
  gcwRefactoringHost.RegisterRefactoring(AAddWithVariableToLocalVarAllUnitsRefactoring,
    'AddLocalVars');

  gcwRefactoringHost.AddGroupName('WithStatementQualify', 'Qualify', 'WithStatement');
  gcwRefactoringHost.AddGroupName('QualifyWithMembers', 'With Members', 'WithStatementQualify');
  AQualifyWithMembersUnitRefactoring := TQualifyWithMembersUnitRefactoring.Create;
  gcwRefactoringHost.RegisterRefactoring(AQualifyWithMembersUnitRefactoring, 'QualifyWithMembers');

  AQualifyWithMembersAllUnitsRefactoring := TQualifyWithMembersAllUnitsRefactoring.Create;
  gcwRefactoringHost.RegisterRefactoring(AQualifyWithMembersAllUnitsRefactoring, 'QualifyWithMembers');

  gcwRefactoringHost.AddGroupName('QualifyAll', 'All', 'WithStatementQualify');
  AQualifyAllUnitRefactoring := TQualifyAllUnitRefactoring.Create;
  gcwRefactoringHost.RegisterRefactoring(AQualifyAllUnitRefactoring, 'QualifyAll');
  AQualifyAllAllUnitsRefactoring := TQualifyAllAllUnitsRefactoring.Create;
  gcwRefactoringHost.RegisterRefactoring(AQualifyAllAllUnitsRefactoring, 'QualifyAll');

  gcwRefactoringHost.AddGroupName('RemoveWithStatement', 'Remove', 'WithStatement');
  ARemoveWithStatementUnitRefactoring := TRemoveWithStatementUnitRefactoring.Create;
  gcwRefactoringHost.RegisterRefactoring(ARemoveWithStatementUnitRefactoring, 'RemoveWithStatement');
  ARemoveWithStatementAllUnitsRefactoring := TRemoveWithStatementAllUnitsRefactoring.Create;
  gcwRefactoringHost.RegisterRefactoring(ARemoveWithStatementAllUnitsRefactoring, 'RemoveWithStatement');

finalization

  gcwRefactoringHost.UnregisterRefactoring(AAddWithVariableToLocalVarUnitRefactoring);
  gcwRefactoringHost.UnregisterRefactoring(AAddWithVariableToLocalVarAllUnitsRefactoring);
  gcwRefactoringHost.UnregisterRefactoring(AQualifyWithMembersUnitRefactoring);
  gcwRefactoringHost.UnregisterRefactoring(AQualifyWithMembersAllUnitsRefactoring);
  gcwRefactoringHost.UnregisterRefactoring(AQualifyAllUnitRefactoring);
  gcwRefactoringHost.UnregisterRefactoring(AQualifyAllAllUnitsRefactoring);
  gcwRefactoringHost.UnregisterRefactoring(ARemoveWithStatementUnitRefactoring);
  gcwRefactoringHost.UnregisterRefactoring(ARemoveWithStatementAllUnitsRefactoring);

end.
