(*  Copyright (c) 2002, the CodeLens project
  All rights reserved.

  Redistribution and use in source and binary forms,
  with or without modification, are permitted provided
  that the following conditions are met:

  Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.

  Redistributions in binary form must reproduce the
  above copyright notice, this list of conditions
  and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
  Neither the name of the CodeLens project nor the names
  of its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS
  AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. *)

program CWrench;

{$APPTYPE CONSOLE}

uses
  SysUtils,
  ucwClasses,
  ucwFileSettings,
  ucwProject,
  ucwStringList,
  ucwBuiltInReports in 'ucwBuiltInReports.pas';

type
  TProgressIndicator = class(TInterfacedObject, IcwProgressNotifier)
    //if the notifier wants to cancel the parsing, it can set the Cancel flah
    procedure ShowProgress(const FullFileName: string; ParseState: TParserState; var CancelParsing: Boolean);
  end;

procedure TProgressIndicator.ShowProgress(const FullFileName: string; ParseState: TParserState; var
  CancelParsing: Boolean);
const
  asPARSERSTATES: array[TParserState] of string = ('', 'Interface', '', 'Implementation');
begin
  WriteLn(FullFileName + ' ' + asPARSERSTATES[ParseState]);
end;

var
  vProjectName: string;
  VisibleReports: IcwStringList;
  ProgressIndicator: IcwProgressNotifier;
  i: Integer;
  vReport: IcwReport;
begin
  vProjectName := paramStr(1);

  if not FileExists(vProjectName) then
  begin
    WriteLn('usage cwrench [projectfilename] : could not find : ' + vProjectName);
    Halt(1);
  end;
  gcwCodeWrenchOptions := TcwFileSettings.Create(ChangeFileExt(paramStr(0), '.ini'));
  gcwProject := TcwProject.Create;
  try
    ProgressIndicator := TProgressIndicator.Create;
    gcwProject.AddProgressNotifier(ProgressIndicator);
    AddBuiltInReports;

    gcwProject.CWOptions.FileToAnalyze := vProjectName;

    gcwProject.Run;
    VisibleReports := TcwStringList.Create;
    gcwProject.ProjectOptions.ReadSection('VisibleReports', VisibleReports);
    for i := 0 to VisibleReports.Count - 1 do
    begin
      vReport := gcwProject.Reports.GetReportByName(VisibleReports[i]);
      if vReport <> nil then
        vReport.Execute;
    end;
  finally
    gcwProject := nil;
    gcwCodeWrenchOptions := nil;
  end;

end.





