unit uCodeWrenchTests;

interface

uses
  SysUtils, TestFrameWork;

type
  TCodeLensTestCase = class(TTestCase)
  protected
    function GetModuleFullName: string; virtual; abstract;
    procedure SetUp; override;
    procedure TearDown; override;
  published
    //procedure TestProjectSetup;
    procedure TestWarnings;
  end;

  TArrayPointerTest = class(TCodeLensTestCase)
  protected
    function GetModuleFullName: string; override;
  published
    procedure TestArrayPointer;
  end;

  TPropertyVisibilityOverrideTest = class(TCodeLensTestCase)
  protected
    function GetModuleFullName: String; override;
  published
    procedure TestPropertyVisibilityOverride;
  end;

  TAsOperatorTest = class(TCodeLensTestCase)
  protected
    function GetModuleFullName: String; override;
  published
    procedure TestAsOperator;
  end;

  TTypeCastTest = class(TCodeLensTestCase)
  protected
    function GetModuleFullName: String; override;
  published
    procedure TestTypeCast;
  end;

  TDefaultPropertyTest = class(TCodeLensTestCase)
  protected
    function GetModuleFullName: String; override;
  published
    procedure TestDefaultProperty;
  end;

  TDefaultPropertyOverrideTest = class(TCodeLensTestCase)
  protected
    function GetModuleFullName: String; override;
  published
    procedure TestDefaultPropertyOverride;
  end;

  TUntypedParameterTest = class(TCodeLensTestCase)
  protected
    function GetModuleFullName: String; override;
  published
    procedure TestUntypedParameter;
  end;

  TFileOfTypeTest = class(TCodeLensTestCase)
  protected
    function GetModuleFullName: String; override;
  published
    procedure TestFileOfType;
  end;

  TQualifiedTypeDeclarationTest = class(TCodeLensTestCase)
  protected
    function GetModuleFullName: String; override;
  published
    procedure TestQualifiedTypeDeclaration;
  end;

  TVariantRecordTest = class(TCodeLensTestCase)
  protected
    function GetModuleFullName: String; override;
  published
    procedure TestVariantRecord;
  end;

  TForwardClassOfTest = class(TCodeLensTestCase)
  protected
    function GetModuleFullName: String; override;
  published
    procedure TestForwardClassOf;
  end;

  TCallingConventionTest = class(TCodeLensTestCase)
  protected
    function GetModuleFullName: String; override;
  published
    procedure TestCallingConvention;
  end;

  TExternalDirectiveTest = class(TCodeLensTestCase)
  protected
    function GetModuleFullName: String; override;
  published
    // merely testing for warnings
    //procedure TestExternalDirective;
  end;

  TCreateInWithTest = class(TCodeLensTestCase)
  protected
    function GetModuleFullName: String; override;
  published
    procedure TestCreateInWith;
  end;

  TProtectedVisibilityTest = class(TCodeLensTestCase)
  protected
    function GetModuleFullName: String; override;
  published
    procedure TestProtectedVisibility;
  end;

  TTypeTest001 = class(TCodeLensTestCase)
  protected
    function GetModuleFullName: String; override;
  published
    procedure TestTypeDeclaration;
  end;

implementation

uses
  ucwProject,
  ucwClasses;

{ TCodeLensTestCase }

procedure TCodeLensTestCase.SetUp;
begin
  inherited;
  gcwProject := TcwProject.Create;
  gcwProject.CWOptions.FileToAnalyze := GetModuleFullName;
  gcwProject.Run;
end;

procedure TCodeLensTestCase.TearDown;
var
  Index: Integer;
begin
  inherited;
  for Index := 0 to gcwProject.Messages.Count - 1 do
    Status(gcwProject.Messages[Index].MessageText);
  gcwProject := nil;
end;

//procedure TCodeLensTestCase.TestProjectSetup;
//begin
//  Check(Assigned(gcwProject));
//
//  // project, system, sysinit, and codewrenchsystem...
//  Check(gcwProject.AllUnits.Count >= 4);
//
//  Check(Assigned(gcwProject.AllUnits[0]));
//end;

procedure TCodeLensTestCase.TestWarnings;
begin
  Check(gcwProject.Messages.Count = 0);
end;

{ TArrayPointerTest }

function TArrayPointerTest.GetModuleFullName: string;
begin
  Result := '.\ArrayPointerTest.dpr';
end;

procedure TArrayPointerTest.TestArrayPointer;
var
  ARecord: TcwRecord;
  AVariable: TcwVariable;
begin
  Check(gcwProject.AllUnits[0].Types.Find('TTestArrayElement') is TcwRecord);
  ARecord := gcwProject.AllUnits[0].Types.Find('TTestArrayElement') as TcwRecord;
  AVariable := ARecord.Fields.Find('X');
  Check(Assigned(AVariable));
  Check(Assigned(AVariable.TypeIntf));
  Check(AVariable.UsedBy.Count = 2);
end;

{ TPropertyVisibilityOverrideTest }

function TPropertyVisibilityOverrideTest.GetModuleFullName: String;
begin
  Result := '.\PropertyVisibilityOverrideTest.dpr';
end;

procedure TPropertyVisibilityOverrideTest.TestPropertyVisibilityOverride;
var
  AClass: TcwClass;
  AProperty: TcwProperty;
begin
  AClass := gcwProject.AllUnits[0].Classes.Find('TMyClass');
  Check(Assigned(AClass));
  AProperty := AClass.FindSymbol(gcwProject.AllUnits[0], AClass,
    'Data', False, UsedByInfo(-1, False, -1, -1)) as TcwProperty;
  Check(Assigned(AProperty));
  Check(Assigned(AProperty.TypeIntf));
  Check(aProperty.UsedBy.Count = 1);
end;

{ TAsOperatorTest }

function TAsOperatorTest.GetModuleFullName: String;
begin
  Result := '.\AsOperatorTest.dpr';
end;

procedure TAsOperatorTest.TestAsOperator;
var
  AField: TcwVariable;
  AClass: TcwClass;
begin
  AClass := gcwProject.AllUnits[0].Classes.Find('TMyClass');
  Check(Assigned(AClass));
  AField := AClass.Fields.Find('Data');
  Check(Assigned(AField));
  Check(Assigned(AField.TypeIntf));
  Check(AField.UsedBy.Count = 1);
end;

{ TTypeCastTest }

function TTypeCastTest.GetModuleFullName: String;
begin
  Result := '.\TypeCastTest.dpr';
end;

procedure TTypeCastTest.TestTypeCast;
var
  AField: TcwVariable;
  AClass: TcwClass;
begin
  AClass := gcwProject.AllUnits[0].Classes.Find('TMyClass');
  Check(Assigned(AClass));
  AField := AClass.Fields.Find('X');
  Check(Assigned(AField));
  Check(Assigned(AField.TypeIntf));
  Check(AField.UsedBy.Count = 1);
end;

{ TDefaultPropertyTest }

function TDefaultPropertyTest.GetModuleFullName: String;
begin
  Result := '.\DefaultPropertyTest.dpr';
end;

procedure TDefaultPropertyTest.TestDefaultProperty;
var
  AField: TcwVariable;
  AClass: TcwClass;
  AInterface: TcwInterface;
begin
  AClass := gcwProject.AllUnits[0].Classes.Find('TDataClass');
  Check(Assigned(AClass));
  AField := AClass.Fields.Find('Data');
  Check(Assigned(AField));
  Check(Assigned(AField.TypeIntf));
  Check(AField.UsedBy.Count = 1);
  AInterface := gcwProject.AllUnits[0].Interfaces.Find('IDataInterface');
  Check(Assigned(AInterface));
  AField := AInterface.Properties.Find('Data');
  Check(Assigned(AField));
  Check(Assigned(AField.TypeIntf));
  Check(AField.UsedBy.Count = 2);
end;

{ TDefaultPropertyOverrideTest }

function TDefaultPropertyOverrideTest.GetModuleFullName: String;
begin
  Result := '.\DefaultPropertyOverrideTest.dpr';
end;

procedure TDefaultPropertyOverrideTest.TestDefaultPropertyOverride;
var
  AField: TcwVariable;
  AInterface: TcwInterface;
begin
  AInterface := gcwProject.AllUnits[0].Interfaces.Find('IData');
  Check(Assigned(AInterface));
  AField := AInterface.Properties.Find('Data');
  Check(Assigned(AField));
  Check(Assigned(AField.TypeIntf));
  Check(AField.UsedBy.Count = 1);
end;

{ TUntypedParameterTest }

function TUntypedParameterTest.GetModuleFullName: String;
begin
  Result := '.\UntypedParameterTest.dpr';
end;

procedure TUntypedParameterTest.TestUntypedParameter;
var
  ARoutine: TcwRoutine;
  AParameter: TcwParameter;
begin
  ARoutine := gcwProject.AllUnits[0].Routines.Find('Test');
  Check(Assigned(ARoutine));
  AParameter := ARoutine.Parameters.Find('ABuffer');
  Check(Assigned(AParameter));
  Check(AParameter.UntypedParameter);

  ARoutine := gcwProject.AllUnits[0].Routines.Find('Test2');
  Check(Assigned(ARoutine));
  AParameter := ARoutine.Parameters.Find('ABuffer');
  Check(Assigned(AParameter));
  Check(AParameter.UntypedParameter);
  AParameter := ARoutine.Parameters.Find('ABuffer2');
  Check(Assigned(AParameter));
  Check(AParameter.UntypedParameter);

  ARoutine := gcwProject.AllUnits[0].Routines.Find('Test3');
  Check(Assigned(ARoutine));
  AParameter := ARoutine.Parameters.Find('ThisData');
  Check(Assigned(AParameter));
  Check(AParameter.UntypedParameter);
  AParameter := ARoutine.Parameters.Find('ThatData');
  Check(Assigned(AParameter));
  Check(AParameter.UntypedParameter);
  AParameter := ARoutine.Parameters.Find('Value');
  Check(Assigned(AParameter));
  Check(not AParameter.UntypedParameter);
  AParameter := ARoutine.Parameters.Find('Value2');
  Check(Assigned(AParameter));
  Check(not AParameter.UntypedParameter);
end;

{ TFileOfTypeTest }

function TFileOfTypeTest.GetModuleFullName: String;
begin
  Result := '.\FileOfTypeTest.dpr';
end;

procedure TFileOfTypeTest.TestFileOfType;
var
  AVariable: TcwVariable;
begin
  AVariable := gcwProject.AllUnits[0].Variables.Find('AFile');
  Check(Assigned(AVariable));
  Check(Assigned(AVariable.TypeIntf));
end;

{ TQualifiedTypeDeclarationTest }

function TQualifiedTypeDeclarationTest.GetModuleFullName: String;
begin
  Result := '.\QualifiedTypeDeclarationTest.dpr';
end;

procedure TQualifiedTypeDeclarationTest.TestQualifiedTypeDeclaration;
var
  AType: TcwSymbol;
  AType2: TcwSymbol;
begin
  AType := gcwProject.AllUnits[0].Types.Find('TOtherUnitType');
  Check(Assigned(AType));
  Check(Assigned(AType.TypeIntf));
  Check(AType.TypeIntf.UnitIntf.Name = 'uQualifiedTypeDeclarationTest');
  AType2 := gcwProject.AllUnits[0].Types.Find('TOtherUnitType2');
  Check(Assigned(AType2));
  Check(Assigned(AType2.TypeIntf));
  Check(AType2.TypeIntf.UnitIntf.Name = 'uQualifiedTypeDeclarationTest2');
end;

{ TVariantRecordTest }

function TVariantRecordTest.GetModuleFullName: String;
begin
  Result := '.\VariantRecordTest.dpr';
end;

procedure TVariantRecordTest.TestVariantRecord;
var
  AType: TcwSymbol;
  AVariable: TcwVariable;
begin
  AType := gcwProject.AllUnits[0].Types.Find('TTestRecord');
  Check(Assigned(AType));
  AVariable := (AType as TcwRecord).Fields.Find('TestField');
  Check(Assigned(AVariable));
  Check(AVariable.UsedBy.Count = 1);
  AVariable := (AType as TcwRecord).Fields.Find('TestField2');
  Check(Assigned(AVariable));
  Check(AVariable.UsedBy.Count = 1);
  AVariable := (AType as TcwRecord).Fields.Find('TestTag');
  Check(Assigned(AVariable));
  Check(AVariable.UsedBy.Count = 1);
end;

{ TForwardClassOfTest }

function TForwardClassOfTest.GetModuleFullName: String;
begin
  Result := '.\ForwardClassOfTest.dpr';
end;

procedure TForwardClassOfTest.TestForwardClassOf;
var
  AType: TcwSymbol;
  AClass: TcwClass;
begin
  AType := gcwProject.AllUnits[0].Types.Find('TSomeObjectClass');
  AClass := gcwProject.AllUnits[0].Classes.Find('TSomeObject');
  Check(Assigned(AType));
  Check(AType.TypeIntf as TcwClass = AClass);
end;

{ TCallingConventionTest }

function TCallingConventionTest.GetModuleFullName: String;
begin
  Result := '.\CallingConventionTest.dpr';
end;

procedure TCallingConventionTest.TestCallingConvention;
begin
  Check(Assigned(gcwProject.AllUnits[0].Routines.Find('Test1')));
  Check(Assigned(gcwProject.AllUnits[0].Routines.Find('Test2')));
  Check(Assigned(gcwProject.AllUnits[0].Routines.Find('Test3')));
  Check(Assigned(gcwProject.AllUnits[0].Routines.Find('Test4')));
  Check(Assigned(gcwProject.AllUnits[0].Routines.Find('Test5')));
  Check(Assigned(gcwProject.AllUnits[0].Routines.Find('Test6')));
end;

{ TExternalDirectiveTest }

function TExternalDirectiveTest.GetModuleFullName: String;
begin
  Result := '.\ExternalDirectiveTest.dpr';
end;

{ TCreateInWithTest }

function TCreateInWithTest.GetModuleFullName: String;
begin
  Result := '.\CreateInWithTest.dpr';
end;

procedure TCreateInWithTest.TestCreateInWith;
var
  AField: TcwVariable;
  AClass: TcwClass;
begin
  AClass := gcwProject.AllUnits[0].Classes.Find('TSomeObject');
  Check(Assigned(AClass));
  AField := AClass.Fields.Find('FX');
  Check(Assigned(AField));
  Check(AField.UsedBy.Count = 1);

  AClass := gcwProject.AllUnits[0].Classes.Find('TSomeObject2');
  Check(Assigned(AClass));
  AField := AClass.Fields.Find('FY');
  Check(Assigned(AField));
  Check(AField.UsedBy.Count = 1);
end;

{ TProtectedVisibilityTest }

function TProtectedVisibilityTest.GetModuleFullName: String;
begin
  Result := '.\ProtectedVisibilityTest.dpr';
end;

procedure TProtectedVisibilityTest.TestProtectedVisibility;
var
  AUnit: TcwUnit;
  AField: TcwVariable;
begin
  AUnit := gcwProject.AllUnits.FindUnit('uProtectedVisibilityTest');
  Check(Assigned(AUnit));
  AField := AUnit.Classes[0].Fields.Find('FX');
  Check(Assigned(AField));
  Check(AField.UsedBy.Count = 2);
end;

{ TTypeTest001 }

function TTypeTest001.GetModuleFullName: String;
begin
  Result := '.\TypeTest001.dpr';
end;

procedure TTypeTest001.TestTypeDeclaration;
var
  AType: TcwSymbol;
begin
  AType := gcwProject.AllUnits[0].Types.Find('TFloat');
  Check(Assigned(AType));
end;

initialization

  RegisterTest('CodeLens', TArrayPointerTest.Suite);
  RegisterTest('CodeLens', TPropertyVisibilityOverrideTest.Suite);
  RegisterTest('CodeLens', TAsOperatorTest.Suite);
  RegisterTest('CodeLens', TTypeCastTest.Suite);
  RegisterTest('CodeLens', TDefaultPropertyTest.Suite);
  RegisterTest('CodeLens', TDefaultPropertyOverrideTest.Suite);
  RegisterTest('CodeLens', TUntypedParameterTest.Suite);
  RegisterTest('CodeLens', TFileOfTypeTest.Suite);
  RegisterTest('CodeLens', TQualifiedTypeDeclarationTest.Suite);
  RegisterTest('CodeLens', TVariantRecordTest.Suite);
  RegisterTest('CodeLens', TForwardClassOfTest.Suite);
  RegisterTest('CodeLens', TCallingConventionTest.Suite);
  RegisterTest('CodeLens', TExternalDirectiveTest.Suite);
  RegisterTest('CodeLens', TCreateInWithTest.Suite);
  RegisterTest('CodeLens', TProtectedVisibilityTest.Suite);
  RegisterTest('CodeLens', TTypeTest001.Suite);

end.

