(*  Copyright (c) 2002, the CodeLens project
  All rights reserved.

  Redistribution and use in source and binary forms,
  with or without modification, are permitted provided
  that the following conditions are met:

  Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.

  Redistributions in binary form must reproduce the
  above copyright notice, this list of conditions
  and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
  Neither the name of the CodeLens project nor the names
  of its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS
  AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. *)

unit frmcwBase;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs;

type
  TFormBase = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    FWndPl: TWindowPlacement;
  protected
  public
  end;

implementation

{$R *.dfm}

uses
  ucwClasses;

procedure TFormBase.FormCreate(Sender: TObject);
var
  vOptions: IcwFileSettings;
begin
  vOptions := gcwCodeWrenchOptions;
  if vOptions.GetOption(Self.ClassName, 'Docked', False) = False then
  begin
    Self.Left := vOptions.GetOption(Self.ClassName, 'Left', Self.Left);
    Self.Top := vOptions.GetOption(Self.ClassName, 'Top', Self.Top);

    if (BorderStyle = bsSizeable) then
    begin
      Self.Width := vOptions.GetOption(Self.ClassName, 'Width', Self.Width);
      Self.Height := vOptions.GetOption(Self.ClassName, 'Height', Self.Height);
      Self.WindowState := vOptions.GetOption(Self.ClassName, 'WindowState',
        Integer(Self.WindowState));

    end;
  end;
end;

procedure TFormBase.FormDestroy(Sender: TObject);
var
  vOptions: IcwFileSettings;
begin
  vOptions := gcwCodeWrenchOptions;
  vOptions.SetOption(Self.ClassName, 'Docked', not Floating);
  vOptions.SetOption(Self.ClassName, 'Left', Self.Left);
  vOptions.SetOption(Self.ClassName, 'Top', Self.Top);
  if BorderStyle = bsSizeable then
  begin

    vOptions.SetOption(Self.ClassName, 'Width', Self.Width);
    vOptions.SetOption(Self.ClassName, 'Height', Self.Height);
    vOptions.SetOption(Self.ClassName, 'WindowState',
      Integer(Self.WindowState));
    with FWndPl do
      if ShowCmd = SW_SHOWMAXIMIZED then
      begin
        vOptions.SetOption(Self.ClassName, 'Left', rcNormalPosition.Left);
        vOptions.SetOption(Self.ClassName, 'Top', rcNormalPosition.Top);
        vOptions.SetOption(Self.ClassName, 'Width', rcNormalPosition.Right -
          rcNormalPosition.Left);
        vOptions.SetOption(Self.ClassName, 'Height', rcNormalPosition.Bottom -
          rcNormalPosition.Top);
      end;
  end;
end;

procedure TFormBase.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  FWndPl.length := SizeOf(FWndPl);
  GetWindowPlacement(Self.Handle, @FWndPl);
end;

end.
