(*  Copyright (c) 2002, the CodeLens project
  All rights reserved.

  Redistribution and use in source and binary forms,
  with or without modification, are permitted provided
  that the following conditions are met:

  Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.

  Redistributions in binary form must reproduce the
  above copyright notice, this list of conditions
  and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
  Neither the name of the CodeLens project nor the names
  of its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS
  AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. *)

unit frmcwCodeOptions;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, frmcwBaseDialog, frmcwBase, StdCtrls, ExtCtrls, ComCtrls, ucwClasses;

type
  TCodeOptions = class(TBaseDialogForm)
    pgcCodingConventions: TPageControl;
    tabNamingConventions: TTabSheet;
    tabComponentPrefixes: TTabSheet;
    lvCmpPrefixes: TListView;
    tbnAddComponentPrefix: TButton;
    btnDeleteComponentPrefix: TButton;
    GroupBox1: TGroupBox;
    chkEnforceOnlyIfComponentUsed: TCheckBox;
    btnGetDefaultPrefixes: TButton;
    Label1: TLabel;
    gbProperties: TGroupBox;
    edtProperties: TEdit;
    Label3: TLabel;
    Label5: TLabel;
    edtGetRoutines: TEdit;
    Label8: TLabel;
    edtSetRoutines: TEdit;
    gbRoutines: TGroupBox;
    Label2: TLabel;
    edtRoutineName: TEdit;
    edtConstParameters: TEdit;
    edtVarParameters: TEdit;
    Label7: TLabel;
    edtOtherParameters: TEdit;
    Label6: TLabel;
    Label4: TLabel;
    gbTypes: TGroupBox;
    Label13: TLabel;
    Label14: TLabel;
    Label15: TLabel;
    Label16: TLabel;
    edtClasses: TEdit;
    edtInterfaces: TEdit;
    edtRecords: TEdit;
    edtPointers: TEdit;
    Label17: TLabel;
    edtObjects: TEdit;
    edtRegExprHelp: TEdit;
    gbVariables: TGroupBox;
    Label10: TLabel;
    edtLocalVariables: TEdit;
    Label11: TLabel;
    edtGlobalVariables: TEdit;
    Label12: TLabel;
    edtConstants: TEdit;
    Label9: TLabel;
    edtClassFields: TEdit;
    tabIndentation: TTabSheet;
    cbSpacesPerIndent: TComboBox;
    Label18: TLabel;
    GroupBox2: TGroupBox;
    chkExtraIndentBeforeBegin: TCheckBox;
    chkExtraIndentBeforeTry: TCheckBox;
    chkExtraIndentBeforeElseInTryBlock: TCheckBox;
    chkExtraIndentBeforeElseInCaseBlock: TCheckBox;
    GroupBox3: TGroupBox;
    chkNeverindentElseIf: TCheckBox;
    chkIndentComments: TCheckBox;
    chkIndentCompilerDirectives: TCheckBox;
    GroupBox4: TGroupBox;
    ListView1: TListView;
    procedure tbnAddComponentPrefixClick(Sender: TObject);
    procedure ButtonOKClick(Sender: TObject);
    procedure btnDeleteComponentPrefixClick(Sender: TObject);
    procedure lvCmpPrefixesKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure lvCmpPrefixesDblClick(Sender: TObject);
    procedure btnGetDefaultPrefixesClick(Sender: TObject);
  private
    FCodingConventions : IcwCodingConventions;
    procedure AddPrefix(const aPrefix : IcwComponentPrefix);
    procedure GetIndentationConventions(const aConventions : IcwIndentationConventions);
    procedure FillIndentationConventions(const aConventions : IcwIndentationConventions);
    procedure GetNamingConventions(const aConventions : IcwNamingConventions);
    procedure FillNamingConventions(const aConventions : IcwNamingConventions);
    procedure FillComponentPrefixes(const aPrefixes : IcwPrefixList);

    procedure DisplayOptions;
  public
    { Public declarations }
  end;

procedure ShowCodingConventions(const aCodingConventions : IcwCodingConventions);

implementation

uses frmcwPrefixDlg;

{$R *.dfm}

procedure ShowCodingConventions(const aCodingConventions : IcwCodingConventions);
var fCodeOptions : TCodeOptions;
begin
  fCodeOptions := TCodeOptions.Create(nil);
  try
    fCodeOptions.FCodingConventions := aCodingConventions;
    fCodeOptions.DisplayOptions;
    if fCodeOptions.ShowModal = mrOK then
      aCodingConventions.Save;
  finally
    fCodeOptions.Release;
  end;
end;


procedure TCodeOptions.GetIndentationConventions(const aConventions : IcwIndentationConventions);
begin
  aConventions.SpacesPerIndent := cbSpacesPerIndent.ItemIndex;
  //Extra Indentations Before
  aConventions.IndentBeforeBegin := chkExtraIndentBeforeBegin.Checked;
  aConventions.IndentBeforeTry   := chkExtraIndentBeforeTry.Checked;
  aConventions.IndentBeforeElseInTryBlock := chkExtraIndentBeforeElseInTryBlock.Checked;
  aConventions.IndentBeforeElseInCaseBlock := chkExtraIndentBeforeElseInCaseBlock.Checked;
  //Indentation exceptions
  aConventions.NeverIndentElseIf := chkNeverindentElseIf.Checked;
  aConventions.IndentComments    := chkIndentComments.Checked;
  aConventions.IndentDirectives  := chkIndentCompilerDirectives.Checked;

end;

procedure TCodeOptions.FillIndentationConventions(const aConventions : IcwIndentationConventions);
begin
  cbSpacesPerIndent.ItemIndex := aConventions.SpacesPerIndent;
  //Extra Indentations Before
  chkExtraIndentBeforeBegin.Checked := aConventions.IndentBeforeBegin;
  chkExtraIndentBeforeTry.Checked   := aConventions.IndentBeforeTry;
  chkExtraIndentBeforeElseInTryBlock.Checked := aConventions.IndentBeforeElseInTryBlock;
  chkExtraIndentBeforeElseInCaseBlock.Checked := aConventions.IndentBeforeElseInCaseBlock;
  //Indentation exceptions
  chkNeverindentElseIf.Checked := aConventions.NeverIndentElseIf;
  chkIndentComments.Checked    := aConventions.IndentComments;
  chkIndentCompilerDirectives.Checked := aConventions.IndentDirectives;
end;


procedure TCodeOptions.GetNamingConventions(const aConventions : IcwNamingConventions);
begin
  aConventions.LocalVariables  := edtLocalVariables.Text;
  aConventions.GlobalVariables := edtGlobalVariables.Text;
  aConventions.Constants       := edtConstants.Text;
  aConventions.ClassFields     := edtClassFields.Text;

  aConventions.RoutineName     := edtRoutineName.Text;
  aConventions.ConstParameters := edtConstParameters.Text;
  aConventions.VarParameters   := edtVarParameters.Text;
  aConventions.Parameters      := edtOtherParameters.Text;


  aConventions.Properties      := edtProperties.Text;
  aConventions.GetRoutines     := edtGetRoutines.Text;
  aConventions.SetRoutines     := edtSetRoutines.Text;

  aConventions.ClassTypes      := edtClasses.Text;
  aConventions.InterfaceTypes  := edtInterfaces.Text;
  aConventions.RecordTypes     := edtRecords.Text;
  aConventions.ObjectTypes     := edtObjects.Text;
  aConventions.PointerTypes    := edtPointers.Text;

end;

procedure TCodeOptions.FillNamingConventions(const aConventions : IcwNamingConventions);
begin
  edtLocalVariables.Text   := aConventions.LocalVariables;
  edtGlobalVariables.Text  := aConventions.GlobalVariables;
  edtConstants.Text        := aConventions.Constants;
  edtClassFields.Text      := aConventions.ClassFields;

  edtRoutineName.Text      := aConventions.RoutineName;
  edtConstParameters.Text  := aConventions.ConstParameters;
  edtVarParameters.Text    := aConventions.VarParameters;
  edtOtherParameters.Text  := aConventions.Parameters;


  edtProperties.Text       := aConventions.Properties;
  edtGetRoutines.Text      := aConventions.GetRoutines;
  edtSetRoutines.Text      := aConventions.SetRoutines;

  edtClasses.Text          := aConventions.ClassTypes;
  edtInterfaces.Text       := aConventions.InterfaceTypes;
  edtRecords.Text          := aConventions.RecordTypes;
  edtObjects.Text          := aConventions.ObjectTypes;
  edtPointers.Text         := aConventions.PointerTypes;
end;


procedure TCodeOptions.AddPrefix(const aPrefix : IcwComponentPrefix);
var vItem : TListItem;
begin
  vItem := lvCmpPrefixes.Items.Add;
  vItem.Caption := aPrefix.Prefix;
  vItem.SubItems.Add(aPrefix.ComponentClass);
  vItem.Data := Pointer(aPrefix);
end;


procedure TCodeOptions.FillComponentPrefixes(const aPrefixes : IcwPrefixList);
var i : Integer;
begin
  chkEnforceOnlyIfComponentUsed.Checked := not aPrefixes.EnforcePrefixOnUnreferecend;
  lvCmpPrefixes.Items.BeginUpdate;
  try
    lvCmpPrefixes.Items.Clear;
    for i := 0 to aPrefixes.Count - 1 do
      AddPrefix(aPrefixes[i]);
  finally
    lvCmpPrefixes.Items.EndUpdate;
  end;
end;

procedure TCodeOptions.DisplayOptions;
begin
  pgcCodingConventions.ActivePage := pgcCodingConventions.Pages[0];
  FillIndentationConventions(FCodingConventions.IndentationConventions);  
  FillNamingConventions(FCodingConventions.NamingConventions);
  FillComponentPrefixes(FCodingConventions.ComponentPrefixes);
end;

procedure TCodeOptions.tbnAddComponentPrefixClick(Sender: TObject);
var aPrefix, aComponentClass : string;
var vPrefix : IcwComponentPrefix;
begin
  aPrefix := '';
  aComponentClass := '';
  if GetComponentPrefix(aPrefix, aComponentClass) then
  begin
    vPrefix := FCodingConventions.ComponentPrefixes.Add(aPrefix, aComponentClass);
    AddPrefix(vPrefix);
  end;
end;

procedure TCodeOptions.ButtonOKClick(Sender: TObject);
begin
  GetIndentationConventions(FCodingConventions.IndentationConventions);
  FCodingConventions.ComponentPrefixes.EnforcePrefixOnUnreferecend := not chkEnforceOnlyIfComponentUsed.Checked;
  GetNamingConventions(FCodingConventions.NamingConventions);

end;

procedure TCodeOptions.btnDeleteComponentPrefixClick(Sender: TObject);
var Item : TListItem;
begin
  Item := lvCmpPrefixes.Selected;
  if Item = nil then EXIT;
  if MessageDlg('Are you sure you want to delete the selected prefix?',
                mtConfirmation,[mbYes, mbNo],0) = mrYes then
  begin
    FCodingConventions.ComponentPrefixes.DeletePrefix(IcwComponentPrefix(Item.data));
    lvCmpPrefixes.Items.Delete(Item.Index);
  end;
end;

procedure TCodeOptions.lvCmpPrefixesKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_DELETE then
    btnDeleteComponentPrefixClick(Sender);
end;

procedure TCodeOptions.lvCmpPrefixesDblClick(Sender: TObject);
var Item : TListItem;
var vPrefix : string;
var vComponentClass : string;
var vPrefixIntf : IcwComponentPrefix;
begin
  Item := lvCmpPrefixes.Selected;
  if Item = nil then EXIT;
  vPrefixIntf := IcwComponentPrefix(Item.Data);
  vPrefix := vPrefixIntf.Prefix;
  vComponentClass := vPrefixIntf.ComponentClass;
  if GetComponentPrefix(vPrefix, vComponentClass) then
  begin
    vPrefixIntf.Prefix := vPrefix;
    vPrefixIntf.ComponentClass := vComponentClass;
    Item.Caption := vPrefix;
    Item.SubItems[0] := vComponentClass;
  end;


end;

procedure TCodeOptions.btnGetDefaultPrefixesClick(Sender: TObject);
begin
  if not MessageDlg('Are you sure you want to reset the default prefixes?',
                mtConfirmation,[mbYes, mbNo],0) = mrYes then EXIT;
  FCodingConventions.ComponentPrefixes.LoadDefaultPrefixes;

  FillComponentPrefixes(FCodingConventions.ComponentPrefixes);
end;

end.
