(*  Copyright (c) 2002, the CodeLens project
  All rights reserved.

  Redistribution and use in source and binary forms,
  with or without modification, are permitted provided
  that the following conditions are met:

  Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.

  Redistributions in binary form must reproduce the
  above copyright notice, this list of conditions
  and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
  Neither the name of the CodeLens project nor the names
  of its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS
  AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. *)

unit frmcwFind;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  ExtCtrls,
  frmcwBase,
  frmcwBaseDialog;

type
  TFindForm = class(TBaseDialogForm)
    FindEdit: TEdit;
    Label1: TLabel;
    CaseSensitiveCheckBox: TCheckBox;
    WholeWordsOnlyCheckBox: TCheckBox;
    RegularExpressionCheckBox: TCheckBox;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    function DoExecRegExpr(AString: string): Boolean;
  public
    function Match(AString: string): Boolean;
  end;

var
  FindForm: TFindForm;

implementation

{$R *.dfm}

uses
  RegExpr;

{ TFindForm }

function TFindForm.DoExecRegExpr(AString: string): Boolean;
var
  ARegExpr: TRegExpr;
begin
  ARegExpr := TRegExpr.Create;
  try
    ARegExpr.Expression := FindEdit.Text;
    ARegExpr.ModifierI := not CaseSensitiveCheckBox.Checked;
    Result := ARegExpr.Exec(AString);
  finally
    ARegExpr.Free;
  end;
end;

function TFindForm.Match(AString: string): Boolean;
begin
  if RegularExpressionCheckBox.Checked then
    Result := DoExecRegExpr(AString)
  else
    if WholeWordsOnlyCheckBox.Checked then
    begin
      if CaseSensitiveCheckBox.Checked then
        Result := AString = FindEdit.Text
      else
        Result := AnsiSameText(AString, FindEdit.Text);
    end else
      if CaseSensitiveCheckBox.Checked then
        Result := Pos(FindEdit.Text, AString) > 0
      else
        Result := Pos(Lowercase(FindEdit.Text), Lowercase(AString)) > 0;
end;

procedure TFindForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if FindEdit.Text = '' then
    ModalResult := mrCancel;
end;

end.
