(*  Copyright (c) 2002, the CodeLens project
  All rights reserved.

  Redistribution and use in source and binary forms,
  with or without modification, are permitted provided
  that the following conditions are met:

  Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.

  Redistributions in binary form must reproduce the
  above copyright notice, this list of conditions
  and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
  Neither the name of the CodeLens project nor the names
  of its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS
  AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. *)

unit frmcwOpenURL;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls;

type
  TfrmOpenURL = class(TForm)
    imgLogo: TImage;
    lblOpen: TLabel;
    btnOK: TButton;
    btnCancel: TButton;
    cbURL: TComboBox;
    procedure cbURLChange(Sender: TObject);

  private
    procedure FillItems(sl: TStrings);

  protected
    function GetURL: string;
    procedure SetURL(const aValue: string);

  public
    function Execute: boolean;

    property URL: string
      read GetURL
      write SetURL;
  end;

function EditURL(Items: TStrings): string;

implementation

{$R *.DFM}

uses
  ucwNet;

// ------------------------------------------------------------------------------------------

function EditURL(Items: TStrings): string;
begin
  with TfrmOpenURL.Create(Application) do
  try
    FillItems(Items);

    if Execute then
      Result := URL
    else
      Result := '';

  finally
    Release;
  end;
end;

// ------------------------------------------------------------------------------------------


function TfrmOpenURL.GetURL: string;
begin
  Result := cbURL.Text;
end;

procedure TfrmOpenURL.SetURL(const aValue: string);
begin
  cbURL.Text := aValue;
end;

function TfrmOpenURL.Execute: boolean;
begin
  Result := (ShowModal = mrOK);
end;

procedure TfrmOpenURL.cbURLChange(Sender: TObject);
begin
  btnOK.Enabled := IsValidURL(cbURL.Text);
end;

procedure TfrmOpenURL.FillItems(sl: TStrings);
var
  i: integer;
begin
  // fill URL history dropdown & filter on URLs
  cbURL.Items := sl;
  for i:=cbURL.Items.Count-1 downto 0 do
    if not IsValidURL(cbURL.Items[i]) then
      cbURL.Items.Delete(i);

  // select first history item
  if (cbURL.Items.Count > 0) then
    cbURL.Text := cbURL.Items[0];
end;

end.
