(*  Copyright (c) 2002, the CodeLens project
  All rights reserved.

  Redistribution and use in source and binary forms,
  with or without modification, are permitted provided
  that the following conditions are met:

  Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.

  Redistributions in binary form must reproduce the
  above copyright notice, this list of conditions
  and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
  Neither the name of the CodeLens project nor the names
  of its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS
  AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. *)

unit frmcwOptions;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, VirtualTrees, ucwClasses, ImgList,
  ExtCtrls, Menus, frmcwBaseDialog;

type
  TfrmOptions = class(TBaseDialogForm)
    pgcOptions: TPageControl;
    tabProject: TTabSheet;
    tabPascalLint: TTabSheet;
    lblProjectDirs: TLabel;
    memProjectDirs: TMemo;
    btnProjectDefaults: TButton;
    btnProjectBrowse: TButton;
    lblLibDirs: TLabel;
    memLibDirs: TMemo;
    btnLibDefaults: TButton;
    btnLibBrowse: TButton;
    vsTReportOptions: TVirtualStringTree;
    cbReports: TComboBox;
    Label1: TLabel;
    liCheckBoxes: TImageList;
    pmGetDefaults: TPopupMenu;
    ofdOpenFolder: TOpenDialog;
    tabProjectDefines: TTabSheet;
    memProjectDefines: TMemo;
    ofdOpenFile: TOpenDialog;
    edtFileToAnalyze: TEdit;
    btnFileBrowse: TButton;
    lblFileToAnalyze: TLabel;
    AliasesMemo: TMemo;
    Label2: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure vsTReportOptionsInitNode(Sender: TBaseVirtualTree; ParentNode,
      Node: PVirtualNode; var InitialStates: TVirtualNodeInitStates);
    procedure btnOKClick(Sender: TObject);
    procedure vsTReportOptionsGetText(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
      var CellText: WideString);
    procedure cbReportsChange(Sender: TObject);
    procedure vsTReportOptionsInitChildren(Sender: TBaseVirtualTree;
      Node: PVirtualNode; var ChildCount: Cardinal);
    procedure vsTReportOptionsPaintText(Sender: TBaseVirtualTree;
      const TargetCanvas: TCanvas; Node: PVirtualNode;
      Column: TColumnIndex; TextType: TVSTTextType);
    procedure vsTReportOptionsGetImageIndex(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Kind: TVTImageKind; Column: TColumnIndex;
      var Ghosted: Boolean; var ImageIndex: Integer);
    procedure vsTReportOptionsMouseDown(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btnProjectDefaultsClick(Sender: TObject);
    procedure btnProjectBrowseClick(Sender: TObject);
    procedure btnLibDefaultsClick(Sender: TObject);
    procedure btnLibBrowseClick(Sender: TObject);
    procedure btnFileBrowseClick(Sender: TObject);
  private
    FSettings: IcwSettings;
    FMemoDirs: TMemo;
    procedure OnSelectDefaultClick(Sender: TObject);
    procedure FillDefaultDirs;
    procedure FillSettings(const aSettings: IcwSettings);
    procedure FillSelectedSettings;
    procedure FillReportList;
    function ButtonToMemo(AButton: TButton): TMemo;
  public
    class function Execute: boolean;
  end;

implementation

{$R *.dfm}

uses
  ucwPathUtils;

class function TfrmOptions.Execute: Boolean;
begin
  with TfrmOptions.Create(Application) do
    try
      Result := ShowModal = mrOK;
    finally
      Release;
    end;
end;

type
  PNodeData = ^TNodeData;
  TNodeData = record
    IsCategory: Boolean;
    Data: Pointer;
  end;

procedure TfrmOptions.FillSettings(const aSettings: IcwSettings);
begin
  FSettings := aSettings;
  vsTReportOptions.RootNodeCount := 0;
  vsTReportOptions.RootNodeCount := aSettings.CategoryCount +
    aSettings.Count;
  vsTReportOptions.Invalidate;
end;

procedure TfrmOptions.FillSelectedSettings;
var
  vReport: IcwReport;
begin
  if cbReports.ItemIndex < 0 then
    Exit;
  vReport := IcwReport(pointer(cbReports.Items.Objects[cbReports.ItemIndex]));
  FillSettings(vReport.Settings);
end;

procedure TfrmOptions.FillReportList;
var
  i: Integer;
  vReport: IcwReport;
begin
  for i := 0 to gcwProject.Reports.Count - 1 do
  begin
    vReport := gcwProject.Reports[i];
    cbReports.Items.AddObject(vReport.Name, Pointer(vReport));
  end;
end;

procedure TfrmOptions.FormCreate(Sender: TObject);
begin
  pgcOptions.Activepage := tabProject;
  FillDefaultDirs;

  edtFileToAnalyze.Text := gcwProject.CWOptions.FileToAnalyze;
  memProjectDirs.Lines.Text := gcwProject.CWOptions.ProjectSearchPath.Text;
  memLibDirs.Lines.Text := gcwProject.SystemOptions.LibrarySearchPath.Text;
  memProjectDefines.Lines.Text := gcwProject.CWOptions.Defines.Text;
  AliasesMemo.Lines.Text := gcwProject.CWOptions.Aliases.Text;

  vsTReportOptions.NodeDataSize := SizeOf(TNodeData);
  FillReportList;
  FillSelectedSettings;
end;

procedure TfrmOptions.vsTReportOptionsInitNode(Sender: TBaseVirtualTree; ParentNode,
  Node: PVirtualNode; var InitialStates: TVirtualNodeInitStates);
var
  Data: PNodeData;
  ParentSettings: IcwSettingsCategory;
begin
  if FSettings = nil then
    Exit;
  if ParentNode = nil then
  begin
    ParentSettings := FSettings;
  end
  else
  begin
    Data := Sender.GetNodeData(ParentNode);
    ParentSettings := IcwSettingsCategory(Data.Data);
  end;

  Data := Sender.GetNodeData(Node);
  if Integer(Node.Index) < ParentSettings.CategoryCount then
  begin
    Data.IsCategory := True;
    Data.Data := Pointer(ParentSettings.Categories[Node.Index]);
    InitialStates := InitialStates + [ivsHasChildren];
  end
  else
  begin
    Data.IsCategory := False;
    Data.Data := Pointer(ParentSettings[Integer(Node.Index) - ParentSettings.CategoryCount]);
  end;

end;

procedure TfrmOptions.btnOKClick(Sender: TObject);

  procedure CheckDirectories(AStrings: TStrings; APathsName: string);
  var
    AText: string;
    Index: Integer;
  begin
    AText := '';
    for Index := 0 to AStrings.Count - 1 do
      if not DirectoryExists(ResolvePathName(AStrings[Index])) then
        if Index = 0 then
          AText := AStrings[Index]
        else
          AText := AText + SLineBreak + AStrings[Index];
    if AText <> '' then
      ShowMessageFmt('The following %s paths do not exist:'#13#10#13#10'%s', [APathsName, AText]);
  end;

var
  i: Integer;
begin
  for i := 0 to gcwProject.Reports.Count - 1 do
    gcwProject.Reports[i].Settings.SaveSettings(gcwProject.ProjectOptions);

  if not FileExists(edtFileToAnalyze.Text) then
    ShowMessageFmt('"File To Analyze", %s, does not exist.', [edtFileToAnalyze.Text]);
  gcwProject.CWOptions.FileToAnalyze := edtFileToAnalyze.Text;

  CheckDirectories(memProjectDirs.Lines, 'Project');
  gcwProject.CWOptions.ProjectSearchPath.Text := memProjectDirs.Lines.Text;

  CheckDirectories(memLibDirs.Lines, 'Library');
  gcwProject.SystemOptions.LibrarySearchPath.Text := memLibDirs.Lines.Text;

  gcwProject.CWOptions.Defines.Text := memProjectDefines.Lines.Text;
  gcwProject.CWOptions.Aliases.Text := AliasesMemo.Lines.Text;
  gcwProject.CWOptions.Save(gcwProject.ProjectOptions);
end;

procedure TfrmOptions.vsTReportOptionsGetText(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
  var CellText: WideString);
var
  Data: PNodeData;
begin
  Data := Sender.GetNodeData(Node);

  if Data.IsCategory then
  begin
    if Column = 0 then
      CellText := IcwSettingsCategory(Data.Data).Name
    else
      CellText := '';
  end
  else
  begin
    if Column = 0 then
      CellText := IcwSetting(Data.Data).Name
    else
      CellText := '';
  end;
end;

procedure TfrmOptions.cbReportsChange(Sender: TObject);
begin
  FillSelectedSettings;
end;

procedure TfrmOptions.vsTReportOptionsInitChildren(Sender: TBaseVirtualTree;
  Node: PVirtualNode; var ChildCount: Cardinal);
var
  Data: PNodeData;
begin
  Data := Sender.GetNodeData(Node);
  if Data.IsCategory then
    ChildCount := IcwSettingsCategory(Data.Data).Count +
      IcwSettingsCategory(Data.Data).CategoryCount;
end;

procedure TfrmOptions.vsTReportOptionsPaintText(Sender: TBaseVirtualTree;
  const TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
  TextType: TVSTTextType);
var
  Data: PNodeData;
begin
  Data := Sender.GetNodeData(Node);
  if Data.IsCategory then
    Canvas.Font.Style := [fsBold]
  else
    Canvas.Font.Style := [];
end;

procedure TfrmOptions.vsTReportOptionsGetImageIndex(
  Sender: TBaseVirtualTree; Node: PVirtualNode; Kind: TVTImageKind;
  Column: TColumnIndex; var Ghosted: Boolean; var ImageIndex: Integer);
var
  Data: PNodeData;
  vSetting: IcwSetting;
begin
  Data := Sender.GetNodeData(Node);
  if Data.IsCategory then
    ImageIndex := 0
  else
  begin
    vSetting := IcwSetting(Data.Data);
    case Column of
      1: if vSetting.ProjectSet then
          ImageIndex := 2
        else
          ImageIndex := 1;

      2: if vSetting.LibrarySet then
          ImageIndex := 2
        else
          ImageIndex := 1;
    end;
  end;

end;

procedure TfrmOptions.vsTReportOptionsMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  HitInfo: THitInfo;
  Data: PNodeData;
  vSetting: IcwSetting;
begin
  // get information about the hit
  vsTReportOptions.GetHitTestInfoAt(X, Y, True, HitInfo);
  if HitInfo.HitNode = nil then
    Exit;
  Data := vsTReportOptions.GetNodeData(HitInfo.HitNode);
  if Data.IsCategory then
    Exit;

  vSetting := IcwSetting(Data.Data);

  case HitInfo.HitColumn of
    1: vSetting.ProjectSet := not vSetting.ProjectSet;
    2: vSetting.LibrarySet := not vSetting.LibrarySet;
  end;
  vsTReportOptions.RepaintNode(HitInfo.HitNode);
end;

procedure TfrmOptions.OnSelectDefaultClick(Sender: TObject);
var
  i: Integer;
begin
  if (FMemoDirs.Lines.Count > 0) and
    (MessageDlg('Do you want to clear the current search dirs ?', mtConfirmation,
    [mbYes, mbNo], 0) = mrYes) then
    FMemoDirs.Lines.Clear;
  with TStringList((Sender as TMenuItem).Tag) do
    for i := 0 to Count - 1 do
      FMemoDirs.Lines.Add(Strings[i]);
end;

procedure TfrmOptions.FillDefaultDirs;

  function AddMenuItem(Menu: TMenuItem; const aItemCaption: string;
    OnClickEvent: TNotifyEvent; Tag: Integer): TMenuItem;
  begin
    Result := TMenuItem.Create(Menu);
    Result.Caption := aItemCaption;
    Result.OnClick := OnClickEvent;
    Result.Tag := Tag;
    Menu.Add(Result);
  end;

var
  i: Integer;
  DirsList: TStringList;
begin
  inherited;
  DirsList := TStringList.Create;
  try
    GetDefaultDirs(gcwProject.AllUnits[0].FullFileName, DirsList);
    for i := 0 to DirsList.Count - 1 do
    begin
      AddMenuItem(pmGetDefaults.Items, DirsList[i], OnSelectDefaultClick,
        Integer(DirsList.Objects[i]));
    end;
    btnProjectDefaults.Enabled := pmGetDefaults.Items.Count > 0;
    btnLibDefaults.Enabled := pmGetDefaults.Items.Count > 0;
  finally
    DirsList.Free;
  end;
end;

procedure TfrmOptions.btnProjectDefaultsClick(Sender: TObject);
var
  Pt: TPoint;
begin
  FMemoDirs := ButtonToMemo(Sender as TButton);
  getCursorPos(Pt);
  pmGetDefaults.Popup(Pt.X, Pt.Y);
end;

function TfrmOptions.ButtonToMemo(AButton: TButton): TMemo;
begin
  Result := nil;
  if AButton = btnProjectDefaults then
    Result := memProjectDirs
  else if AButton = btnLibDefaults then
    Result := memLibDirs
  else
    Assert(False);
end;

procedure TfrmOptions.btnProjectBrowseClick(Sender: TObject);
var
  i: integer;
  sFolder: string;
  slSubFolders: TStringList;
begin
  inherited;
  FMemoDirs := ButtonToMemo(Sender as TButton);
  if ofdOpenFolder.Execute then
  begin
    sFolder := ExtractFilePath(ofdOpenFolder.FileName);
    FMemoDirs.Lines.Add(sFolder);

    if (MessageDlg('Do you want to add all subfolders?', mtConfirmation, [mbYes, mbNo], 0) = mrYes) then
    begin
      slSubFolders := TStringList.Create;
      try
        GetSubFolders(sFolder, slSubFolders);

        for i := 0 to slSubFolders.Count - 1 do
          FMemoDirs.Lines.Add(slSubFolders[i]);

      finally
        slSubFolders.Free;
      end;
    end;
  end;
end;

procedure TfrmOptions.btnLibDefaultsClick(Sender: TObject);
var
  Pt: TPoint;
begin
  FMemoDirs := ButtonToMemo(Sender as TButton);
  getCursorPos(Pt);
  pmGetDefaults.Popup(Pt.X, Pt.Y);
end;

procedure TfrmOptions.btnLibBrowseClick(Sender: TObject);
var
  i: integer;
  sFolder: string;
  slSubFolders: TStringList;
begin
  inherited;
  FMemoDirs := ButtonToMemo(Sender as TButton);
  if ofdOpenFolder.Execute then
  begin
    sFolder := ExtractFilePath(ofdOpenFolder.FileName);
    FMemoDirs.Lines.Add(sFolder);

    if (MessageDlg('Do you want to add all subfolders?', mtConfirmation, [mbYes, mbNo], 0) = mrYes) then
    begin
      slSubFolders := TStringList.Create;
      try
        GetSubFolders(sFolder, slSubFolders);

        for i := 0 to slSubFolders.Count - 1 do
          FMemoDirs.Lines.Add(slSubFolders[i]);

      finally
        slSubFolders.Free;
      end;
    end;
  end;
end;

procedure TfrmOptions.btnFileBrowseClick(Sender: TObject);
begin
  inherited;
  if ofdOpenFile.Execute then
    edtFileToAnalyze.Text := ofdOpenFile.FileName;
end;

end.
