(*  Copyright (c) 2002, the CodeLens project
  All rights reserved.

  Redistribution and use in source and binary forms,
  with or without modification, are permitted provided
  that the following conditions are met:

  Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.

  Redistributions in binary form must reproduce the
  above copyright notice, this list of conditions
  and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
  Neither the name of the CodeLens project nor the names
  of its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS
  AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. *)

unit ucwClassTypesReport;

interface
uses
  Classes, ucwClasses, ucwReport;

type
  TRPTClassTypes = class(TcwReport, IcwReport)
  private
    FOptions : IcwSetting;

  public
    function GetName : string;override;
    procedure CreateColumns;override;

    procedure CreateOptions;override;
    function GetHasDetails : Boolean;override;
      //run the report, so it will be filled with Data
    procedure DoExecute;override;
  end;

procedure AddRPTClassTypes;


implementation


function TRPTClassTypes.GetName : string;
begin
  Result := 'Class Types';
end;


procedure TRPTClassTypes.CreateColumns;
begin
  Columns.AddColumn('Unit',dtString);
  Columns.AddColumn('Class Name',dtString);
  Columns.AddColumn('Used Count',dtInteger);
end;

procedure TRPTClassTypes.CreateOptions;
begin
  FOptions := Settings.AddSetting('Process Classes', True, True);
end;

function TRPTClassTypes.GetHasDetails : Boolean;
begin
  Result := True;
end;

//run the report, so it will be filled with Data
procedure TRPTClassTypes.DoExecute;
var i, j : Integer;
var vUnit   : TcwUnit;
var vClass  : TcwClass; 
begin
  for i := 0 to gcwProject.AllUnits.Count - 1 do
  begin
    vUnit := gcwProject.AllUnits[i];
    for j := 0 to vUnit.Classes.Count - 1 do
    begin
      vClass := vUnit.Classes[j];
      Data.AddRow(vUnit, FOptions, vClass, vClass.UsedBy, [vUnit.Name, vClass.Name, vClass.UsedBy.Count]);
    end;
  end;
end;


procedure AddRPTClassTypes;
begin
  gcwProject.Reports.AddReport(TRPTClassTypes.Create);
end;
end.
