(*  Copyright (c) 2002, the CodeLens project
  All rights reserved.

  Redistribution and use in source and binary forms,
  with or without modification, are permitted provided
  that the following conditions are met:

  Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.

  Redistributions in binary form must reproduce the
  above copyright notice, this list of conditions
  and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
  Neither the name of the CodeLens project nor the names
  of its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS
  AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. *)

unit ucwClasses;

interface

uses
  Classes,
  Contnrs,
  Types,
  mwPasLexTypes;

type
  TVisibilityKind = (
    vkNone, vkInterface, vkImplementation, vkPublic, vkAutomated, vkPublished, vkProtected,
    vkStrictProtected, vkPrivate, vkStrictPrivate);

  TVisibilityKindSet = set of TVisibilityKind;

  TcwTypeKind = (tkNone, tkAnsiString, tkArray, tkArrayOfConst, tkBoolean, tkByte, tkBytebool,
    tkCardinal, tkChar, tkClass, tkComp, tkCurrency, tkDouble, tkDWord, tkExtended, tkFile,
    tkFunction, tkInt64, tkInteger, tkInterface, tkLongBool, tkLongInt, tkLongWord, tkOleVariant,
    tkPacked, tkPChar, tkPointer, tkProcedure, tkReal, tkReal48, tkRecord, tkRoutine, tkSet,
    tkShortInt, tkShortString, tkSingle, tkSmallInt, tkString, tkText, tkVariant,
    tkWideChar, tkWideString, tkWord, tkWordbool, tkObject, tkEnum, tkTextFile);

  TcwDelphiVersion = (dvDelphi4, dvDelphi5, dvDelphi6, dvDelphi7, dvDelphi8,
    dvDelphi2005, dvDelphi2006);

const
  {  TOrdinalTypes = [tkBoolean, tkByte, tkBytebool, tkCardinal, tkChar, tkDWord, tkInt64,
                    tkInteger, tkLongBool, tkLongInt, tkLongWord, tkPChar, tkShortInt,
                    tkSmallInt, tkWideChar, tkWord, tkWordbool];}
  {  TFloatTypes = [tkReal48, tkReal, tkSingle, tkDouble, tkExtended, tkCurrency, tkComp];}
  TStringTypes = [tkString, tkAnsiString, tkShortString, tkWideString];
  TVariantTypes = [tkOleVariant, tkVariant];

type
  TRoutineKind = (rkProcedure, rkFunction, rkConstructor, rkDestructor);

  TDirectoryType = (dtProject, dtLibrary);

  TParserState = (psrNone, psrInterfaceParsing, psrInterfaceParsed, psrFull);
  TParseState = (psNone, psClassheritage, psInterfaceHeritage);

  TcwDataType = (dtString, dtInteger, dtDouble, dtBoolean);

  TRoutineDirective = (rdNone, rdAbstract, rdCdecl, rdPascal, rdAssembler, rdRegister, rdSafeCall,
    rdStdCall, rdExport, rdFar, rdNear, rdExternal, rdDynamic, rdMessage, rdOverload, rdOverride,
    rdReintroduce, rdVirtual, rdLocal);

  TRoutineDirectives = set of TRoutineDirective;

  TParamKind = (pkNone, pkConst, pkVar, pkIn, pkOut);

  TcwUnit = class;

  TcwCodeLocation = class
  private
    FEndX: Integer;
    FEndY: Integer;
    FFileIndex: Integer;
    FUnitIndex: Integer;
    FX: Integer;
    FY: Integer;
  protected
    function GetEndX: Integer;
    function GetEndY: Integer;
    function GetFileIndex: Integer;
    function GetFileName: string;
    function GetUnitIndex: Integer;
    function GetUnitIntf: TcwUnit;
    function GetX: Integer;
    function GetY: Integer;
    procedure SetEndX(const aValue: Integer);
    procedure SetEndY(const aValue: Integer);
    procedure SetX(const aValue: Integer);
    procedure SetY(const aValue: Integer);
  public
    constructor Create(const AFileIndex: Integer; const aX, aY: Integer); virtual;
    function ComparesTo(const ACodeLocation: TcwCodeLocation): Boolean;
    property EndX: Integer read GetEndX write SetEndX;
    property EndY: Integer read GetEndY write SetEndY;
    property FileIndex: Integer read GetFileIndex;
    property FileName: string read GetFileName;
    property UnitIndex: Integer read GetUnitIndex;
    property UnitIntf: TcwUnit read GetUnitIntf;
    property X: Integer read GetX write SetX;
    property Y: Integer read GetY write SetY;
  end;

  TcwUsedByList = class;

  TcwUsedByInfo = record
    FileIndex: Integer;
    IsWrongCase: Boolean;
    X, Y: Integer
  end;

  TcwUsedByLocation = class(TcwCodeLocation)
  private
    FIsWrongCase: Boolean;
    FUsedByList: TcwUsedByList;
  protected
    function GetIsWrongCase: Boolean;
    function GetUsedByList: TcwUsedByList;
  public
    constructor Create(const AUsedByList: TcwUsedByList; const AUsedByInfo: TcwUsedByInfo);
      reintroduce;
    property IsWrongCase: Boolean read GetIsWrongCase;
    property UsedByList: TcwUsedByList read GetUsedByList;
  end;

  TcwList = class
  private
    FList: TObjectList;
    function GetItem(Index: Integer): TObject;
    procedure SetItem(Index: Integer; const Value: TObject);
    function GetCount: Integer;
  protected
    function GetOwnsObjects: Boolean; virtual;
    function Add(const AObject: TObject): Integer;
    property Items[Index: Integer]: TObject read GetItem write SetItem;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Clear; virtual;
    property Count: Integer read GetCount;
  end;

  TcwHashedObjectList = class
  private
    FHashStringList: TmwAssignAbleHash;
    function GetString(Index: Integer): string;
    procedure SetObject(Index: Integer; const Value: TObject);
    function GetObject(Index: Integer): TObject;
    property Objects[Index: Integer]: TObject read GetObject write SetObject; default;
    property Strings[Index: Integer]: string read GetString;
  public
    constructor Create;
    destructor Destroy; override;
    function Add(const AString: string; const AObject: TObject): Integer;
    procedure Clear;
    function Count: Integer;
    procedure Delete(Index: Integer);
    function Find(const AString: string; out AIndex: Integer): Boolean;
    procedure Sort;
  end;

  TcwListSorted = class(TcwList)
  private
    FSortedList: TcwHashedObjectList;
    FStringList: TStringList;
    function GetSortedItem(Index: Integer): TObject;
    procedure LoadStringList;
  protected
    function DoItemAdded(const AName: string; AObject: TObject): Integer;
    function Find(AName: string): TObject;
    property SortedItem[Index: Integer]: TObject read GetSortedItem;
  public
    destructor Destroy; override;
    procedure Clear; override;
  end;

  TcwVarContainer = class;

  TcwContainerList = class(TcwList)
  private
    FContainer: TcwVarContainer;
  protected
    function GetContainer: TcwVarContainer;
    function GetOwnsObjects: Boolean; override;
  public
    constructor Create(const AContainer: TcwVarContainer);
    property Container: TcwVarContainer read GetContainer;
  end;

  TcwType = class;
  TcwSymbol = class;

  TcwContainerListSorted = class(TcwContainerList)
  private
    FSortedList: TcwHashedObjectList;
    FStringList: TStringList;
    function InternalSymbolAdded(const ASymbol: TcwSymbol): Integer;
    function GetSortedItem(Index: Integer): TObject;
    procedure LoadStringList;
  protected
    function DoSymbolAdded(const ASymbol: TcwSymbol): Integer; virtual;
    function DoTypeAdded(const ASymbol: TcwSymbol): Integer;
    function Find(const aName: string): TObject;
    property SortedItem[Index: Integer]: TObject read GetSortedItem;
  public
    destructor Destroy; override;
  end;

  TcwVisiblityCodeLocation = packed record
    CodeLine: Integer;
    Visibility: TVisibilityKind;
  end;

  TcwSymbolContainer = class;
  TcwClass = class;

  TcwSymbol = class(TcwCodeLocation)
  private
    FName: string;
    FPointerType: Boolean;
    FPointerTypeName: string;
    FPredefinedArrayType: TcwSymbol;
    FType: TcwSymbol;
    FUsedBy: TcwUsedByList;
    FVisibility: TVisibilityKind;
    FTypeKind: TcwTypeKind;
  protected
    function GetTypeIntf: TcwSymbol; virtual;
    procedure SetTypeIntf(const aValue: TcwSymbol); virtual;
    function GetName: string;
    function GetPointerType: Boolean;
    function GetPointerTypeName: string;
    function GetPredefinedArrayType: TcwSymbol;
    function GetTypeAssigned: Boolean;
    function GetTypeKind: TcwTypeKind; virtual;
    function GetUsedBy: TcwUsedByList;
    function GetVisibility: TVisibilityKind;
    procedure SetPointerType(const Value: Boolean);
    procedure SetPointerTypeName(const Value: string);
    procedure SetPredefinedArrayType(const Value: TcwSymbol);
    procedure SetTypeKind(const aValue: TcwTypeKind);
    procedure SetUsedBy(const aValue: TcwUsedByList);
  public
    constructor Create(const aName: string; aVisibility: TVisibilityKind; aTypeKind: TcwTypeKind;
      const AFileIndex: Integer; const aX, aY: Integer); reintroduce;
    destructor Destroy; override;
    function ComparesTo(const ASymbol: TcwSymbol): Boolean;
    function FindContainer: TcwSymbolContainer;
    function FindSymbol(const AFromUnit: TcwUnit; const aFromClass: TcwClass;
      const aName: string; const AAddUsedBy: Boolean; AUsedByInfo: TcwUsedByInfo): TcwSymbol;
      overload; virtual;
    function FindSymbol(const AFromUnit: TcwUnit; const aFromClass: TcwClass;
      const aName: string): TcwSymbol; overload; virtual;
    function TypeRequired: Boolean; virtual;
    property Name: string read GetName;
    property PointerType: Boolean read GetPointerType write SetPointerType;
    property PointerTypeName: string read GetPointerTypeName write SetPointerTypeName;
    property PredefinedArrayType: TcwSymbol read GetPredefinedArrayType write
      SetPredefinedArrayType;
    property TypeAssigned: Boolean read GetTypeAssigned;
    property UsedBy: TcwUsedByList read GetUsedBy write SetUsedBy;
    property Visibilty: TVisibilityKind read GetVisibility;
    property TypeIntf: TcwSymbol read GetTypeIntf write SetTypeIntf;
    property TypeKind: TcwTypeKind read GetTypeKind write SetTypeKind;
  end;

  TcwSymbolList = class(TcwList)
  private
    FOwnsObjects: Boolean;
    FSortedList: TcwHashedObjectList;
  protected
    function GetSortedSymbols(Index: Integer): TcwSymbol;
    function GetSymbol(Index: Integer): TcwSymbol;
    function GetOwnsObjects: Boolean; override;
  public
    constructor Create(const AOwnsSymbols: Boolean = True);
    destructor Destroy; override;
    procedure Clear; override;
    function AddSymbol(const aItem: TcwSymbol): Integer; overload;
    function AddSymbol(const aName: string; aVisibility: TVisibilityKind; const AFileIndex: Integer;
      const aX, aY: Integer): TcwSymbol; overload;
    function Find(const aSymbolName: string): TcwSymbol;
    property SortedSymbols[Index: Integer]: TcwSymbol read GetSortedSymbols;
    property Symbols[Index: Integer]: TcwSymbol read GetSymbol; default;
  end;

  TcwCodeStatistics = class
  private
    FAnsiCommentLines: Integer;
    FBorCommentLines: Integer;
    FCodeLines: Integer;
    FEmptyLines: Integer;
    FSlashCommentLines: Integer;
  public
    function GetAnsiCommentLines: Integer;
    function GetBorCommentLines: Integer;
    function GetCodeLines: Integer;
    function GetCommentLines: Integer;
    function GetEmptyLines: Integer;
    function GetSlashCommentLines: Integer;
    function GetTotalLines: Integer;
    procedure IncAnsiCommentLines;
    procedure IncBorCommentLines;
    procedure IncCodeLines;
    procedure IncEmptyLines;
    procedure IncSlashCommentLines;
    property AnsiCommentLines: Integer read GetAnsiCommentLines;
    property BorCommentLines: Integer read GetBorCommentLines;
    property CodeLines: Integer read GetCodeLines;
    property CommentLines: Integer read GetCommentLines;
    property EmptyLines: Integer read GetEmptyLines;
    property SlashCommentLines: Integer read GetSlashCommentLines;
    property TotalLines: Integer read GetTotalLines;
  end;

  TcwSymbolContainer = class(TcwSymbol)
  private
    FHashedSymbols: TcwHashedObjectList;
  protected
    function GetSymbol(Index: Integer): TcwSymbol;
    function GetSymbolCount: Integer;
    property HashedSymbols: TcwHashedObjectList read FHashedSymbols;
  public
    constructor Create(const aName: string; const aVisibility: TVisibilityKind;
      const AFileIndex: Integer; const aX, aY: Integer);
    destructor Destroy; override;
    procedure AddSymbol(const aSymbol: TcwSymbol);
    function FindSymbol(const AFromUnit: TcwUnit; const aFromClass: TcwClass;
      const aName: string; const AAddUsedBy: Boolean; AUsedByInfo: TcwUsedByInfo): TcwSymbol;
      override;
    property SymbolCount: Integer read GetSymbolCount;
    property Symbols[Index: Integer]: TcwSymbol read GetSymbol;
  end;

  TcwUsedByList = class(TcwList)
  private
    FSymbol: TcwSymbol;
  protected
    function GetLocations(Index: Integer): TcwUsedByLocation;
  public
    constructor Create(const ASymbol: TcwSymbol);
    procedure AddUsedBy(const AUsedByInfo: TcwUsedByInfo);
    property Locations[Index: Integer]: TcwUsedByLocation read GetLocations; default;
    property Symbol: TcwSymbol read FSymbol;
  end;

  TcwType = class(TcwSymbol)
  private
    FTypeIntf: TcwSymbol;
    FTypeRequired: Boolean;
  protected
    function GetTypeIntf: TcwSymbol; override;
    procedure SetTypeIntf(const aValue: TcwSymbol); override;
  public
    constructor Create(const aName: string; aTypeKind: TcwTypeKind; aVisibility: TVisibilityKind;
      const AFileIndex: Integer; const aX, aY: Integer; ATypeRequired: Boolean);
    function TypeRequired: Boolean; override;
  end;

  TcwTypesList = class(TcwContainerListSorted)
  private
    FUnitName: string;
  protected
    function GetSortedTypes(Index: Integer): TcwSymbol;
    function GetTypes(Index: Integer): TcwSymbol;
    function DoSymbolAdded(const ASymbol: TcwSymbol): Integer; override;
  public
    constructor Create(const aContainer: TcwVarContainer);
    function AddType(const aType: TcwSymbol): Integer; overload;
    function AddType(const aTypeName: string; aVisibility: TVisibilityKind;
      const AFileIndex: Integer; const aX, aY: Integer; ATypeRequired: Boolean): TcwSymbol;
      overload;
    function Find(const aTypeName: string): TcwSymbol;
    property SortedTypes[Index: Integer]: TcwSymbol read GetSortedTypes;
    property Types[Index: Integer]: TcwSymbol read GetTypes; default;
  end;

  TcwVariable = class(TcwSymbol)
  private
    FIsConst: Boolean;
    FIsThreadVar: Boolean;
    FVarType: TcwSymbol;
  protected
    function GetTypeIntf: TcwSymbol; override;
    procedure SetTypeIntf(const aValue: TcwSymbol); override;
    function GetIsConst: Boolean;
    function GetIsThreadVar: Boolean;
    procedure SetIsThreadVar(const aValue: Boolean);
  public
    constructor Create(const aName: string; aVisibility: TVisibilityKind; aIsConst: Boolean; const
      AFileIndex: Integer; const aX, aY: Integer);
    function TypeRequired: Boolean; override;
    property IsConst: Boolean read GetIsConst;
    property IsThreadVar: Boolean read GetIsThreadVar write SetIsThreadVar;
  end;

  TcwVariablesList = class(TcwContainerListSorted)
  protected
    function GetSortedVariables(Index: Integer): TcwVariable;
    function GetVariables(Index: Integer): TcwVariable;
  public
    function AddVariable(const aVarName: string; aVisibility: TVisibilityKind; aIsConst: Boolean;
      const AFileIndex: Integer; const aX, aY: Integer): TcwVariable;
    function Find(AFieldName: string): TcwVariable;
    property Variables[Index: Integer]: TcwVariable read GetVariables; default;
    property SortedVariables[Index: Integer]: TcwVariable read GetSortedVariables;
  end;

  TcwEnumerationMemberContainer = class(TcwSymbolContainer)
  protected
    function GetItems: TcwSymbolList; virtual; abstract;
  public
    function TypeRequired: Boolean; override;
    property Items: TcwSymbolList read GetItems;
  end;

  TcwEnumeration = class(TcwEnumerationMemberContainer)
  private
    FItems: TcwSymbolList;
  protected
    function GetItems: TcwSymbolList; override;
    function GetTypeKind: TcwTypeKind; override;
  public
    constructor Create(const aName: string; aVisibility: TVisibilityKind; const AFileIndex: Integer;
      const aX, aY: Integer);
    destructor Destroy; override;
  end;

  TcwEnumerationList = class(TcwContainerList)
  protected
    function GetEnumeration(Index: Integer): TcwEnumeration;
    function GetOwnsObjects: Boolean; override;
  public
    procedure AddItem(const AItem: TcwEnumeration); overload;
    function AddItem(const aEnumName: string; aVisibility: TVisibilityKind;
      const AFileIndex: Integer; const aX, aY: Integer): TcwEnumeration; overload;
    property Enumerations[Index: Integer]: TcwEnumeration read GetEnumeration; default;
  end;

  TcwSet = class(TcwEnumerationMemberContainer)
  private
    FItems: TcwSymbolList;
    FEnumeration: TcwEnumeration;
  protected
    function GetEnumeration: TcwEnumeration;
    function GetTypeKind: TcwTypeKind; override;
    procedure SetEnumeration(Value: TcwEnumeration);
    function GetItems: TcwSymbolList; override;
  public
    destructor Destroy; override;
    property Enumeration: TcwEnumeration read GetEnumeration write SetEnumeration;
  end;

  TcwSetList = class(TcwContainerList)
  protected
    function GetSet(Index: Integer): TcwSet;
    function GetOwnsObjects: Boolean; override;
  public
    procedure AddItem(const AItem: TcwSet); overload;
    function AddItem(const aSetName: string; aVisibility: TVisibilityKind; const AFileIndex: Integer;
      const aX, aY: Integer): TcwSet; overload;
    property Sets[Index: Integer]: TcwSet read GetSet; default;
  end;

  TcwArray = class(TcwType)
  private
    FArrayType: TcwSymbol;
    FDimensions: TObjectList;
  protected
    function GetTypeIntf: TcwSymbol; override;
    function GetArrayType: TcwSymbol;
    function GetDimensionCount: Integer;
    function GetDimensions(Index: Integer): TcwSymbol;
    procedure SetArrayType(const aType: TcwSymbol);
  public
    constructor Create(const aName: string; aVisibility: TVisibilityKind;
      const AFileIndex, aX, aY: Integer);
    destructor Destroy; override;
    procedure AddDimension(const aSymbol: TcwSymbol);
    function TypeRequired: Boolean; override;
    property ArrayType: TcwSymbol read GetArrayType write SetArrayType;
    property DimensionCount: Integer read GetDimensionCount;
    property Dimensions[Index: Integer]: TcwSymbol read GetDimensions;
  end;

  TcwArrayList = class(TcwContainerList)
  protected
    function GetArray(Index: Integer): TcwArray;
    function GetOwnsObjects: Boolean; override;
  public
    function AddArray(const aArrayName: string; aVisibility: TVisibilityKind; const AFileIndex: Integer;
      const aX, aY: Integer): TcwArray;
    property Arrays[Index: Integer]: TcwArray read GetArray; default;
  end;

  TcwLabel = class(TcwSymbol)
  private
    FIsIntLabel: Boolean;
  protected
    function GetIsIntLabel: Boolean;
  public
    constructor Create(const aName: string; aIsIntLabel: Boolean; const AFileIndex: Integer; const aX,
      aY: Integer);
    function TypeRequired: Boolean; override;
    property IsIntLabel: Boolean read GetIsIntLabel;
  end;

  TcwLabelsList = class(TcwList)
  protected
    function GetLabels(Index: Integer): TcwLabel;
  public
    function AddLabel(const aLabelName: string; aIsIntLabel: Boolean; const AFileIndex: Integer; const
      aX, aY: Integer): TcwLabel;
    property Labels[Index: Integer]: TcwLabel read GetLabels; default;
  end;

  TcwClassList = class;
  TcwInterfaceList = class;
  TcwRecordsList = class;
  TcwRoutineList = class;

  TcwVarContainer = class(TcwSymbolContainer)
  private
    FEnumerationList: TcwEnumerationList;
    FArraysList: TcwArrayList;
    FConstantList: TcwVariablesList;
    FLabels: TcwLabelsList;
    FRecordList: TcwRecordsList;
    FSetList: TcwSetList;
    FTypes: TcwTypesList;
    FPointerTypeList: TcwHashedObjectList;
    FVariableList: TcwVariablesList;
    FClassList: TcwClassList;
    FObjectList: TcwClassList;
    FInterfaceList: TcwInterfaceList;
    FRoutineList: TcwRoutineList;
  protected
    function GetEnumerations: TcwEnumerationList;
    function GetArrays: TcwArrayList;
    function GetClasses: TcwClassList;
    function GetConstants: TcwVariablesList;
    function GetInterfaces: TcwInterfaceList;
    function GetLabels: TcwLabelsList;
    function GetObjects: TcwClassList;
    function GetRecords: TcwRecordsList;
    function GetRoutines: TcwRoutineList;
    function GetSets: TcwSetList;
    function GetTypes: TcwTypesList;
    function GetVariables: TcwVariablesList;
  public
    destructor Destroy; override;
    function FindForwardPointerType(const AName: string): TcwType;
    procedure AddForwardPointerType(AType: TcwType);
    property Arrays: TcwArrayList read GetArrays;
    property Classes: TcwClassList read GetClasses;
    property Constants: TcwVariablesList read GetConstants;
    property Interfaces: TcwInterfaceList read GetInterfaces;
    property Labels: TcwLabelsList read GetLabels;
    property Objects: TcwClassList read GetObjects;
    property Records: TcwRecordsList read GetRecords;
    property Routines: TcwRoutineList read GetRoutines;
    property Sets: TcwSetList read GetSets;
    property Types: TcwTypesList read GetTypes;
    property Variables: TcwVariablesList read GetVariables;
    property Enumerations: TcwEnumerationList read GetEnumerations;
  end;

  TcwRecord = class(TcwVarContainer)
  private
    FList: TcwVariablesList;
  protected
    function GetTypeIntf: TcwSymbol; override;
    procedure SetTypeIntf(const aValue: TcwSymbol); override;
    function GetFieldList: TcwVariablesList;
    function GetTypeKind: TcwTypeKind; override;
    procedure SetTypeKind(const aValue: TcwTypeKind);
  public
    constructor Create(const aName: string; aVisibility: TVisibilityKind; const AFileIndex: Integer;
      const aX, aY: Integer);
    destructor Destroy; override;
    function TypeRequired: Boolean; override;
    property Fields: TcwVariablesList read GetFieldList;
    property TypeKind: TcwTypeKind read GetTypeKind write SetTypeKind;
  end;

  TcwRecordsList = class(TcwContainerListSorted)
  protected
    function GetRecords(Index: Integer): TcwRecord;
    function GetSortedRecords(Index: Integer): TcwRecord;
    function GetOwnsObjects: Boolean; override;
  public
    function AddRecord(const aRecordName: string; aVisibility: TVisibilityKind; const AFileIndex:
      Integer; const aX, aY: Integer): TcwRecord;
    function Find(const aRecordName: string): TcwRecord;
    property Records[Index: Integer]: TcwRecord read GetRecords; default;
    property SortedRecords[Index: Integer]: TcwRecord read GetSortedRecords;
  end;

  TcwProperty = class(TcwVariable)
  private
    FDefaultProperty: Boolean;
    FIsArrayProperty: Boolean;
    FOwnerClass: TcwClass;
    FReadAccess: TcwSymbol;
    FWriteAccess: TcwSymbol;
  protected
    function GetDefaultProperty: Boolean;
    function GetIsArrayProperty: Boolean;
    function GetOwnerClass: TcwClass;
    function GetReadAccess: TcwSymbol;
    function GetWriteAccess: TcwSymbol;
    procedure SetDefaultProperty(const Value: Boolean);
    procedure SetIsArrayProperty(const Value: Boolean);
    procedure SetReadAccess(const aValue: TcwSymbol);
    procedure SetWriteAccess(const aValue: TcwSymbol);
  public
    constructor Create(const aName: string; aVisibility: TVisibilityKind; const AOwnerClass: TcwClass;
      const AFileIndex, aX, aY: Integer);
    property DefaultProperty: Boolean read GetDefaultProperty write SetDefaultProperty;
    property IsArrayProperty: Boolean read GetIsArrayProperty write SetIsArrayProperty;
    property OwnerClass: TcwClass read GetOwnerClass;
    property ReadAccess: TcwSymbol read GetReadAccess write SetReadAccess;
    property WriteAccess: TcwSymbol read GetWriteAccess write SetWriteAccess;
  end;

  TcwPropertyList = class(TcwContainerListSorted)
  protected
    function GetProperties(Index: Integer): TcwProperty;
    function GetSortedProperties(Index: Integer): TcwProperty;
  public
    function AddProperty(const aPropertyName: string; aVisibility: TVisibilityKind; const AOwnerClass:
      TcwClass; const AFileIndex: Integer; const aX, aY: Integer): TcwProperty;
    function Find(const aPropName: string): TcwProperty;
    property Properties[Index: Integer]: TcwProperty read GetProperties; default;
    property SortedProperties[Index: Integer]: TcwProperty read GetSortedProperties;
  end;

  TcwClass = class(TcwVarContainer)
  private
    FDefaultProperty: TcwProperty;
    FDescendants: TObjectList;
    FInheritsFrom: TcwClass;
    FPropertyList: TcwPropertyList;
    FVisibilitySpecifiers: array of TcwVisiblityCodeLocation;
    FVisibilitySpecifiersCount: Integer;
    FVisibilitySpecifiersSize: Integer;
  protected
    function GetTypeIntf: TcwSymbol; override;
    procedure SetTypeIntf(const aValue: TcwSymbol); override;
    function GetDefaultProperty: TcwProperty;
    function GetDefaultVisibility: TVisibilityKind; virtual;
    function GetDescendants: TObjectList;
    function GetFieldList: TcwVariablesList;
    function GetInheritsFromClass: TcwClass;
    function GetProperties: TcwPropertyList;
    function GetTypeKind: TcwTypeKind; override;
    function GetVisibilitySpecifierCount: Integer;
    function GetVisibilitySpecifierLine(aIndex: Integer): Integer;
    function GetVisibilitySpecifierVisibility(aIndex: Integer): TVisibilityKind;
    procedure SetDefaultProperty(const Value: TcwProperty);
    procedure SetInheritsFromClass(const aValue: TcwClass);
    procedure SetTypeKind(const aValue: TcwTypeKind);
  public
    constructor Create(const aName: string; const aVisibility: TVisibilityKind;
      const AFileIndex: Integer; const aX, aY: Integer);
    destructor Destroy; override;
    procedure AddVisibilitySpecifier(aCodeLocationLine: Integer; aVisibility: TVisibilityKind);
    function FindSymbol(const AFromUnit: TcwUnit; const aFromClass: TcwClass;
      const aName: string; const AAddUsedBy: Boolean; AUsedByInfo: TcwUsedByInfo): TcwSymbol;
      override;
    procedure GetMembersWithVisibility(out aMemberList: TcwSymbolList; aScope: TVisibilityKindSet);
    function IsDescendant(const aClass: TcwClass): Boolean;
    function IsSymbolFromInterface(const aSymbolName: string): Boolean;
    function TypeRequired: Boolean; override;
    property DefaultProperty: TcwProperty read GetDefaultProperty write SetDefaultProperty;
    property DefaultVisibility: TVisibilityKind read GetDefaultVisibility;
    property Descendants: TObjectList read GetDescendants;
    property Fields: TcwVariablesList read GetFieldList;
    property InheritsFromClass: TcwClass read GetInheritsFromClass write SetInheritsFromClass;
    property Properties: TcwPropertyList read GetProperties;
    property TypeKind: TcwTypeKind read GetTypeKind write SetTypeKind;
    property VisibilitySpecifierLine[aIndex: Integer]: Integer read GetVisibilitySpecifierLine;
    property VisibilitySpecifiersCount: Integer read GetVisibilitySpecifierCount;
    property VisibilitySpecifierVisibility[aIndex: Integer]: TVisibilityKind read
      GetVisibilitySpecifierVisibility;
  end;

  TcwClassList = class(TcwContainerListSorted)
  protected
    function GetClasses(Index: Integer): TcwClass;
    function GetSortedClasses(Index: Integer): TcwClass;
    function GetOwnsObjects: Boolean; override;
  public
    function AddClass(const aName: string; const aVisibility: TVisibilityKind;
      const AFileIndex: Integer; const aX, aY: Integer): TcwClass; overload; virtual; 
    procedure AddClass(const aClass: TcwClass); overload; virtual;
    function Find(const aClassName: string): TcwClass;
    property Classes[Index: Integer]: TcwClass read GetClasses; default;
    property SortedClasses[Index: Integer]: TcwClass read GetSortedClasses;
  end;

  TcwOwnedClassList = class(TcwClassList)
  protected
    function GetOwnsObjects: Boolean; override;
  public
    procedure AddClass(const aClass: TcwClass); overload; override;
  end;

  TcwObject = class(TcwClass)
  protected
    function GetTypeKind: TcwTypeKind; override;
  public
    function GetDefaultVisibility: TVisibilityKind; override;
  end;

  TcwObjectList = class(TcwClassList)
  public
    function AddClass(const aName: string; const aVisibility: TVisibilityKind;
      const AFileIndex: Integer; const aX, aY: Integer): TcwClass; overload; override;
  end;

  TcwOwnedObjectList = class(TcwOwnedClassList)
  public
    function AddClass(const aName: string; const aVisibility: TVisibilityKind;
      const AFileIndex: Integer; const aX, aY: Integer): TcwClass; overload; override;
  end;

  TcwInterface = class(TcwClass)
  private
    FGUID: string;
    FIsDispatchInterface: Boolean;
  protected
    function GetGUID: string;
    function GetIsDispatchInterface: Boolean;
    procedure SetGUID(const aValue: string);
    procedure SetIsDispatchInterface(const aValue: Boolean);
  public
    function GetDefaultVisibility: TVisibilityKind; override;
    function GetTypeKind: TcwTypeKind; override;
    property GUID: string read GetGUID write SetGUID;
    property IsDispatchInterface: Boolean read GetIsDispatchInterface write SetIsDispatchInterface;
  end;

  TcwInterfaceList = class(TcwContainerListSorted)
  protected
    function GetInterfaces(Index: Integer): TcwInterface;
    function GetSortedInterfaces(Index: Integer): TcwInterface;
    function GetOwnsObjects: Boolean; override;
  public
    procedure AddInterface(const aInterface: TcwInterface); overload; virtual;
    function AddInterface(const aName: string; const aVisibility: TVisibilityKind;
      const AFileIndex: Integer; const aX, aY: Integer): TcwInterface; overload;
    function Find(const aInterfaceName: string): TcwInterface;
    property Interfaces[Index: Integer]: TcwInterface read GetInterfaces; default;
    property SortedInterfaces[Index: Integer]: TcwInterface read GetSortedInterfaces;
  end;

  TcwOwnedInterfaceList = class(TcwInterfaceList)
  protected
    function GetOwnsObjects: Boolean; override;
  public
    procedure AddInterface(const aInterface: TcwInterface); override;
  end;

  TcwParameter = class(TcwSymbol)
  private
    FParamKind: TParamKind;
    FParamType: TcwSymbol;
    FUntypedParameter: Boolean;
  protected
    function GetTypeIntf: TcwSymbol; override;
    function GetParamKind: TParamKind;
    function GetParamType: TcwSymbol;
    function GetUntypedParameter: Boolean;
    procedure SetParamType(const aType: TcwSymbol);
    procedure SetUntypedParameter(const Value: Boolean);
  public
    constructor Create(const aParamName: string; aParamKind: TParamKind; const AFileIndex: Integer;
      const aX, aY: Integer);
    function TypeRequired: Boolean; override;
    property ParamKind: TParamKind read GetParamKind;
    property ParamType: TcwSymbol read GetParamType write SetParamType;
    property UntypedParameter: Boolean read GetUntypedParameter write SetUntypedParameter;
  end;

  TcwParameterList = class(TcwContainerListSorted)
  protected
    function GetParameters(Index: Integer): TcwParameter;
  public
    function AddParameter(const aParamName: string; aParamKind: TParamKind; const AFileIndex: Integer;
      const aX, aY: Integer): TcwParameter;
    function Find(const aParamName: string): TcwParameter;
    property Parameters[Index: Integer]: TcwParameter read GetParameters; default;
  end;

  TcwRoutine = class(TcwVarContainer)
  private
    FDirectives: TRoutineDirectives;
    FImplEnd: TPoint;
    FImplStart: TPoint;
    FParameters: TcwParameterList;
    FReturnType: TcwVariable;
    FRoutineKind: TRoutineKind;
    FRoutineList: TcwRoutineList;
    FRoutineType: TcwVariable;
  protected
    function GetTypeIntf: TcwSymbol; override;
    function GetDirectives: TRoutineDirectives;
    function GetImplEndX: Integer;
    function GetImplEndY: Integer;
    function GetImplStartX: Integer;
    function GetImplStartY: Integer;
    function GetIsUsedFromDFM: Boolean;
    function GetParameters: TcwParameterList;
    function GetReturnType: TcwVariable;
    function GetRoutineKind: TRoutineKind;
    function GetRoutineList: TcwRoutineList;
    function GetRoutineType: TcwVariable;
    procedure SetImplEndX(aValue: Integer);
    procedure SetImplEndY(aValue: Integer);
    procedure SetImplStartX(aValue: Integer);
    procedure SetImplStartY(aValue: Integer);
    procedure SetReturnType(const aValue: TcwVariable);
    procedure SetRoutineType(const aValue: TcwVariable);
  public
    constructor Create(const ARoutineList: TcwRoutineList; const aName: string; aVisibility:
      TVisibilityKind; aRoutineKind: TRoutineKind; const AFileIndex: Integer; const aX, aY: Integer);
    destructor Destroy; override;
    procedure AddDirective(const aDirective: TRoutineDirective);
    function TypeRequired: Boolean; override;
    property Directives: TRoutineDirectives read GetDirectives;
    property ImplEndX: Integer read GetImplEndX write SetImplEndX;
    property ImplEndY: Integer read GetImplEndY write SetImplEndY;
    property ImplStartX: Integer read GetImplStartX write SetImplStartX;
    property ImplStartY: Integer read GetImplStartY write SetImplStartY;
    property IsUsedFromDFM: Boolean read GetIsUsedFromDFM;
    property Parameters: TcwParameterList read GetParameters;
    property ReturnType: TcwVariable read GetReturnType write SetReturnType;
    property RoutineKind: TRoutineKind read GetRoutineKind;
    property RoutineList: TcwRoutineList read GetRoutineList;
    property RoutineType: TcwVariable read GetRoutineType write SetRoutineType;
  end;

  TcwRoutineList = class(TcwContainerListSorted)
  protected
    function GetRoutine(Index: Integer): TcwRoutine;
    function GetSortedRoutines(Index: Integer): TcwRoutine;
  public
    function AddRoutine(const aName: string; aVisibility: TVisibilityKind; aRoutineKind: TRoutineKind;
      const AFileIndex: Integer; const aX, aY: Integer): TcwRoutine;
    function Find(const aRoutineName: string): TcwRoutine;
    property Routines[Index: Integer]: TcwRoutine read GetRoutine; default;
    property SortedRoutines[Index: Integer]: TcwRoutine read GetSortedRoutines;
  end;

  TCodeErrorKind = (ekBadIndentation, ekBadSpacing, ekBadNewLine);

  TcwCodeError = class(TcwCodeLocation)
  private
    FErrorKind: TCodeErrorKind;
  protected
    function GetErrorKind: TCodeErrorKind;
  public
    constructor Create(const aError: TCodeErrorKind; const AFileIndex: Integer;
      const aX, aY: Integer); reintroduce;
    property ErrorKind: TCodeErrorKind read GetErrorKind;
  end;

  TcwCodingErrorsList = class
  private
    FList: TObjectList;
  protected
    function GetCount: Integer;
    function GetItems(Index: Integer): TcwCodeError;
  public
    constructor Create;
    destructor Destroy; override;
    function AddError(const aError: TCodeErrorKind; const AFileIndex: Integer; const aX, aY: Integer):
      TcwCodeError;
    procedure Clear;
    property Count: Integer read GetCount;
    property Items[Index: Integer]: TcwCodeError read GetItems; default;
  end;

  TcwUsedUnit = class
  private
    FUsedList: TcwUsedByList;
    FUnit: TcwUnit;
  public
    constructor Create(const AUnit: TcwUnit);
    destructor Destroy; override;
    property UsedList: TcwUsedByList read FUsedList;
    property UnitIntf: TcwUnit read FUnit;
  end;

  TcwUsedUnitList = class(TcwListSorted)
  private
    function GetUsedUnit(Index: Integer): TcwUsedUnit;
    function GetSortedUsedUnit(Index: Integer): TcwUsedUnit;
  public
    procedure AddUsedUnit(const AUsedUnit: TcwUsedUnit);
    property UsedUnits[Index: Integer]: TcwUsedUnit read GetUsedUnit; default;
    function FindUsedUnit(AUnitName: string): TcwUsedUnit;
    property SortedUsedUnits[Index: Integer]: TcwUsedUnit read GetSortedUsedUnit;
  end;

  TcwDFMResource = class;

  TcwUnitWarning = (uwInterface, uwImplementation);
  TcwUnitWarningSet = set of TcwUnitWarning;

  TcwUnit = class(TcwVarContainer)
  private
    FCodingErrorsList: TcwCodingErrorsList;
    FDFMFileName: string;
    FDFMResource: TcwDFMResource;
    FImplementationLine: Integer;
    FInterfaceLine: Integer;
    FParsed: TParserState;
    FStringLiteralsList: TcwSymbolList;
    FUsedUnitList: TcwUsedUnitList;
    FUnitType: TDirectoryType;
    FUsedByLocations: TObjectList;
    FUnitWarnings: TcwUnitWarningSet;
    function GetFullFileName: string;
  protected
    function GetCodingErrorsList: TcwCodingErrorsList;
    function GetDFMResource: TcwDFMResource;
    function GetImplementationLine: Integer;
    function GetInterfaceLine: Integer;
    function GetStringLiterals: TcwSymbolList;
    function GetUnitType: TDirectoryType;
    function GetUsedByLocations: TObjectList;
    function GetUsedUnits: TcwUsedUnitList;
    procedure SetImplementationLine(aValue: Integer);
    procedure SetInterfaceLine(aValue: Integer);
    procedure SetUnitType(const aValue: TDirectoryType);
  public
    constructor Create(const AFileIndex: Integer; const aName: string);
    destructor Destroy; override;
    function FindReferencedSymbol(aFromClass: TcwClass; const aName: string;
      const AAddUsedBy: Boolean; AUsedByInfo: TcwUsedByInfo): TcwSymbol;
    function FindSymbol(const AFromUnit: TcwUnit; const aFromClass: TcwClass;
      const aName: string; const AAddUsedBy: Boolean; AUsedByInfo: TcwUsedByInfo): TcwSymbol;
      override;
    function IsLibraryUnit: Boolean;
    procedure LoadFormResource(const aDFMFileName: string);
    function TypeRequired: Boolean; override;
    property CodingErrorsList: TcwCodingErrorsList read GetCodingErrorsList;
    property DFMResource: TcwDFMResource read GetDFMResource;
    property DFMFileName: string read FDFMFileName;
    property FullFileName: string read GetFullFileName;
    property ImplementationLine: Integer read GetImplementationLine write SetImplementationLine;
    property InterfaceLine: Integer read GetInterfaceLine write SetInterfaceLine;
    property Parsed: TParserState read FParsed write FParsed;
    property StringLiterals: TcwSymbolList read GetStringLiterals;
    property UnitType: TDirectoryType read GetUnitType write SetUnitType;
    property UsedByLocations: TObjectList read GetUsedByLocations;
    property UsedUnits: TcwUsedUnitList read GetUsedUnits;
    property UnitWarnings: TcwUnitWarningSet read FUnitWarnings write FUnitWarnings;
  end;

  TSupplySourceStream = procedure (Sender: TcwUnit; var aSourceStream: TMemoryStream) of object;

  TcwComponentList = class;

  TDFMPropType = (dptString, dptIdentifier, dptInteger, dptFloat, dptSet,
    dptList);

  TcwComponentProperty = class
  private
    FName: string;
    FValue: Variant;
    FValueType: TDFMPropType;
  protected
  public
    constructor Create(const aName : string);
    function GetName: string;
    function GetValue: Variant;
    function GetValueType: TDFMPropType;
    procedure SetValue(const aValue: Variant);
    procedure SetValueType(const aValue: TDFMPropType);
    property Name: string read GetName;
    property Value: Variant read GetValue write SetValue;
    property ValueType: TDFMPropType read GetValueType write SetValueType;
  end;

  TcwComponentPropertiesList = class
  private
    FList: TObjectList;
    FSortedIdentifiers: TcwHashedObjectList;
    FSortedList: TcwHashedObjectList;
  protected
  public
    constructor Create;
    destructor Destroy; override;
    procedure AddIdentifierName(const aIdentName : string;const aProperty : TcwComponentProperty);
    function AddProperty(const aName : string): TcwComponentProperty;
    function GetCount: Integer;
    function GetProperties(Index : Integer): TcwComponentProperty;
    function HasProperty(const aPropName : string): Boolean;
    function HasWithIdentifier(const aIdentName : string): Boolean;
    property Count: Integer read GetCount;
    property Properties[Index : Integer]: TcwComponentProperty read GetProperties; default;
  end;

  TcwComponent = class
  private
    FChildComponents: TcwComponentList;
    FClassName: string;
    FIsInherited: Boolean;
    FIsInline: Boolean;
    FName: string;
    FPosition: Integer;
    FProperties: TcwComponentPropertiesList;
  protected
  public
    constructor Create(const aName,aClassName : string);
    destructor Destroy; override;
    function GetCClassName: string;
    function GetChildComponents: TcwComponentList;
    function GetIsInherited: Boolean;
    function GetIsInline: Boolean;
    function GetName: string;
    function GetPosition: Integer;
    function GetProperties: TcwComponentPropertiesList;
    function HasProperty(const aPropName : string): Boolean;
    function HasPropertyWithIdentifier(const aIdentName : string): Boolean;
    procedure SetIsInherited(const aValue: Boolean);
    procedure SetIsInline(const aValue: Boolean);
    procedure SetPosition(const aValue: Integer);
    property CClassName: string read GetCClassName;
    property ChildComponents: TcwComponentList read GetChildComponents;
    property IsInherited: Boolean read GetIsInherited write SetIsInherited;
    property IsInline: Boolean read GetIsInline write SetIsInline;
    property Name: string read GetName;
    property Position: Integer read GetPosition write SetPosition;
    property Properties: TcwComponentPropertiesList read GetProperties;
  end;

  TcwComponentList = class
  private
    FList: TObjectList;
  protected
  public
    constructor Create;
    destructor Destroy; override;
    function AddComponent(const aName : string; const aClassName : string): TcwComponent;
    function GetComponents(Index : Integer): TcwComponent;
    function GetCount: Integer;
    property Components[Index : Integer]: TcwComponent read GetComponents; default;
    property Count: Integer read GetCount;
  end;

  TcwDFMResource = class
  private
    FClassList: TcwComponentList;
    FFileIndex: Integer;
    FFileName: string;
    FUnitIndex: Integer;
    procedure ReadDFM(const aUnit: TcwUnit; const aInStream: TStream);
    class function IsBinary(AStream: TStream): Boolean;
  protected
    function GetComponents: TcwComponentList;
    function GetFileName: string;
    function GetUnitIndex: Integer;
    procedure SetUnitIndex(const aValue: Integer);
  public
    constructor Create;
    destructor Destroy; override;
    function HasProperty(const aPropName: string): Boolean;
    function HasPropertyWithIdentifier(const aIdentName: string): Boolean;
    procedure ReadResource(const aUnit: TcwUnit; const aFileName: string);
    property Components: TcwComponentList read GetComponents;
    property FileName: string read GetFileName;
    property UnitIndex: Integer read GetUnitIndex write SetUnitIndex;
  end;

  TcwSourceFile = class
  private
    FFileIndex: Integer;
    FFullFileName: string;
    FUnitIndex: Integer;
    FFileExists: Boolean;
  protected
    function GetFileIndex: Integer;
    function GetFullFileName: string;
    function GetUnitIndex: Integer;
    function GetUnitIntf: TcwUnit;
  public
    constructor Create(const aFileName: string; const AFileExists: Boolean;
      const AFileIndex: Integer; const aUnitIndex: Integer = -1);
    property FileIndex: Integer read GetFileIndex;
    property FullFileName: string read GetFullFileName;
    property UnitIndex: Integer read GetUnitIndex;
    property UnitIntf: TcwUnit read GetUnitIntf;
    property FileExists: Boolean read FFileExists;
  end;

  //interface to save to file settings, in a INI - file format
  TcwFileNameList = class
  private
    FFileNameList: TcwHashedObjectList;
    FList: TObjectList;
  protected
    function GetCount: Integer;
    function GetFileNameList: TcwHashedObjectList;
    function GetItems(Index: Integer): TcwSourceFile;
  public
    constructor Create;
    destructor Destroy; override;
    function Add(const aFileName: string; const AFileExists: Boolean; AUnitIndex: Integer = -1):
      Integer;
    procedure Clear;
    function Find(AFileName: string): TcwSourceFile;
    property Count: Integer read GetCount;
    property FileNameList: TcwHashedObjectList read GetFileNameList;
    property Items[Index: Integer]: TcwSourceFile read GetItems; default;
  end;

  IcwStringList = interface(IInterface)
    ['{A5A50DE6-FB1C-4937-8A6D-89B7CE4855DC}']
    function Add(const avalue: string): Integer;
    procedure Clear;
    procedure CopyToClipboard;
    function GetCount: Integer;
    function GetString(Index: Integer): string;
    function GetText: string;
    procedure SaveToFile(const aFileName: string);
    procedure SetText(const aValue: string);
    function IndexOf(AString: string): Integer;
    property Count: Integer read GetCount;
    property Strings[Index: Integer]: string read GetString; default;
    property Text: string read GetText write SetText;
  end;

  IcwFileSettings = interface(IInterface)
    ['{4F2C704D-FE4F-4B56-BCF4-967901DC1C1F}']
    function GetFileName: string;
    function GetOption(const aSection: string; const aOption: string; const aDefaultValue: Variant):
      Variant;
    function Load(const aLocation: widestring): Boolean;
    procedure ReadSection(const aSection: string; const aOptionStrings: IcwStringList);
    procedure ReadSectionValues(const aSection: string; const aOptionStrings: IcwStringList);
    procedure Save(const aLocation: widestring);
    procedure SaveSection(const aSection: string; const aOptionStrings: IcwStringList);
    procedure SetOption(const aSection: string; const aOption: string; const aNewValue: Variant);
    property FileName: string read GetFileName;
  end;

  IcwRefactorOptions = interface(IInterface)
    ['{85859A49-C97D-4BBB-973D-1222206FC2EC}']
    function GetFolderComparisonExe: string;
    function GetFolderComparisonExeParams: string;
    function GetRefactorings: TStrings;
    procedure Load(const aFile: IcwFileSettings);
    procedure LoadDefaults;
    procedure Save(const aFile: IcwFileSettings);
    procedure SetFolderComparisonExe(const Value: string);
    procedure SetFolderComparisonExeParams(const Value: string);
    procedure SetRefactorings(const Value: TStrings);
    property FolderComparisonExe: string read GetFolderComparisonExe write SetFolderComparisonExe;
    property FolderComparisonExeParams: string read GetFolderComparisonExeParams write
      SetFolderComparisonExeParams;
    property Refactorings: TStrings read GetRefactorings write SetRefactorings;
  end;

  IcwSystemOptions = interface(IInterface)
    ['{ED4CBDA8-8588-49CD-938B-F61DD136A269}']
    function GetLibrarySearchPath: IcwStringList;
    function GetRefactorOptions: IcwRefactorOptions;
    procedure Load(const aFile: IcwFileSettings);
    procedure LoadDefaults;
    procedure Save(const aFile: IcwFileSettings);
    property LibrarySearchPath: IcwStringList read GetLibrarySearchPath;
    property RefactorOptions: IcwRefactorOptions read GetRefactorOptions;
  end;

  IcwOptions = interface(IInterface)
    ['{00C66710-D6DF-4F9B-829E-80FE744CA726}']
    procedure Clear;
    function GetAliases: IcwStringList;
    function GetDefines: IcwStringList;
    function GetFileToAnalyze: widestring;
    function GetProjectSearchPath: IcwStringList;
    procedure Load(const aFileSettings: IcwFileSettings);
    procedure Save(const aFileSettings: IcwFileSettings);
    procedure SetFileToAnalyze(const aValue: widestring);
    property Aliases: IcwStringList read GetAliases;
    property Defines: IcwStringList read GetDefines;
    property FileToAnalyze: widestring read GetFileToAnalyze write SetFileToAnalyze;
    property ProjectSearchPath: IcwStringList read GetProjectSearchPath;
  end;

  IcwCodeViewer = interface(IInterface)
    ['{523EDCE0-9EF4-4032-91D1-A6B31C872371}']
    procedure ShowSourceFile(const aFileName: string; aX, aY: Integer);
  end;

  IcwProgressNotifier = interface(IInterface)
    ['{BB7D22F5-4FC0-4C0C-BB2F-919763BF5164}']
    procedure ShowProgress(const FullFileName: string; ParserState: TParserState; var CancelParsing:
      Boolean);
  end;

  IcwSetting = interface(IInterface)
    ['{72F1C3FF-9408-49E8-82C5-1E27124FAA3C}']
    function GetCategory: string;
    function GetLibrarySet: Boolean;
    function GetName: string;
    function GetProjectSet: Boolean;
    procedure SetLibrarySet(const aValue: Boolean);
    procedure SetProjectSet(const aValue: Boolean);
    property Category: string read GetCategory;
    property LibrarySet: Boolean read GetLibrarySet write SetLibrarySet;
    property Name: string read GetName;
    property ProjectSet: Boolean read GetProjectSet write SetProjectSet;
  end;

  IcwSettingsCategory = interface(IInterface)
    ['{6B4A6B43-1867-4665-99BD-B125DF8FF855}']
    function AddCategory(const aName: string): IcwSettingsCategory;
    function AddSetting(const aName: string; const aDefaultProject: Boolean; const aDefaultLibrary:
      Boolean): IcwSetting;
    function GetCategories(Index: Integer): IcwSettingsCategory;
    function GetCategoryCount: Integer;
    function GetCount: Integer;
    function GetItems(Index: Integer): IcwSetting;
    function GetName: string;
    property Categories[Index: Integer]: IcwSettingsCategory read GetCategories;
    property CategoryCount: Integer read GetCategoryCount;
    property Count: Integer read GetCount;
    property Items[Index: Integer]: IcwSetting read GetItems; default;
    property Name: string read GetName;
  end;

  IcwSettings = interface(IcwSettingsCategory)
    ['{BCFE8D72-F6CA-477F-9F60-38A317D6E2C3}']
    procedure LoadSettings(const aFileSettings: IcwFileSettings);
    procedure SaveSettings(const aFileSettings: IcwFileSettings);
  end;

  IcwComponentPrefix = interface(IInterface)
    ['{C0C6F2AC-CF9F-4BA9-93B7-9848FB5CC6A6}']
    function GetComponentClass: string;
    function GetPrefix: string;
    procedure SetComponentClass(const aValue: string);
    procedure SetPrefix(const aValue: string);
    property ComponentClass: string read GetComponentClass write SetComponentClass;
    property Prefix: string read GetPrefix write SetPrefix;
  end;

  IcwPrefixList = interface(IInterface)
    ['{C2C19BB1-AE90-43CD-988F-A4B240BE2603}']
    function Add(const aPrefix: string; const aComponentClass: string): IcwComponentPrefix;
    procedure Clear;
    function ComponentPrefixComplies(const aComponentClass: string; const aSymbol: TcwSymbol; var
      aExpectedPrefix: string): Boolean;
    procedure Delete(Index: Integer);
    procedure DeletePrefix(const aPrefix: IcwComponentPrefix);
    function FindComponentClass(const aComponentClass: string): IcwComponentPrefix;
    function GetCount: Integer;
    function GetEnforcePrefixOnUnreferecend: Boolean;
    function GetItems(Index: Integer): IcwComponentPrefix;
    procedure Load(const aFile: IcwFileSettings);
    procedure LoadDefaultPrefixes;
    procedure Save(const aFile: IcwFileSettings);
    procedure SetEnforcePrefixOnUnreferecend(aValue: Boolean);
    property Count: Integer read GetCount;
    property EnforcePrefixOnUnreferecend: Boolean read GetEnforcePrefixOnUnreferecend write
      SetEnforcePrefixOnUnreferecend;
    property Items[Index: Integer]: IcwComponentPrefix read GetItems; default;
  end;

  IcwNamingConventions = interface(IInterface)
    ['{1CAE66C1-2FF7-47EA-B69B-2EE7CD575862}']
    function GetClassFields: string;
    function GetClassTypes: string;
    function GetConstants: string;
    function GetConstParameters: string;
    function GetGetRoutines: string;
    function GetGlobalVariables: string;
    function GetInterfaceTypes: string;
    function GetLocalVariables: string;
    function GetObjectTypes: string;
    function GetParameters: string;
    function GetPointerTypes: string;
    function GetProperties: string;
    function GetRecordTypes: string;
    function GetRoutineNames: string;
    function GetSetRoutines: string;
    function GetVarParameters: string;
    procedure Load(const aFile: IcwFileSettings);
    procedure LoadDefaults;
    procedure Save(const aFile: IcwFileSettings);
    procedure SetClassFields(const aValue: string);
    procedure SetClassTypes(const aValue: string);
    procedure SetConstants(const aValue: string);
    procedure SetConstParameters(const aValue: string);
    procedure SetGetRoutines(const aValue: string);
    procedure SetGlobalVariables(const aValue: string);
    procedure SetInterfaceTypes(const aValue: string);
    procedure SetLocalVariables(const aValue: string);
    procedure SetObjectTypes(const aValue: string);
    procedure SetParameters(const aValue: string);
    procedure SetPointerTypes(const aValue: string);
    procedure SetProperties(const aValue: string);
    procedure SetRecordTypes(const aValue: string);
    procedure SetRoutineNames(const aValue: string);
    procedure SetSetRoutines(const aValue: string);
    procedure SetVarParameters(const aValue: string);
    property ClassFields: string read GetClassFields write SetClassFields;
    property ClassTypes: string read GetClassTypes write SetClassTypes;
    property Constants: string read GetConstants write SetConstants;
    property ConstParameters: string read GetConstParameters write SetConstParameters;
    property GetRoutines: string read GetGetRoutines write SetGetRoutines;
    property GlobalVariables: string read GetGlobalVariables write SetGlobalVariables;
    property InterfaceTypes: string read GetInterfaceTypes write SetInterfaceTypes;
    property LocalVariables: string read GetLocalVariables write SetLocalVariables;
    property ObjectTypes: string read GetObjectTypes write SetObjectTypes;
    property Parameters: string read GetParameters write SetParameters;
    property PointerTypes: string read GetPointerTypes write SetPointerTypes;
    property Properties: string read GetProperties write SetProperties;
    property RecordTypes: string read GetRecordTypes write SetRecordTypes;
    property RoutineName: string read GetRoutineNames write SetRoutineNames;
    property SetRoutines: string read GetSetRoutines write SetSetRoutines;
    property VarParameters: string read GetVarParameters write SetVarParameters;
  end;

  IcwIndentationConventions = interface(IInterface)
    ['{956021B3-46F7-4AFF-A92F-DFA12DAF3C51}']
    function GetIndentBeforeBegin: Boolean;
    function GetIndentBeforeElseInCaseBlock: Boolean;
    function GetIndentBeforeElseInTryBlock: Boolean;
    function GetIndentBeforeTry: Boolean;
    function GetIndentComments: Boolean;
    function GetIndentDirectives: Boolean;
    function GetNeverIndentElseIf: Boolean;
    function GetSpacesPerIndent: Integer;
    procedure Load(const aFile: IcwFileSettings);
    procedure LoadDefaults;
    procedure Save(const aFile: IcwFileSettings);
    procedure SetIndentBeforeBegin(const aValue: Boolean);
    procedure SetIndentBeforeElseInCaseBlock(const aValue: Boolean);
    procedure SetIndentBeforeElseInTryBlock(const aValue: Boolean);
    procedure SetIndentBeforeTry(const aValue: Boolean);
    procedure SetIndentComments(const aValue: Boolean);
    procedure SetIndentDirectives(const aValue: Boolean);
    procedure SetNeverIndentElseIf(const aValue: Boolean);
    procedure SetSpacesPerIndent(const aValue: Integer);
    property IndentBeforeBegin: Boolean read GetIndentBeforeBegin write SetIndentBeforeBegin;
    property IndentBeforeElseInCaseBlock: Boolean read GetIndentBeforeElseInCaseBlock write
      SetIndentBeforeElseInCaseBlock;
    property IndentBeforeElseInTryBlock: Boolean read GetIndentBeforeElseInTryBlock write
      SetIndentBeforeElseInTryBlock;
    property IndentBeforeTry: Boolean read GetIndentBeforeTry write SetIndentBeforeTry;
    property IndentComments: Boolean read GetIndentComments write SetIndentComments;
    property IndentDirectives: Boolean read GetIndentDirectives write SetIndentDirectives;
    property NeverIndentElseIf: Boolean read GetNeverIndentElseIf write SetNeverIndentElseIf;
    property SpacesPerIndent: Integer read GetSpacesPerIndent write SetSpacesPerIndent;
  end;

  IcwCodingConventions = interface(IInterface)
    ['{F5965926-03D2-4C89-8D37-EA0D2EA3ADD7}']
    function GetComponentPrefixes: IcwPrefixList;
    function GetIndentationConventions: IcwIndentationConventions;
    function GetNamingConventions: IcwNamingConventions;
    procedure Load;
    procedure LoadDefault;
    procedure LoadDefaultComponentPrefixes;
    procedure LoadDefaultIndentationConventions;
    procedure LoadDefaultNamingConventions;
    procedure LoadDefaultSystemSymbols;
    procedure LoadFromFile(const aFileSettings: IcwFileSettings);
    procedure Save;
    procedure SaveToFile(const aFileSettings: IcwFileSettings);
    property ComponentPrefixes: IcwPrefixList read GetComponentPrefixes;
    property IndentationConventions: IcwIndentationConventions read GetIndentationConventions;
    property NamingConventions: IcwNamingConventions read GetNamingConventions;
  end;

  IcwReportData = interface(IInterface)
    ['{5EF6D97A-951D-4FCF-BF38-6A2C8AB402F9}']
    procedure AddEmptyLine; overload;
    procedure AddEmptyLine(const aSetting: IcwSetting); overload;
    procedure AddEmptyLine(const aSetting: IcwSetting; const aIsLibraryUnit: Boolean); overload;
    procedure AddRow(const aSetting: IcwSetting; const aIsLibraryUnit: Boolean; const aCodeLocation:
      TcwCodeLocation; const aDetails: TcwUsedByList; const aValues: array of Variant); overload;
    procedure AddRow(const aSetting: IcwSetting; const aCodeLocation: TcwCodeLocation; const aDetails:
      TcwUsedByList; const aValues: array of Variant); overload;
    procedure AddRow(const aCodeLocation: TcwCodeLocation; const aDetails: TcwUsedByList; const aValues:
      array of Variant); overload;
    procedure AddRow(const aUnit: TcwUnit; const aSetting: IcwSetting; const aCodeLocation:
      TcwCodeLocation; const aDetails: TcwUsedByList; const aValues: array of Variant); overload;
    procedure AddSummary(const aValues: array of Variant);
    procedure AddTitle(const aSetting: IcwSetting; const aIsLibraryUnit: Boolean; const aValue: variant);
      overload;
    procedure AddTitle(const aSetting: IcwSetting; const aValue: variant); overload;
    procedure AddTitle(const aValue: variant); overload;
    procedure Clear;
    procedure ExecRow(const aRow: Integer);
    function GetCount: Integer;
    function GetDetails(Index: Integer): TcwUsedByList;
    function GetValues(Row: Integer; Column: Integer): Variant;
    procedure SortBy(const aAsc: Boolean; const aColumn: Integer);
    property aValues[Row: Integer; Column: Integer]: Variant read GetValues; default;
    property Count: Integer read GetCount;
    property Details[Index: Integer]: TcwUsedByList read GetDetails;
  end;

  IcwColumn = interface(IInterface)
    ['{0FAC4110-06C1-40E6-855B-2B38041EAA0C}']
    function GetDataType: TcwDataType;
    function GetIndex: Integer;
    function GetName: string;
    property DataType: TcwDataType read GetDataType;
    property Index: Integer read GetIndex;
    property Name: string read GetName;
  end;

  IcwColumns = interface(IInterface)
    ['{E081A698-1E51-4656-9F87-C86FFC1D565A}']
    function AddColumn(const aName: string; const aDataType: TcwDataType): IcwColumn;
    function GetColumns(Index: Integer): IcwColumn;
    function GetCount: Integer;
    property Columns[Index: Integer]: IcwColumn read GetColumns; default;
    property Count: Integer read GetCount;
  end;

  IcwReport = interface(IInterface)
    ['{BF298C61-9A88-468C-9797-9826A5379C68}']
    procedure Clear;
    procedure Execute;
    function GetAsCSV(const aIncludeHeaders: Boolean; aQuotedValues: Boolean): IcwStringList;
    function GetColumns: IcwColumns;
    function GetData: IcwReportData;
    function GetHasDetails: Boolean;
    function GetName: string;
    function GetSettings: IcwSettings;
    function GetSortable: Boolean;
    function GetVisible: Boolean;
    procedure SetVisible(aValue: Boolean);
    property Columns: IcwColumns read GetColumns;
    property Data: IcwReportData read GetData;
    property HasDetails: Boolean read GetHasDetails;
    property Name: string read GetName;
    property Settings: IcwSettings read GetSettings;
    property Sortable: Boolean read GetSortable;
    property Visible: Boolean read GetVisible write SetVisible;
  end;

  //interface for regular expressions
  IcwReports = interface(IInterface)
    ['{F54A3712-CF91-4A15-B7D3-37EDB34EE807}']
    function AddReport(const aReport: IcwReport): Integer;
    procedure Clear;
    function GetCount: Integer;
    function GetReportByName(const aReportName: string): IcwReport;
    function GetReports(Index: Integer): IcwReport;
    function RemoveReport(const aReport: IcwReport): Boolean;
    property Count: Integer read GetCount;
    property Reports[Index: Integer]: IcwReport read GetReports; default;
  end;

  TMessageListSortBy = (mlsbMessage, mlsbUnit, mlsbInterface);

  //parsing error messages
  IcwRegExpr = interface(IInterface)
    ['{4FB7AC8E-9F2E-4288-8E0F-A61D61563379}']
    function Match(const aRegExpr: string; const aString: string): Boolean;
  end;

  IcwMessage = interface(IInterface)
    ['{E32A27C5-1273-4163-A660-2BF7C28EB88A}']
    function GetCodeLocation: TcwCodeLocation;
    function GetInterfaceSection: Boolean;
    function GetMessageText: string;
    property CodeLocation: TcwCodeLocation read GetCodeLocation;
    property InterfaceSection: Boolean read GetInterfaceSection;
    property MessageText: string read GetMessageText;
  end;

  IcwMessageList = interface(IInterface)
    ['{C431285B-BFE7-4E15-917F-0D9BB0A08EA5}']
    function AddMessage(const aMsg: string; const AFileIndex: Integer; const aX, aY: Integer;
      const AInterfaceSection: Boolean): IcwMessage;
    procedure Clear;
    function GetCount: Integer;
    function GetMessage(Index: Integer): IcwMessage;
    procedure Sort(ASortBy: TMessageListSortBy);
    property Count: Integer read GetCount;
    property Messages[Index: Integer]: IcwMessage read GetMessage; default;
  end;

  TcwUnitList = class(TcwListSorted)
  protected
    function GetSortedUnits(Index: Integer): TcwUnit;
    function GetUnits(Index: Integer): TcwUnit;
  public
    function AddUnit(const AUnit: TcwUnit): Integer;
    function FindUnit(const aUnitName: string): TcwUnit;
    property SortedUnits[Index: Integer]: TcwUnit read GetSortedUnits;
    property Units[Index: Integer]: TcwUnit read GetUnits; default;
  end;

  IcwProject = interface(IInterface)
    ['{1BCCB41B-6209-43E9-B94C-FE6031F5525C}']
    function GetProjectUnit: TcwUnit;
    procedure AddCodeViewer(const aViewer: IcwCodeViewer);
    procedure AddProgressNotifier(const aNotifier: IcwProgressNotifier);
    procedure Clear;
    function CreateCWSettings(const aName: string): IcwSettings;
    function GetAllUnits: TcwUnitList;
    function GetCanceled: Boolean;
    function GetCodingConventions: IcwCodingConventions;
    function GetCWOptions: IcwOptions;
    function GetFileNames: TcwFileNameList;
    function GetMessages: IcwMessageList;
    function GetPredefinedType(ATypeName: string): TcwSymbol; overload;
    function GetPredefinedType(const aTypeKind: TcwTypeKind): TcwSymbol; overload;
    function GetProjectOptions: IcwFileSettings;
    function GetSystemOptions: IcwSystemOptions;
    function GetRegularExpressions: IcwRegExpr;
    function GetReports: IcwReports;
    function GetSystemFileSettings: IcwFileSettings;
    function GetUnparsedClasses: TcwClassList;
    function GetUnparsedInterfaces: TcwInterfaceList;
    function GetUnparsedObjects: TcwClassList;
    function GetUnparsedSymbols: TcwSymbolList;
    function Load(const aLocation: widestring): Boolean;
    procedure OnParseUnit(const aFullFileName: string; ParseState: TParserState);
    procedure RemoveCodeViewer(const aViewer: IcwCodeViewer);
    procedure RemoveProgressNotifier(const aNotifier: IcwProgressNotifier);
    function ResolveUnitName(const aUnitName: string; const ARelativeToPath: string; out DirType:
      TDirectoryType): string;
    function AddUnit(const aUnitName: string; const aFullFileName: string = ''): TcwUnit;
    procedure Run(aParseAllImplementations: Boolean = True);
    procedure ParseInterface(const AUnit: TcwUnit);
    procedure Save(const aLocation: widestring);
    procedure ShowLocation(const aLocation: TcwCodeLocation);
    procedure ShowSourceFile(const aFileName: string; aX, aY: Integer);
    property AllUnits: TcwUnitList read GetAllUnits;
    property Canceled: Boolean read GetCanceled;
    property CodingConventions: IcwCodingConventions read GetCodingConventions;
    property CWOptions: IcwOptions read GetCWOptions;
    property FileNames: TcwFileNameList read GetFileNames;
    property Messages: IcwMessageList read GetMessages;
    property ProjectOptions: IcwFileSettings read GetProjectOptions;
    property SystemOptions: IcwSystemOptions read GetSystemOptions;
    property RegularExpressions: IcwRegExpr read GetRegularExpressions;
    property Reports: IcwReports read GetReports;
    property SystemFileSettings: IcwFileSettings read GetSystemFileSettings;
    property UnparsedClasses: TcwClassList read GetUnparsedClasses;
    property UnparsedInterfaces: TcwInterfaceList read GetUnparsedInterfaces;
    property UnparsedObjects: TcwClassList read GetUnparsedObjects;
    property UnparsedSymbols: TcwSymbolList read GetUnparsedSymbols;
    property ProjectUnit: TcwUnit read GetProjectUnit;
  end;

  IcwRefactoring = interface(IInterface)
    ['{2719AD0C-999A-4F00-9F68-952C4FB87A45}']
    procedure Execute;
    function GetCanExecute: Boolean;
    function GetGroupName: string;
    function GetMenuCaption: string;
    function GetName: string;
    property CanExecute: Boolean read GetCanExecute;
    property GroupName: string read GetGroupName;
    property MenuCaption: string read GetMenuCaption;
    property Name: string read GetName;
  end;

  IcwRefactoringHost = interface(IInterface)
    ['{CE37AB84-309B-4E44-86E6-5B3B718D65AA}']
    procedure AcceptRefactoringMessage(const ARefactoring: IcwRefactoring;
      const AMessage: IcwMessage);
    procedure AddGroupName(AGroupName: string; AGroupCaption: string;
      AParentGroupName: string = '');
    function GetSelectedSymbol: TcwSymbol;
    function GetSelectedUnit: TcwUnit;
    procedure RefactoringStatus(const AUnit: TcwUnit);
    procedure RegisterRefactoring(const ARefactoring: IcwRefactoring; AGroupName: string = '');
    procedure RemoveGroupName(AGroupName: string);
    procedure UnregisterRefactoring(const ARefactoring: IcwRefactoring);
    property SelectedSymbol: TcwSymbol read GetSelectedSymbol;
    property SelectedUnit: TcwUnit read GetSelectedUnit;
  end;

var
  gcwProject: IcwProject = nil;
  gcwCodeWrenchOptions: IcwFileSettings = nil;
  gcwRefactoringHost: IcwRefactoringHost = nil;

var
  GlobalKeyHashTable: TmwPasHash = nil;
  GlobalDirectiveHash: TmwDirectiveHash = nil;

var
  RoutineKindName: array[TRoutineKind] of string =
    ('Procedure', 'Function', 'Constructor', 'Destructor');

function UsedByInfo(const AFileIndex: Integer; const AIsWrongCase: Boolean;
  const AX, AY: Integer): TcwUsedByInfo;

implementation

uses
  Math,
  SysUtils,
  RTLConsts,
  ucwFind,
  ucwQuickSort;

function UsedByInfo(const AFileIndex: Integer; const AIsWrongCase: Boolean;
  const AX, AY: Integer): TcwUsedByInfo;
begin
  Result.FileIndex := AFileIndex;
  Result.IsWrongCase := aIsWrongCase;
  Result.X := AX;
  Result.Y := AY;
end;

{- TcwCodeLocation Category: No category }
function TcwCodeLocation.ComparesTo(const ACodeLocation: TcwCodeLocation): Boolean;
begin
  Result := (FileIndex = ACodeLocation.FileIndex) and (X = ACodeLocation.X) and
    (Y = ACodeLocation.Y);
end;

constructor TcwCodeLocation.Create(const AFileIndex: Integer; const aX, aY: Integer);
begin
  inherited Create;
  // FileIndex does not necessarily equal UnitIndex, since FileIndex might be an include file, and
  // UnitIndex would be the unit that includes this file
  FFileIndex := AFileIndex;
  FUnitIndex := -1;
  SetX(AX);
  SetY(aY);
end;

function TcwCodeLocation.GetEndX: Integer;
begin
  Result := FEndX;
end;

function TcwCodeLocation.GetEndY: Integer;
begin
  Result := FEndY;
end;

function TcwCodeLocation.GetFileIndex: Integer;
begin
  Result := FFileIndex;
end;

function TcwCodeLocation.GetFileName: string;
begin
  if FileIndex >= 0 then
    Result := gcwProject.FileNames[FileIndex].FullFileName
  else
    Result := '???';
end;

function TcwCodeLocation.GetUnitIndex: Integer;
begin
  if (FUnitIndex = -1) and (FFileIndex >= 0) then
    FUnitIndex := gcwProject.FileNames[FFileIndex].UnitIndex;
  Result := FUnitIndex;
end;

function TcwCodeLocation.GetUnitIntf: TcwUnit;
begin
  if UnitIndex >= 0 then
    Result := gcwProject.AllUnits[UnitIndex]
  else
    Result := nil;
end;

function TcwCodeLocation.GetX: Integer;
begin
  Result := FX;
end;

function TcwCodeLocation.GetY: Integer;
begin
  Result := FY;
end;

procedure TcwCodeLocation.SetEndX(const aValue: Integer);
begin
  FEndX := aValue;
end;

procedure TcwCodeLocation.SetEndY(const aValue: Integer);
begin
  FEndY := aValue;
end;

procedure TcwCodeLocation.SetX(const aValue: Integer);
begin
  FX := aValue;
  FEndX := FX;
end;

procedure TcwCodeLocation.SetY(const aValue: Integer);
begin
  FY := aValue;
  FEndY := FY;
end;

{- TcwUsedByLocation Category: No category }
constructor TcwUsedByLocation.Create(const AUsedByList: TcwUsedByList;
  const AUsedByInfo: TcwUsedByInfo);
begin
  inherited Create(AUsedByInfo.FileIndex, AUsedByInfo.X, AUsedByInfo.Y);
  FUsedByList := AUsedByList;
  FIsWrongCase := AUsedByInfo.IsWrongCase;
  if Assigned(UnitIntf) then
    UnitIntf.UsedByLocations.Add(Self);
end;

function TcwUsedByLocation.GetIsWrongCase: Boolean;
begin
  Result := FIsWrongCase;
end;

function TcwUsedByLocation.GetUsedByList: TcwUsedByList;
begin
  Result := FUsedByList;
end;

{- TcwList Category: No category }
constructor TcwList.Create;
begin
  inherited;
  FList := TObjectList.Create;
  FList.OwnsObjects := GetOwnsObjects;
end;

destructor TcwList.Destroy;
begin
  FList.Free;
  inherited;
end;

function TcwList.Add(const AObject: TObject): Integer;
begin
  Result := FList.Add(AObject);
end;

procedure TcwList.Clear;
begin
  FList.Clear;
end;

function TcwList.GetCount: Integer;
begin
  Result := FList.Count;
end;

function TcwList.GetOwnsObjects: Boolean;
begin
  Result := True;
end;

function TcwList.GetItem(Index: Integer): TObject;
begin
  Result := FList[Index];
end;

procedure TcwList.SetItem(Index: Integer; const Value: TObject);
begin
  FList[Index] := Value;
end;

{- TcwHashedObjectList Category: No category }
constructor TcwHashedObjectList.Create;
begin
  inherited;
  FHashStringList := TmwAssignAbleHash.Create;
end;

destructor TcwHashedObjectList.Destroy;
begin
  FHashStringList.Free;
  inherited;
end;

function TcwHashedObjectList.Add(const AString: string; const AObject: TObject): Integer;
var
  AItem: TmwLexHashStringItem;
begin
  AItem := TmwLexHashStringItem.Create;
  AItem.Key := AString;
  AItem.Data := Pointer(AObject);
  AItem.HashValue := FHashStringList.HashOf(AString);
  FHashStringList.AddItem(AItem);
  Result := FHashStringList.Count - 1;
end;

procedure TcwHashedObjectList.Clear;
begin
  FHashStringList.Clear;
end;

function TcwHashedObjectList.Count: Integer;
begin
  Result := FHashStringList.Count;
end;

procedure TcwHashedObjectList.Delete(Index: Integer);
begin
  FHashStringList.Delete(Index);
end;

function TcwHashedObjectList.Find(const AString: string; out AIndex: Integer): Boolean;
begin
  AIndex := FHashStringList.IndexOf(AString);
  Result := AIndex >= 0;
end;

function TcwHashedObjectList.GetObject(Index: Integer): TObject;
begin
  Result := TObject(FHashStringList[Index].Data);
end;

function TcwHashedObjectList.GetString(Index: Integer): string;
begin
  Result := FHashStringList[Index].Key;
end;

procedure TcwHashedObjectList.SetObject(Index: Integer; const Value: TObject);
begin
  FHashStringList[Index].Data := Pointer(Value);
end;

procedure TcwHashedObjectList.Sort;
begin
  if not FHashStringList.Sorted then
    FHashStringList.QuickSort;
end;

{- TcwListSorted }
destructor TcwListSorted.Destroy;
begin
  FSortedList.Free;
  FStringList.Free;
  inherited;
end;

procedure TcwListSorted.Clear;
begin
  inherited;
  if Assigned(FSortedList) then
    FSortedList.Clear;
end;

function TcwListSorted.Find(AName: string): TObject;
var
  AIndex: Integer;
begin
  if Assigned(FSortedList) and FSortedList.Find(AName, AIndex) then
    Result := FSortedList[AIndex]
  else
    Result := nil;
end;

function TcwListSorted.DoItemAdded(const AName: string; AObject: TObject): Integer;
begin
  Result := Add(AObject);
  if not Assigned(FSortedList) then
    FSortedList := TcwHashedObjectList.Create;
  FSortedList.Add(AName, AObject);
  if Assigned(FStringList) then
    FreeAndNil(FStringList);
end;

procedure TcwListSorted.LoadStringList;
var
  Index: Integer;
begin
  Assert(not Assigned(FStringList));
  FStringList := TStringList.Create;
  for Index := 0 to Count - 1 do
    FStringList.AddObject(FSortedList.Strings[Index], FSortedList.Objects[Index]);
  FStringList.Sorted := True;
end;

function TcwListSorted.GetSortedItem(Index: Integer): TObject;
begin
  if not Assigned(FStringList) then
    LoadStringList;
  Result := FStringList.Objects[Index];
end;

{- TcwContainerList Category: No category }
constructor TcwContainerList.Create(const AContainer: TcwVarContainer);
begin
  inherited Create;
  FContainer := AContainer;
end;

function TcwContainerList.GetContainer: TcwVarContainer;
begin
  Result := FContainer;
end;

function TcwContainerList.GetOwnsObjects: Boolean;
begin
  Result := True;
end;

{- TcwContainerListSorted }
destructor TcwContainerListSorted.Destroy;
begin
  FSortedList.Free;
  FStringList.Free;
  inherited;
end;

procedure TcwContainerListSorted.LoadStringList;
var
  Index: Integer;
begin
  Assert(not Assigned(FStringList));
  FStringList := TStringList.Create;
  for Index := 0 to Count - 1 do
    FStringList.AddObject(FSortedList.Strings[Index], FSortedList.Objects[Index]);
  FStringList.Sorted := True;
end;

function TcwContainerListSorted.GetSortedItem(Index: Integer): TObject;
begin
  if not Assigned(FStringList) then
    LoadStringList;
  Result := FStringList.Objects[Index];
end;

function TcwContainerListSorted.InternalSymbolAdded(const ASymbol: TcwSymbol): Integer;
begin
  Result := Add(ASymbol);
  if not Assigned(FSortedList) then
    FSortedList := TcwHashedObjectList.Create;
  FSortedList.Add(ASymbol.Name, ASymbol);
  if Assigned(FStringList) then
    FreeAndNil(FStringList);
end;

function TcwContainerListSorted.DoSymbolAdded(const ASymbol: TcwSymbol): Integer;
begin
  Result := InternalSymbolAdded(ASymbol);
  Container.AddSymbol(ASymbol);
end;

function TcwContainerListSorted.DoTypeAdded(const ASymbol: TcwSymbol): Integer;
begin
  Result := InternalSymbolAdded(ASymbol);
  Container.Types.AddType(ASymbol);
end;

function TcwContainerListSorted.Find(const aName: string): TObject;
var
  I: Integer;
begin
  if Assigned(FSortedList) and FSortedList.Find(AName, I) then
    Result := FSortedList[I]
  else
    Result := nil;
end;

{- TcwSymbol Category: No category }
function TcwSymbol.ComparesTo(const ASymbol: TcwSymbol): Boolean;
begin
  Result := (Name = ASymbol.Name) and (inherited ComparesTo(ASymbol));
end;

constructor TcwSymbol.Create(const aName: string; aVisibility: TVisibilityKind;
  aTypeKind: TcwTypeKind; const AFileIndex: Integer; const aX, aY: Integer);
begin
  inherited Create(AFileIndex, aX, aY);
  FName := aName;
  FVisibility := aVisibility;
end;

destructor TcwSymbol.Destroy;
begin
  FUsedBy.Free;
  inherited;
end;

function TcwSymbol.FindContainer: TcwSymbolContainer;

  function GetPointerOrArrayType(const aType: TcwSymbol): TcwSymbol;
  var
    aArray: TcwArray;
  begin
    Result := aType;
    // the container should be what we're pointing to, or what the array reference is...
    repeat
      while Assigned(Result) and Result.PointerType do
        if Result <> Result.TypeIntf then
          Result := Result.TypeIntf
        else
          Break;

      while Assigned(Result) and (Result is TcwArray) do
      begin
        aArray := Result as TcwArray;
        if Result <> aArray.ArrayType then
          Result := aArray.ArrayType
        else
          Break;
      end;
    until not Assigned(Result) or (not Result.PointerType and not (Result is TcwArray));
  end;

var
  aTempType: TcwSymbol;
begin
  //TODO: have to add support for special 'Self' variable, since its type should fluctuate based
  // on the current class
  Result := nil;
  if Assigned(PredefinedArrayType) then
    aTempType := PredefinedArrayType
  else
    ATempType := TypeIntf;
  while Assigned(aTempType) do
  begin
    aTempType := GetPointerOrArrayType(aTempType);
    if Assigned(aTempType) then
    begin
      if (aTempType.TypeIntf is TcwSymbolContainer) then
      begin
        Result := aTempType.TypeIntf as TcwSymbolContainer;
        Break;
      end;
      if aTempType <> aTempType.TypeIntf then
        aTempType := aTempType.TypeIntf
      else
        Break;
    end;
  end;
end;

function TcwSymbol.FindSymbol(const AFromUnit: TcwUnit; const aFromClass: TcwClass;
  const aName: string; const AAddUsedBy: Boolean; AUsedByInfo: TcwUsedByInfo): TcwSymbol;
begin
  Result := nil;
end;

function TcwSymbol.FindSymbol(const AFromUnit: TcwUnit; const aFromClass: TcwClass;
  const aName: string): TcwSymbol;
begin
  Result := FindSymbol(AFromUnit, aFromClass, aName, False, UsedByInfo(-1, False, -1, -1));
end;

function TcwSymbol.GetName: string;
begin
  Result := FName;
end;

function TcwSymbol.GetPointerType: Boolean;
begin
  Result := FPointerType;
end;

function TcwSymbol.GetPointerTypeName: string;
begin
  Result := FPointerTypeName;
end;

function TcwSymbol.GetPredefinedArrayType: TcwSymbol;
begin
  Result := FPredefinedArrayType;
end;

function TcwSymbol.GetTypeAssigned: Boolean;
begin
  Result := Assigned(FType);
end;

function TcwSymbol.GetTypeIntf: TcwSymbol;
begin
  if Assigned(FType) then
    Result := FType
  else
    Result := Self;
end;

function TcwSymbol.GetTypeKind: TcwTypeKind;
begin
  Result := FTypeKind;
end;

function TcwSymbol.GetUsedBy: TcwUsedByList;
begin
  if not Assigned(FUsedBy) then
    FUsedBy := TcwUsedByList.Create(Self);
  Result := FUsedBy;
end;

function TcwSymbol.GetVisibility: TVisibilityKind;
begin
  Result := FVisibility;
end;

procedure TcwSymbol.SetPointerType(const Value: Boolean);
begin
  FPointerType := Value;
end;

procedure TcwSymbol.SetPointerTypeName(const Value: string);
begin
  FPointerTypeName := Value;
end;

procedure TcwSymbol.SetPredefinedArrayType(const Value: TcwSymbol);
begin
  FPredefinedArrayType := Value;
end;

procedure TcwSymbol.SetTypeIntf(const aValue: TcwSymbol);
begin
  FType := aValue;
end;

procedure TcwSymbol.SetTypeKind(const aValue: TcwTypeKind);
begin
  FTypeKind := aValue;
end;

procedure TcwSymbol.SetUsedBy(const aValue: TcwUsedByList);
begin
  Assert(False);
end;

function TcwSymbol.TypeRequired: Boolean;
begin
  Result := True;
end;

{- TcwSymbolList Category: No category }
constructor TcwSymbolList.Create(const AOwnsSymbols: Boolean);
begin
  FOwnsObjects := AOwnsSymbols;
  inherited Create;
end;

destructor TcwSymbolList.Destroy;
begin
  FSortedList.Free;
  inherited;
end;

function TcwSymbolList.GetOwnsObjects: Boolean;
begin
  Result := FOwnsObjects;
end;

function TcwSymbolList.AddSymbol(const aItem: TcwSymbol): Integer;
begin
  Result := Add(aItem);
  if not Assigned(FSortedList) then
    FSortedList := TcwHashedObjectList.Create;
  FSortedList.Add(aItem.Name, aItem);
end;

function TcwSymbolList.AddSymbol(const aName: string; aVisibility: TVisibilityKind;
  const AFileIndex: Integer; const aX, aY: Integer): TcwSymbol;
begin
  Result := TcwSymbol.Create(aName, aVisibility, tkNone, AFileIndex, aX, aY);
  AddSymbol(Result);
end;

procedure TcwSymbolList.Clear;
begin
  inherited;
  if Assigned(FSortedList) then
    FSortedList.Clear;
end;

function TcwSymbolList.Find(const aSymbolName: string): TcwSymbol;
var
  Index: Integer;
begin
  if Assigned(FSortedList) and FSortedList.Find(aSymbolName, Index) then
    Result := FSortedList[Index] as TcwSymbol
  else
    Result := nil;
end;

function TcwSymbolList.GetSortedSymbols(Index: Integer): TcwSymbol;
begin
  Result := FSortedList[Index] as TcwSymbol;
end;

function TcwSymbolList.GetSymbol(Index: Integer): TcwSymbol;
begin
  Result := Items[Index] as TcwSymbol;
end;

{- TcwCodeStatistics Category: No category }
function TcwCodeStatistics.GetAnsiCommentLines: Integer;
begin
  Result := FAnsiCommentLines;
end;

function TcwCodeStatistics.GetBorCommentLines: Integer;
begin
  Result := FBorCommentLines;
end;

function TcwCodeStatistics.GetCodeLines: Integer;
begin
  Result := FCodeLines;
end;

function TcwCodeStatistics.GetCommentLines: Integer;
begin
  Result := FAnsiCommentLines + FBorCommentLines + FSlashCommentLines;
end;

function TcwCodeStatistics.GetEmptyLines: Integer;
begin
  Result := FEmptyLines;
end;

function TcwCodeStatistics.GetSlashCommentLines: Integer;
begin
  Result := FSlashCommentLines;
end;

function TcwCodeStatistics.GetTotalLines: Integer;
begin
  Result := FCodeLines + CommentLines + FEmptyLines;
end;

procedure TcwCodeStatistics.IncAnsiCommentLines;
begin
  Inc(FAnsiCommentLines);
end;

procedure TcwCodeStatistics.IncBorCommentLines;
begin
  Inc(FBorCommentLines);
end;

procedure TcwCodeStatistics.IncCodeLines;
begin
  Inc(FCodeLines);
end;

procedure TcwCodeStatistics.IncEmptyLines;
begin
  Inc(FEmptyLines);
end;

procedure TcwCodeStatistics.IncSlashCommentLines;
begin
  Inc(FSlashCommentLines);
end;

{- TcwSymbolContainer Category: No category }
constructor TcwSymbolContainer.Create(const aName: string; const aVisibility: TVisibilityKind;
  const AFileIndex: Integer; const aX, aY: Integer);
begin
  inherited Create(aName, aVisibility, tkNone, AFileIndex, aX, aY);
  FHashedSymbols := TcwHashedObjectList.Create;
end;

destructor TcwSymbolContainer.Destroy;
begin
  FHashedSymbols.Free;
  inherited;
end;

procedure TcwSymbolContainer.AddSymbol(const aSymbol: TcwSymbol);
begin
  FHashedSymbols.Add(ASymbol.Name, aSymbol);
end;

function TcwSymbolContainer.FindSymbol(const AFromUnit: TcwUnit; const aFromClass: TcwClass;
  const aName: string; const AAddUsedBy: Boolean; AUsedByInfo: TcwUsedByInfo): TcwSymbol;
var
  Index: Integer;
begin
  if FHashedSymbols.Find(aName, Index) then
  begin
    Result := FHashedSymbols.Objects[Index] as TcwSymbol;
    AUsedByInfo.IsWrongCase := aName <> Result.Name;
    Result.UsedBy.AddUsedBy(AUsedByInfo);
  end else
    Result := nil;
end;

function TcwSymbolContainer.GetSymbol(Index: Integer): TcwSymbol;
begin
  Result := FHashedSymbols[Index] as TcwSymbol;
end;

function TcwSymbolContainer.GetSymbolCount: Integer;
begin
  Result := FHashedSymbols.Count;
end;

{- TcwUsedByList Category: No category }
procedure TcwUsedByList.AddUsedBy(const AUsedByInfo: TcwUsedByInfo);
var
  vLocation: TcwUsedByLocation;
begin
  vLocation := TcwUsedByLocation.Create(Self, AUsedByInfo);
  Add(vLocation);
end;

constructor TcwUsedByList.Create(const ASymbol: TcwSymbol);
begin
  inherited Create;
  FSymbol := ASymbol;
end;

function TcwUsedByList.GetLocations(Index: Integer): TcwUsedByLocation;
begin
  Result := Items[Index] as TcwUsedByLocation;
end;

{- TcwType Category: No category }
constructor TcwType.Create(const aName: string; aTypeKind: TcwTypeKind; aVisibility: TVisibilityKind;
  const AFileIndex: Integer; const aX, aY: Integer; ATypeRequired: Boolean);
begin
  inherited Create(aName, aVisibility, aTypeKind, AFileIndex, aX, aY);
  FTypeRequired := ATypeRequired;
end;

function TcwType.GetTypeIntf: TcwSymbol;
begin
  Result := FTypeIntf;
end;

procedure TcwType.SetTypeIntf(const aValue: TcwSymbol);
begin
  FTypeIntf := aValue;
end;

function TcwType.TypeRequired: Boolean;
begin
  Result := FTypeRequired;
end;

{- TcwTypesList Category: No category }
constructor TcwTypesList.Create(const aContainer: TcwVarContainer);
begin
  FUnitName := aContainer.Name;
  inherited Create(aContainer);
end;

function TcwTypesList.AddType(const aTypeName: string; aVisibility: TVisibilityKind;
  const AFileIndex: Integer; const aX, aY: Integer; ATypeRequired: Boolean): TcwSymbol;
begin
  Result := TcwType.Create(aTypeName, tkNone, aVisibility, AFileIndex, aX, aY,
    ATypeRequired);
  DoSymbolAdded(Result);
end;

function TcwTypesList.DoSymbolAdded(const ASymbol: TcwSymbol): Integer;
var
  ATempType: TcwType;
begin
  Result := inherited DoSymbolAdded(ASymbol);
  // initialize pointer types that might point to this type.
  // this is for 'forward pointers' for example in a linked list definition
  ATempType := Container.FindForwardPointerType(ASymbol.Name);
  if Assigned(ATempType) then
    if ATempType.PointerType and SameText(ATempType.PointerTypeName, ASymbol.Name) then
      ATempType.TypeIntf := ASymbol;
end;

function TcwTypesList.AddType(const aType: TcwSymbol): Integer;
begin
  Result := DoSymbolAdded(aType);
end;

function TcwTypesList.Find(const aTypeName: string): TcwSymbol;
begin
  Result := inherited Find(aTypeName) as TcwSymbol;
end;

function TcwTypesList.GetSortedTypes(Index: Integer): TcwSymbol;
begin
  Result := SortedItem[Index] as TcwSymbol;
end;

function TcwTypesList.GetTypes(Index: Integer): TcwSymbol;
begin
  Result := Items[Index] as TcwSymbol;
end;

{- TcwVariable Category: No category }
constructor TcwVariable.Create(const aName: string; aVisibility: TVisibilityKind; aIsConst: Boolean;
  const AFileIndex: Integer; const aX, aY: Integer);
begin
  inherited Create(aName, aVisibility, tkNone, AFileIndex, aX, aY);
  FIsConst := aIsConst;
end;

function TcwVariable.GetIsConst: Boolean;
begin
  Result := FIsConst;
end;

function TcwVariable.GetIsThreadVar: Boolean;
begin
  Result := FIsThreadVar;
end;

function TcwVariable.GetTypeIntf: TcwSymbol;
begin
  Result := FVarType;
end;

procedure TcwVariable.SetIsThreadVar(const aValue: Boolean);
begin
  FIsThreadVar := aValue;
end;

procedure TcwVariable.SetTypeIntf(const aValue: TcwSymbol);
begin
  FVarType := AValue;
end;

function TcwVariable.TypeRequired: Boolean;
begin
  Result := not IsConst;
end;

{- TcwVariablesList Category: No category }
function TcwVariablesList.AddVariable(const aVarName: string; aVisibility: TVisibilityKind; aIsConst:
  Boolean; const AFileIndex: Integer; const aX, aY: Integer): TcwVariable;
begin
  Result := TcwVariable.Create(aVarName, aVisibility, aIsConst, AFileIndex, aX, aY);
  DoSymbolAdded(Result);
end;

function TcwVariablesList.Find(AFieldName: string): TcwVariable;
begin
  Result := inherited Find(AFieldName) as TcwVariable;
end;

function TcwVariablesList.GetSortedVariables(Index: Integer): TcwVariable;
begin
  Result := SortedItem[Index] as TcwVariable;
end;

function TcwVariablesList.GetVariables(Index: Integer): TcwVariable;
begin
  Result := Items[Index] as TcwVariable;
end;

{- TcwEnumerationMemberContainer Category: No category }
function TcwEnumerationMemberContainer.TypeRequired: Boolean;
begin
  Result := True;
end;

{- TcwEnumeration Category: No category }
constructor TcwEnumeration.Create(const aName: string; aVisibility: TVisibilityKind;
  const AFileIndex, aX, aY: Integer);
begin
  inherited Create(aName, aVisibility, AFileIndex, AX, AY);
  FItems := TcwSymbolList.Create;
end;

destructor TcwEnumeration.Destroy;
begin
  FItems.Free;
  inherited;
end;

function TcwEnumeration.GetItems: TcwSymbolList;
begin
  Result := FItems;
end;

function TcwEnumeration.GetTypeKind: TcwTypeKind;
begin
  Result := tkEnum;
end;

{- TcwEnumerationList Category: No category }
function TcwEnumerationList.AddItem(const aEnumName: string; aVisibility: TVisibilityKind; const
  AFileIndex: Integer; const aX, aY: Integer): TcwEnumeration;
begin
  Result := TcwEnumeration.Create(aEnumName, aVisibility, AFileIndex, aX, aY);
  AddItem(Result);
end;

procedure TcwEnumerationList.AddItem(const AItem: TcwEnumeration);
begin
  Add(AItem);
  Container.Types.AddType(AItem as TcwSymbol);
end;

function TcwEnumerationList.GetEnumeration(Index: Integer): TcwEnumeration;
begin
  Result := Items[Index] as TcwEnumeration;
end;

function TcwEnumerationList.GetOwnsObjects: Boolean;
begin
  Result := False;
end;

{- TcwSet Category: No category }
destructor TcwSet.Destroy;
begin
  FItems.Free;
  inherited;
end;

function TcwSet.GetEnumeration: TcwEnumeration;
begin
  Result := FEnumeration;
end;

function TcwSet.GetItems: TcwSymbolList;
begin
  if Assigned(FEnumeration) then
    Result := FEnumeration.Items
  else
  begin
    if not Assigned(FItems) then
      FItems := TcwSymbolList.Create;
    Result := FItems;
  end;
end;

function TcwSet.GetTypeKind: TcwTypeKind;
begin
  Result := tkSet;
end;

procedure TcwSet.SetEnumeration(Value: TcwEnumeration);
begin
  Assert(not Assigned(FItems));
  FEnumeration := Value;
end;

{- TcwSetList Category: No category }
function TcwSetList.AddItem(const aSetName: string; aVisibility: TVisibilityKind;
  const AFileIndex: Integer; const aX, aY: Integer): TcwSet;
begin
  Result := TcwSet.Create(aSetName, aVisibility, AFileIndex, aX, aY);
  AddItem(Result);
end;

procedure TcwSetList.AddItem(const AItem: TcwSet);
begin
  Add(AItem);
  Container.Types.AddType(AItem as TcwSymbol);
end;

function TcwSetList.GetOwnsObjects: Boolean;
begin
  Result := False;
end;

function TcwSetList.GetSet(Index: Integer): TcwSet;
begin
  Result := Items[Index] as TcwSet;
end;

{- TcwArray Category: No category }
constructor TcwArray.Create(const aName: string; aVisibility: TVisibilityKind; const AFileIndex: Integer;
  const aX, aY: Integer);
begin
  inherited Create(aName, tkArray, aVisibility, AFileIndex, aX, aY, True);
end;

destructor TcwArray.Destroy;
begin
  FDimensions.Free;
  inherited Destroy;
end;

procedure TcwArray.AddDimension(const aSymbol: TcwSymbol);
begin
  if not Assigned(FDimensions) then
    FDimensions := TObjectList.Create(False);
  FDimensions.Add(aSymbol);
end;

function TcwArray.GetArrayType: TcwSymbol;
begin
  Result := FArrayType;
end;

function TcwArray.GetDimensionCount: Integer;
begin
  if Assigned(FDimensions) then
    Result := FDimensions.Count
  else
    Result := 0;
end;

function TcwArray.GetDimensions(Index: Integer): TcwSymbol;
begin
  Result := FDimensions[index] as TcwSymbol;
end;

function TcwArray.GetTypeIntf: TcwSymbol;
begin
  Result := FArrayType;
end;

procedure TcwArray.SetArrayType(const aType: TcwSymbol);
begin
  Assert(not Assigned(FArrayType));
  FArrayType := aType;
end;

function TcwArray.TypeRequired: Boolean;
begin
  Result := True;
end;

{- TcwArrayList Category: No category }
function TcwArrayList.AddArray(const aArrayName: string; aVisibility: TVisibilityKind; const AFileIndex:
  Integer; const aX, aY: Integer): TcwArray;
begin
  Result := TcwArray.Create(aArrayName, aVisibility, AFileIndex, aX, aY);
  Add(Result);
  Container.Types.AddType(Result as TcwSymbol);
end;

function TcwArrayList.GetArray(Index: Integer): TcwArray;
begin
  Result := Items[Index] as TcwArray;
end;

function TcwArrayList.GetOwnsObjects: Boolean;
begin
  Result := False;
end;

{- TcwLabel Category: No category }
constructor TcwLabel.Create(const aName: string; aIsIntLabel: Boolean; const AFileIndex: Integer; const
  aX, aY: Integer);
begin
  inherited Create(aName, vkPublic, tkNone, AFileIndex, aX, aY);
  FIsIntLabel := aIsIntLabel;
end;

function TcwLabel.GetIsIntLabel: Boolean;
begin
  Result := FIsIntLabel;
end;

function TcwLabel.TypeRequired: Boolean;
begin
  Result := False;
end;

{- TcwLabelsList Category: No category }
function TcwLabelsList.AddLabel(const aLabelName: string; aIsIntLabel: Boolean; const AFileIndex:
  Integer; const aX, aY: Integer): TcwLabel;
begin
  Result := TcwLabel.Create(aLabelName, aIsIntLabel, AFileIndex, aX, aY);
  Add(Result);
end;

function TcwLabelsList.GetLabels(Index: Integer): TcwLabel;
begin
  Result := Items[Index] as TcwLabel;
end;

{- TcwVarContainer Category: No category }
destructor TcwVarContainer.Destroy;
begin
  FEnumerationList.Free;
  FSetList.Free;
  FArraysList.Free;
  FVariableList.Free;
  FConstantList.Free;
  FLabels.Free;
  FTypes.Free;
  FRoutineList.Free;
  FRecordList.Free;
  FClassList.Free;
  FObjectList.Free;
  FInterfaceList.Free;
  FPointerTypeList.Free;
  inherited;
end;

procedure TcwVarContainer.AddForwardPointerType(AType: TcwType);
begin
  if not Assigned(FPointerTypeList) then
    FPointerTypeList := TcwHashedObjectList.Create;
  FPointerTypeList.Add(AType.PointerTypeName, AType);
end;

function TcwVarContainer.FindForwardPointerType(const AName: string): TcwType;
var
  AIndex: Integer;
begin
  Result := nil;
  if Assigned(FPointerTypeList) and FPointerTypeList.Find(AName, AIndex) then
    Result := FPointerTypeList[AIndex] as TcwType
end;

function TcwVarContainer.GetArrays: TcwArrayList;
begin
  if not Assigned(FArraysList) then
    FArraysList := TcwArrayList.Create(Self);
  Result := FArraysList;
end;

function TcwVarContainer.GetClasses: TcwClassList;
begin
  if not Assigned(FClassList) then
    FClassList := TcwOwnedClassList.Create(Self);
  Result := FClassList;
end;

function TcwVarContainer.GetConstants: TcwVariablesList;
begin
  if not Assigned(FConstantList) then
    FConstantList := TcwVariablesList.Create(Self);
  Result := FConstantList;
end;

function TcwVarContainer.GetEnumerations: TcwEnumerationList;
begin
  if not Assigned(FEnumerationList) then
    FEnumerationList := TcwEnumerationList.Create(Self);
  Result := FEnumerationList;
end;

function TcwVarContainer.GetInterfaces: TcwInterfaceList;
begin
  if not Assigned(FInterfaceList) then
    FInterfaceList := TcwOwnedInterfaceList.Create(Self);
  Result := FInterfaceList;
end;

function TcwVarContainer.GetLabels: TcwLabelsList;
begin
  if not Assigned(FLabels) then
    FLabels := TcwLabelsList.Create;
  Result := FLabels;
end;

function TcwVarContainer.GetObjects: TcwClassList;
begin
  if not Assigned(FObjectList) then
    FObjectList := TcwOwnedObjectList.Create(Self);
  Result := FObjectList;
end;

function TcwVarContainer.GetRecords: TcwRecordsList;
begin
  if not Assigned(FRecordList) then
    FRecordList := TcwRecordsList.Create(Self);
  Result := FRecordList;
end;

function TcwVarContainer.GetRoutines: TcwRoutineList;
begin
  if not Assigned(FRoutineList) then
    FRoutineList := TcwRoutineList.Create(Self);
  Result := FRoutineList;
end;

function TcwVarContainer.GetSets: TcwSetList;
begin
  if not Assigned(FSetList) then
    FSetList := TcwSetList.Create(Self);
  Result := FSetList;
end;

function TcwVarContainer.GetTypes: TcwTypesList;
begin
  if not Assigned(FTypes) then
    FTypes := TcwTypesList.Create(Self);
  Result := FTypes;
end;

function TcwVarContainer.GetVariables: TcwVariablesList;
begin
  if not Assigned(FVariableList) then
    FVariableList := TcwVariablesList.Create(Self);
  Result := FVariableList;
end;

{- TcwRecord Category: No category }
constructor TcwRecord.Create(const aName: string; aVisibility: TVisibilityKind; const AFileIndex:
  Integer; const aX, aY: Integer);
begin
  inherited Create(aName, aVisibility, AFileIndex, aX, aY);
  FList := TcwVariablesList.Create(Self);
end;

destructor TcwRecord.Destroy;
begin
  FList.Free;
  inherited;
end;

function TcwRecord.GetFieldList: TcwVariablesList;
begin
  Result := FList;
end;

function TcwRecord.GetTypeIntf: TcwSymbol;
begin
  Result := Self;
end;

function TcwRecord.GetTypeKind: TcwTypeKind;
begin
  Result := tkRecord;
end;

procedure TcwRecord.SetTypeIntf(const aValue: TcwSymbol);
begin
  Assert(False);
end;

procedure TcwRecord.SetTypeKind(const aValue: TcwTypeKind);
begin
  Assert(False);
end;

function TcwRecord.TypeRequired: Boolean;
begin
  Result := True;
end;

{- TcwRecordsList Category: No category }
function TcwRecordsList.AddRecord(const aRecordName: string; aVisibility: TVisibilityKind;
  const AFileIndex: Integer; const aX, aY: Integer): TcwRecord;
begin
  Result := TcwRecord.Create(aRecordName, aVisibility, AFileIndex, aX, aY);
  DoTypeAdded(Result);
end;

function TcwRecordsList.Find(const aRecordName: string): TcwRecord;
begin
  Result := inherited Find(aRecordName) as TcwRecord;
end;

function TcwRecordsList.GetRecords(Index: Integer): TcwRecord;
begin
  Result := Items[Index] as TcwRecord;
end;

function TcwRecordsList.GetSortedRecords(Index: Integer): TcwRecord;
begin
  Result := SortedItem[Index] as TcwRecord;
end;

function TcwRecordsList.GetOwnsObjects: Boolean;
begin
  Result := False;
end;

{- TcwProperty Category: No category }
constructor TcwProperty.Create(const aName: string; aVisibility: TVisibilityKind; const AOwnerClass:
  TcwClass; const AFileIndex, aX, aY: Integer);
begin
  inherited Create(aName, aVisibility, False, AFileIndex, aX, aY);
  FOwnerClass := AOwnerClass;
end;

function TcwProperty.GetDefaultProperty: Boolean;
begin
  Result := FDefaultProperty;
end;

function TcwProperty.GetIsArrayProperty: Boolean;
begin
  Result := FIsArrayProperty;
end;

function TcwProperty.GetOwnerClass: TcwClass;
begin
  Result := FOwnerClass;
end;

function TcwProperty.GetReadAccess: TcwSymbol;
begin
  Result := FReadAccess;
end;

function TcwProperty.GetWriteAccess: TcwSymbol;
begin
  Result := FWriteAccess;
end;

procedure TcwProperty.SetDefaultProperty(const Value: Boolean);
begin
  FDefaultProperty := Value;
  if FDefaultProperty then
    OwnerClass.DefaultProperty := Self;
end;

procedure TcwProperty.SetIsArrayProperty(const Value: Boolean);
begin
  FIsArrayProperty := Value;
end;

procedure TcwProperty.SetReadAccess(const aValue: TcwSymbol);
begin
  FReadAccess := aValue;
end;

procedure TcwProperty.SetWriteAccess(const aValue: TcwSymbol);
begin
  FWriteAccess := aValue;
end;

{- TcwPropertyList Category: No category }
function TcwPropertyList.AddProperty(const aPropertyName: string; aVisibility: TVisibilityKind; const
  AOwnerClass: TcwClass; const AFileIndex: Integer; const aX, aY: Integer): TcwProperty;
begin
  Result := TcwProperty.Create(aPropertyName, aVisibility, AOwnerClass, AFileIndex,
    aX, aY);
  DoSymbolAdded(Result);
end;

function TcwPropertyList.Find(const aPropName: string): TcwProperty;
begin
  Result := inherited Find(aPropName) as TcwProperty;
end;

function TcwPropertyList.GetProperties(Index: Integer): TcwProperty;
begin
  Result := Items[Index] as TcwProperty;
end;

function TcwPropertyList.GetSortedProperties(Index: Integer): TcwProperty;
begin
  Result := SortedItem[Index] as TcwProperty;
end;

{- TcwClass Category: No category }
constructor TcwClass.Create(const aName: string; const aVisibility: TVisibilityKind;
  const AFileIndex: Integer; const aX, aY: Integer);
begin
  inherited Create(aName, aVisibility, AFileIndex, aX, aY);
  FDescendants := TObjectList.Create(False);
  FPropertyList := TcwPropertyList.Create(Self);
end;

destructor TcwClass.Destroy;
begin
  FDescendants.Free;
  FPropertyList.Free;
  inherited;
end;

procedure TcwClass.AddVisibilitySpecifier(aCodeLocationLine: Integer; aVisibility: TVisibilityKind);
begin
  if FVisibilitySpecifiersSize <= FVisibilitySpecifiersCount then
  begin
    Inc(FVisibilitySpecifiersSize, 10);
    SetLength(FVisibilitySpecifiers, FVisibilitySpecifiersSize);
  end;
  FVisibilitySpecifiers[FVisibilitySpecifiersCount].CodeLine := aCodeLocationLine;
  FVisibilitySpecifiers[FVisibilitySpecifiersCount].Visibility := aVisibility;
  Inc(FVisibilitySpecifiersCount);
end;

function TcwClass.FindSymbol(const AFromUnit: TcwUnit; const aFromClass: TcwClass;
  const aName: string; const AAddUsedBy: Boolean; AUsedByInfo: TcwUsedByInfo): TcwSymbol;
var
  Index: Integer;
begin
  Result := nil;

  if HashedSymbols.Find(aName, Index) then
    Result := HashedSymbols[Index] as TcwSymbol
  else
    if Assigned(FInheritsFrom) then
      Result := InheritsFromClass.FindSymbol(AFromUnit, aFromClass, aName, AAddUsedBy,
        AUsedByInfo);

  //check if the symbol is visible to the class calling :
  if Assigned(Result) then
    if Result.Visibilty = vkProtected then
    begin
      //protected members are only visible to routines in the same unit, or if the calling class descends
      //from the current class
      if Assigned(AFromClass) and (AFromUnit.UnitIndex <> UnitIndex) and
        not (ComparesTo(AFromClass) or IsDescendant(aFromClass)) then
        Result := nil;
    end else
      if Result.Visibilty = vkPrivate then
      begin
        //private members are only visible in the same unit
        if AFromUnit.UnitIndex <> UnitIndex then
          Result := nil;
      end;

  if AAddUsedBy and Assigned(Result) then
  begin
    AUsedByInfo.IsWrongCase := AName <> Result.Name;
    Result.UsedBy.AddUsedBy(AUsedByInfo);
  end;
end;

function TcwClass.GetDefaultProperty: TcwProperty;
begin
  if Assigned(FDefaultProperty) then
    Result := TcwProperty(FDefaultProperty)
  else
    if Assigned(InheritsFromClass) then
      Result := InheritsFromClass.DefaultProperty
    else
      Result := nil;
end;

function TcwClass.GetDefaultVisibility: TVisibilityKind;
begin
  Result := vkPublished;
end;

function TcwClass.GetDescendants: TObjectList;
begin
  Result := FDescendants;
end;

function TcwClass.GetFieldList: TcwVariablesList;
begin
  Result := Variables;
end;

function TcwClass.GetInheritsFromClass: TcwClass;
begin
  Result := FInheritsFrom;
end;

procedure TcwClass.GetMembersWithVisibility(out aMemberList: TcwSymbolList;
  aScope: TVisibilityKindSet);
var
  i: Integer;
begin
  aMemberList := TcwSymbolList.Create(False);
  for i := 0 to Fields.Count - 1 do
    if Fields[i].Visibilty in aScope then
      aMemberList.AddSymbol(Fields[i]);

  for i := 0 to Routines.Count - 1 do
    if Routines[i].Visibilty in aScope then
      aMemberList.AddSymbol(Routines[i]);

  for i := 0 to Properties.Count - 1 do
    if Properties[i].Visibilty in aScope then
      aMemberList.AddSymbol(Properties[i]);
end;

function TcwClass.GetProperties: TcwPropertyList;
begin
  Result := FPropertyList;
end;

function TcwClass.GetTypeIntf: TcwSymbol;
begin
  Result := Self;
end;

function TcwClass.GetTypeKind: TcwTypeKind;
begin
  Result := tkClass;
end;

function TcwClass.GetVisibilitySpecifierCount: Integer;
begin
  Result := FVisibilitySpecifiersCount;
end;

function TcwClass.GetVisibilitySpecifierLine(aIndex: Integer): Integer;
begin
  if InRange(aIndex, 0, FVisibilitySpecifiersCount-1)  then
    Result := FVisibilitySpecifiers[aIndex].CodeLine
  else
    raise EListError.CreateFmt(SListIndexError, [aIndex]);
end;

function TcwClass.GetVisibilitySpecifierVisibility(aIndex: Integer): TVisibilityKind;
begin
  if InRange(aIndex, 0, FVisibilitySpecifiersCount-1) then
    Result := FVisibilitySpecifiers[aIndex].Visibility
  else
    raise EListError.CreateFmt(SListIndexError, [aIndex]);
end;

function TcwClass.IsDescendant(const aClass: TcwClass): Boolean;
var
  i: Integer;
  vDescendant: TcwClass;
begin
  Result := False;
  if Assigned(aClass) then
    for i := 0 to Descendants.Count - 1 do
    begin
      vDescendant := Descendants[i] as TcwClass;
      if (vDescendant.ComparesTo(aClass)) or (vDescendant.IsDescendant(aClass)) then
      begin
        Result := True;
        Break;
      end;
    end;
end;

function TcwClass.IsSymbolFromInterface(const aSymbolName: string): Boolean;
var
  i: Integer;
  vInterface: TcwInterface;
begin
  Result := False;
  for i := 0 to Interfaces.Count - 1 do
  begin
    vInterface := Interfaces[i];
    if vInterface.FindSymbol(vInterface.UnitIntf, vInterface, aSymbolName, False,
      UsedByInfo(-1, False, -1, -1)) <> nil then
    begin
      Result := True;
      Exit;
    end
  end;
end;

procedure TcwClass.SetDefaultProperty(const Value: TcwProperty);
begin
  FDefaultProperty := Value;
end;

procedure TcwClass.SetInheritsFromClass(const aValue: TcwClass);
begin
  Assert(not Assigned(FInheritsFrom));
  FInheritsFrom := aValue;
end;

procedure TcwClass.SetTypeIntf(const aValue: TcwSymbol);
begin
  Assert(False);
end;

procedure TcwClass.SetTypeKind(const aValue: TcwTypeKind);
begin
  Assert(False);
end;

function TcwClass.TypeRequired: Boolean;
begin
  Result := True;
end;

{- TcwClassList Category: No category }
function TcwClassList.AddClass(const aName: string; const aVisibility: TVisibilityKind;
  const AFileIndex: Integer; const aX, aY: Integer): TcwClass;
begin
  Result := TcwClass.Create(aName, aVisibility, AFileIndex, aX, aY);
  AddClass(Result);
end;

procedure TcwClassList.AddClass(const aClass: TcwClass);
begin
  DoSymbolAdded(AClass);
end;

function TcwClassList.Find(const aClassName: string): TcwClass;
begin
  Result := inherited Find(aClassName) as TcwClass;
end;

function TcwClassList.GetClasses(Index: Integer): TcwClass;
begin
  Result := Items[Index] as TcwClass;
end;

function TcwClassList.GetSortedClasses(Index: Integer): TcwClass;
begin
  Result := SortedItem[Index] as TcwClass;
end;

function TcwClassList.GetOwnsObjects: Boolean;
begin
  Result := True;
end;

{- TcwOwnedClassList Category: No category }
function TcwOwnedClassList.GetOwnsObjects: Boolean;
begin
  Result := False;
end;

procedure TcwOwnedClassList.AddClass(const aClass: TcwClass);
begin
  DoTypeAdded(AClass);
end;

{- TcwObject Category: No category }
function TcwObject.GetDefaultVisibility: TVisibilityKind;
begin
  Result := vkPublic;
end;

function TcwObject.GetTypeKind: TcwTypeKind;
begin
  Result := tkObject;
end;

{- TcwObjectList Category: No category }
function TcwObjectList.AddClass(const aName: string; const aVisibility: TVisibilityKind;
  const AFileIndex: Integer; const aX, aY: Integer): TcwClass;
begin
  Result := TcwObject.Create(aName, aVisibility, AFileIndex, aX, aY);
  AddClass(Result);
end;

{- TcwOwnedObjectList Category: No category }
function TcwOwnedObjectList.AddClass(const aName: string; const aVisibility: TVisibilityKind;
  const AFileIndex: Integer; const aX, aY: Integer): TcwClass;
begin
  Result := TcwObject.Create(aName, aVisibility, AFileIndex, aX, aY);
  AddClass(Result);
end;

{- TcwInterface Category: No category }
function TcwInterface.GetDefaultVisibility: TVisibilityKind;
begin
  Result := vkPublic;
end;

function TcwInterface.GetGUID: string;
begin
  Result := FGUID;
end;

function TcwInterface.GetIsDispatchInterface: Boolean;
begin
  Result := FIsDispatchInterface;
end;

function TcwInterface.GetTypeKind: TcwTypeKind;
begin
  Result := tkInterface;
end;

procedure TcwInterface.SetGUID(const aValue: string);
begin
  FGUID := aValue;
end;

procedure TcwInterface.SetIsDispatchInterface(const aValue: Boolean);
begin
  FIsDispatchInterface := aValue;
end;

{- TcwInterfaceList Category: No category }
function TcwInterfaceList.GetOwnsObjects: Boolean;
begin
  Result := True;
end;

procedure TcwInterfaceList.AddInterface(const aInterface: TcwInterface);
begin
  DoSymbolAdded(aInterface);
end;

function TcwInterfaceList.AddInterface(const aName: string; const aVisibility: TVisibilityKind;
  const AFileIndex: Integer; const aX, aY: Integer): TcwInterface;
begin
  Result := TcwInterface.Create(aName, aVisibility, AFileIndex, aX, aY);
  AddInterface(Result);
end;

function TcwInterfaceList.Find(const aInterfaceName: string): TcwInterface;
begin
  Result := inherited Find(aInterfaceName) as TcwInterface;
end;

function TcwInterfaceList.GetInterfaces(Index: Integer): TcwInterface;
begin
  Result := Items[Index] as TcwInterface;
end;

function TcwInterfaceList.GetSortedInterfaces(Index: Integer): TcwInterface;
begin
  Result := SortedItem[Index] as TcwInterface;
end;

{- TcwOwnedInterfaceList Category: No category }
function TcwOwnedInterfaceList.GetOwnsObjects: Boolean;
begin
  Result := False;
end;

procedure TcwOwnedInterfaceList.AddInterface(const aInterface: TcwInterface);
begin
  DoTypeAdded(aInterface);
end;

{- TcwParameter Category: No category }
constructor TcwParameter.Create(const aParamName: string; aParamKind: TParamKind; const AFileIndex:
  Integer; const aX, aY: Integer);
begin
  inherited Create(aParamName, vkNone, tkNone, AFileIndex, aX, aY);
  FParamKind := aParamKind;
end;

function TcwParameter.GetParamKind: TParamKind;
begin
  Result := FParamKind;
end;

function TcwParameter.GetParamType: TcwSymbol;
begin
  Result := FParamType;
end;

function TcwParameter.GetTypeIntf: TcwSymbol;
begin
  Result := FParamType;
end;

function TcwParameter.GetUntypedParameter: Boolean;
begin
  Result := FUntypedParameter;
end;

procedure TcwParameter.SetParamType(const aType: TcwSymbol);
begin
  FParamType := aType;
end;

procedure TcwParameter.SetUntypedParameter(const Value: Boolean);
begin
  FUntypedParameter := Value;
end;

function TcwParameter.TypeRequired: Boolean;
begin
  Result := not UntypedParameter;
end;

{- TcwParameterList Category: No category }
function TcwParameterList.AddParameter(const aParamName: string; aParamKind: TParamKind; const
  AFileIndex: Integer; const aX, aY: Integer): TcwParameter;
begin
  Result := TcwParameter.Create(aParamName, aParamKind, AFileIndex, aX, aY);
  DoSymbolAdded(Result);
end;

function TcwParameterList.Find(const aParamName: string): TcwParameter;
begin
  Result := inherited Find(aParamName) as TcwParameter;
end;

function TcwParameterList.GetParameters(Index: Integer): TcwParameter;
begin
  Result := Items[Index] as TcwParameter;
end;

{- TcwRoutine Category: No category }
constructor TcwRoutine.Create(const ARoutineList: TcwRoutineList; const aName: string; aVisibility:
  TVisibilityKind; aRoutineKind: TRoutineKind; const AFileIndex: Integer; const aX, aY: Integer);
begin
  inherited Create(aName, aVisibility, AFileIndex, aX, aY);
  FRoutineList := ARoutineList;
  FRoutineKind := aRoutineKind;
  FParameters := TcwParameterList.Create(Self);
  FDirectives := [];
end;

destructor TcwRoutine.Destroy;
begin
  FParameters.Free;
  FRoutineType.Free;
  FReturnType.Free;
  inherited;
end;

procedure TcwRoutine.AddDirective(const aDirective: TRoutineDirective);
begin
  Include(FDirectives, aDirective);
end;

function TcwRoutine.GetDirectives: TRoutineDirectives;
begin
  Result := FDirectives;
end;

function TcwRoutine.GetImplEndX: Integer;
begin
  Result := FImplEnd.X;
end;

function TcwRoutine.GetImplEndY: Integer;
begin
  Result := FImplEnd.Y;
end;

function TcwRoutine.GetImplStartX: Integer;
begin
  Result := FImplStart.X;
end;

function TcwRoutine.GetImplStartY: Integer;
begin
  Result:=FImplStart.Y;
end;

function TcwRoutine.GetIsUsedFromDFM: Boolean;
var
  vUnit: TcwUnit;
begin
  vUnit := UnitIntf;
  Result := (vUnit.DFMResource <> nil) and
    (vUnit.DFMResource.HasPropertyWithIdentifier(Name));
end;

function TcwRoutine.GetParameters: TcwParameterList;
begin
  Result := FParameters;
end;

function TcwRoutine.GetReturnType: TcwVariable;
begin
  Result := FReturnType;
end;

function TcwRoutine.GetRoutineKind: TRoutineKind;
begin
  Result := FRoutineKind;
end;

function TcwRoutine.GetRoutineList: TcwRoutineList;
begin
  Result := FRoutineList;
end;

function TcwRoutine.GetRoutineType: TcwVariable;
begin
  Result := FRoutineType;
end;

function TcwRoutine.GetTypeIntf: TcwSymbol;
begin
  if ReturnType <> nil then
    Result := ReturnType.TypeIntf
  else
    Result := nil;
end;

procedure TcwRoutine.SetImplEndX(aValue: Integer);
begin
  FImplEnd.X := aValue;
end;

procedure TcwRoutine.SetImplEndY(aValue: Integer);
begin
  FImplEnd.Y := aValue;
end;

procedure TcwRoutine.SetImplStartX(aValue: Integer);
begin
  FImplStart.X := aValue;
end;

procedure TcwRoutine.SetImplStartY(aValue: Integer);
begin
  FImplStart.Y := aValue;
end;

procedure TcwRoutine.SetReturnType(const aValue: TcwVariable);
begin
  FReturnType := aValue;
end;

procedure TcwRoutine.SetRoutineType(const aValue: TcwVariable);
begin
  FRoutineType := aValue;
end;

function TcwRoutine.TypeRequired: Boolean;
begin
  Result := RoutineKind in [rkFunction];
end;

{- TcwRoutineList Category: No category }
function TcwRoutineList.AddRoutine(const aName: string; aVisibility: TVisibilityKind; aRoutineKind:
  TRoutineKind; const AFileIndex: Integer; const aX, aY: Integer): TcwRoutine;
begin
  Result := TcwRoutine.Create(Self, aName, aVisibility, aRoutineKind, AFileIndex,
    aX, aY);
  DoSymbolAdded(Result);
end;

function TcwRoutineList.Find(const aRoutineName: string): TcwRoutine;
begin
  Result := inherited Find(aRoutineName) as TcwRoutine;
end;

function TcwRoutineList.GetRoutine(Index: Integer): TcwRoutine;
begin
  Result := Items[Index] as TcwRoutine;
end;

function TcwRoutineList.GetSortedRoutines(Index: Integer): TcwRoutine;
begin
  Result := SortedItem[Index] as TcwRoutine;
end;

{- TcwCodeError Category: No category }
constructor TcwCodeError.Create(const aError: TCodeErrorKind; const AFileIndex: Integer; const aX, aY:
  Integer);
begin
  inherited Create(AFileIndex, aX, aY);
  FErrorKind := aError;
end;

function TcwCodeError.GetErrorKind: TCodeErrorKind;
begin
  Result := FErrorKind;
end;

{- TcwCodingErrorsList Category: No category }
constructor TcwCodingErrorsList.Create;
begin
  inherited;
  FList := TObjectList.Create;
end;

destructor TcwCodingErrorsList.Destroy;
begin
  FList.Free;
  inherited;
end;

function TcwCodingErrorsList.AddError(const aError: TCodeErrorKind; const AFileIndex: Integer; const aX,
  aY: Integer): TcwCodeError;
begin
  Result := TcwCodeError.Create(aError, AFileIndex, aX, aY);
  FList.Add(Result);
end;

procedure TcwCodingErrorsList.Clear;
begin
  FList.Clear;
end;

function TcwCodingErrorsList.GetCount: Integer;
begin
  Result := FList.Count;
end;

function TcwCodingErrorsList.GetItems(Index: Integer): TcwCodeError;
begin
  Result := FList[Index] as TcwCodeError;
end;

{- TcwUsedUnit }
constructor TcwUsedUnit.Create(const AUnit: TcwUnit);
begin
  inherited Create;
  FUnit := AUnit;
  FUsedList := TcwUsedByList.Create(FUnit);
end;

destructor TcwUsedUnit.Destroy;
begin
  FUsedList.Free;
  inherited;
end;

{- TcwUsedUnitList }
function TcwUsedUnitList.GetSortedUsedUnit(Index: Integer): TcwUsedUnit;
begin
  Result := SortedItem[Index] as TcwUsedUnit;
end;

procedure TcwUsedUnitList.AddUsedUnit(const AUsedUnit: TcwUsedUnit);
begin
  DoItemAdded(AUsedUnit.UnitIntf.Name, AUsedUnit);
end;

function TcwUsedUnitList.FindUsedUnit(AUnitName: string): TcwUsedUnit;
begin
  Result := inherited Find(AUnitName) as TcwUsedUnit;
end;

function TcwUsedUnitList.GetUsedUnit(Index: Integer): TcwUsedUnit;
begin
  Result := Items[Index] as TcwUsedUnit;
end;

{- TcwUnit Category: No category }
constructor TcwUnit.Create(const AFileIndex: Integer; const aName: string);
begin
  FUsedByLocations := TObjectList.Create(False);
  inherited Create(aName, vkInterface, AFileIndex, 0, 0);
  FParsed := psrNone;
  FUsedUnitList := TcwUsedUnitList.Create;
  FStringLiteralsList := TcwSymbolList.Create;
  FCodingErrorsList := TcwCodingErrorsList.Create;
  FDFMResource := TcwDFMResource.Create;
end;

destructor TcwUnit.Destroy;
begin
  FUsedByLocations.Free;
  FUsedUnitList.Free;
  FCodingErrorsList.Free;
  FStringLiteralsList.Free;
  FDFMResource.Free;
  inherited;
end;

function TcwUnit.FindReferencedSymbol(aFromClass: TcwClass; const aName: string;
  const AAddUsedBy: Boolean; AUsedByInfo: TcwUsedByInfo): TcwSymbol;
var
  i: Integer;
begin
  Result := FindSymbol(Self, aFromClass, aName, AAddUsedBy, AUsedByInfo);
  if Result <> nil then
    Exit;
  for i := UsedUnits.Count - 1 downto 0 do
  begin
    Result := UsedUnits[i].UnitIntf.FindSymbol(Self, aFromClass, aName, AAddUsedBy,
      AUsedByInfo);
    if Result <> nil then
      Break;
  end;
  if AAddUsedBy and Assigned(Result) then
  begin
    AUsedByInfo.IsWrongCase := AName <> Result.Name;
    Result.UsedBy.AddUsedBy(AUsedByInfo);
  end;
end;

function TcwUnit.FindSymbol(const AFromUnit: TcwUnit; const aFromClass: TcwClass;
  const aName: string; const AAddUsedBy: Boolean; AUsedByInfo: TcwUsedByInfo): TcwSymbol;
var
  ASymbol: TcwSymbol;
  Index: Integer;
begin
  Result := nil;
  if HashedSymbols.Find(aName, Index) then
  begin
    while (Index <= HashedSymbols.Count - 1) and
      SameText(HashedSymbols.Strings[Index], AName) do
    begin
      ASymbol := HashedSymbols[Index] as TcwSymbol;
      Assert(Assigned(ASymbol));
      // visibility = vkNone could mean that this symbol is part of a sub-procedure,
      // so it is not visible in the unit's other routines
      if (ASymbol.Visibilty <> vkNone) and
        ((AFromUnit.UnitIndex = UnitIndex) or (ASymbol.Visibilty <> vkImplementation)) then
        Result := ASymbol;
      Inc(Index);
    end;
  end;
  if AAddUsedBy and Assigned(Result) then
  begin
    AUsedByInfo.IsWrongCase := AName <> Result.Name;
    Result.UsedBy.AddUsedBy(AUsedByInfo);
  end;
end;

function TcwUnit.GetCodingErrorsList: TcwCodingErrorsList;
begin
  Result := FCodingErrorsList;
end;

function TcwUnit.GetDFMResource: TcwDFMResource;
begin
  Result := FDFMResource;
end;

function TcwUnit.GetFullFileName: string;
begin
  Result := gcwProject.FileNames[FileIndex].FullFileName;
end;

function TcwUnit.GetImplementationLine: Integer;
begin
  Result:=FImplementationLine;
end;

function TcwUnit.GetInterfaceLine: Integer;
begin
  Result:=FInterfaceLine;
end;

function TcwUnit.GetStringLiterals: TcwSymbolList;
begin
  Result := FStringLiteralsList;
end;

function TcwUnit.GetUnitType: TDirectoryType;
begin
  Result := FUnitType;
end;

function TcwUnit.GetUsedByLocations: TObjectList;
begin
  Result := FUsedByLocations;
end;

function TcwUnit.GetUsedUnits: TcwUsedUnitList;
begin
  Result := FUsedUnitList;
end;

function TcwUnit.IsLibraryUnit: Boolean;
begin
  Result := (UnitType = dtLibrary) or (Parsed <> psrFull);
end;

procedure TcwUnit.LoadFormResource(const aDFMFileName: string);
begin
  FDFMFileName := aDFMFileName;
end;

procedure TcwUnit.SetImplementationLine(aValue: Integer);
begin
  FImplementationLine:=aValue;
end;

procedure TcwUnit.SetInterfaceLine(aValue: Integer);
begin
  FInterfaceLine:=aValue;
end;

procedure TcwUnit.SetUnitType(const aValue: TDirectoryType);
begin
  FUnitType := aValue;
end;

function TcwUnit.TypeRequired: Boolean;
begin
  Result := False;
end;

{- TcwComponentProperty Category: No category }
constructor TcwComponentProperty.Create(const aName : string);
begin
  inherited Create;
  FName := aName;
end;

function TcwComponentProperty.GetName: string;
begin
  Result := FName;
end;

function TcwComponentProperty.GetValue: Variant;
begin
  Result := FValue;
end;

function TcwComponentProperty.GetValueType: TDFMPropType;
begin
  Result := FValueType;
end;

procedure TcwComponentProperty.SetValue(const aValue: Variant);
begin
  FValue := aValue;
end;

procedure TcwComponentProperty.SetValueType(const aValue: TDFMPropType);
begin
  FValueType := aValue;
end;

{- TcwComponentPropertiesList Category: No category }
constructor TcwComponentPropertiesList.Create;
begin
  inherited Create;
  FList := TObjectList.Create;
  FSortedList := TcwHashedObjectList.Create;
  FSortedIdentifiers := TcwHashedObjectList.Create;
end;

destructor TcwComponentPropertiesList.Destroy;
begin
  FList.Free;
  FSortedList.Free;
  FSortedIdentifiers.Free;
  inherited Destroy;
end;

procedure TcwComponentPropertiesList.AddIdentifierName(const aIdentName : string;const aProperty :
  TcwComponentProperty);
begin
  FSortedIdentifiers.Add(aIdentName, aProperty);
end;

function TcwComponentPropertiesList.AddProperty(const aName : string): TcwComponentProperty;
begin
  Result := TcwComponentProperty.Create(aName);
  FList.Add(Result);
  FSortedList.Add(aName, Result);
end;

function TcwComponentPropertiesList.GetCount: Integer;
begin
  Result := FList.Count;
end;

function TcwComponentPropertiesList.GetProperties(Index : Integer): TcwComponentProperty;
begin
  Result := TcwComponentProperty(FList[Index]);
end;

function TcwComponentPropertiesList.HasProperty(const aPropName : string): Boolean;
var
  Index: Integer;
begin
  Result := FSortedList.Find(aPropName,Index);
end;

function TcwComponentPropertiesList.HasWithIdentifier(const aIdentName : string): Boolean;
var
  Index: Integer;
begin
  Result := FSortedIdentifiers.Find(aIdentName, Index);
end;

{- TcwComponent Category: No category }
constructor TcwComponent.Create(const aName,aClassName : string);
begin
  inherited Create;
  FName := aName;
  FClassName := aClassName;
  FProperties := TcwComponentPropertiesList.Create;
  FChildComponents := TcwComponentList.Create;
end;

destructor TcwComponent.Destroy;
begin
  FProperties.Free;
  FChildComponents.Free;
  inherited;
end;

function TcwComponent.GetCClassName: string;
begin
  Result := FClassName;
end;

function TcwComponent.GetChildComponents: TcwComponentList;
begin
  Result := FChildComponents;
end;

function TcwComponent.GetIsInherited: Boolean;
begin
  Result := FIsInherited;
end;

function TcwComponent.GetIsInline: Boolean;
begin
  Result := FIsInline;
end;

function TcwComponent.GetName: string;
begin
  Result := FName;
end;

function TcwComponent.GetPosition: Integer;
begin
  Result := FPosition;
end;

function TcwComponent.GetProperties: TcwComponentPropertiesList;
begin
  Result := FProperties;
end;

function TcwComponent.HasProperty(const aPropName : string): Boolean;
var
  i: Integer;
begin
  Result := Properties.HasProperty(aPropName);
  if Result = False then
    for i := 0 to ChildComponents.Count - 1 do
    begin
      Result := ChildComponents[i].Properties.HasProperty(aPropName);
      if Result then
        Exit
    end;
end;

function TcwComponent.HasPropertyWithIdentifier(const aIdentName : string): Boolean;
var
  i: Integer;
begin
  Result := Properties.HasWithIdentifier(aIdentName);
  if not Result  then
    for i := 0 to ChildComponents.Count - 1 do
    begin
      Result := ChildComponents[i].HasPropertyWithIdentifier(aIdentName);
      if Result then
        Exit;
    end;
end;

procedure TcwComponent.SetIsInherited(const aValue: Boolean);
begin
  FIsInherited := aValue;
end;

procedure TcwComponent.SetIsInline(const aValue: Boolean);
begin
  FIsInline := aValue;
end;

procedure TcwComponent.SetPosition(const aValue: Integer);
begin
  FPosition := aValue;
end;

{- TcwComponentList Category: No category }
constructor TcwComponentList.Create;
begin
  inherited Create;
  FList := TObjectList.Create;
end;

destructor TcwComponentList.Destroy;
begin
  FList.Free;
  inherited Destroy;
end;

function TcwComponentList.AddComponent(const aName : string; const aClassName : string): TcwComponent;
begin
  Result := TcwComponent.Create(aName,aClassName);
  FList.Add(Result);
end;

function TcwComponentList.GetComponents(Index : Integer): TcwComponent;
begin
  Result := TcwComponent(FList[Index]);
end;

function TcwComponentList.GetCount: Integer;
begin
  Result := FList.Count;
end;

{- TcwDFMResource Category: No category }
constructor TcwDFMResource.Create;
begin
  inherited Create;
  FClassList := TcwComponentList.Create;
end;

destructor TcwDFMResource.Destroy;
begin
  FClassList.Free;
  inherited Destroy;
end;

function TcwDFMResource.GetComponents: TcwComponentList;
begin
  Result := FClassList;
end;

function TcwDFMResource.GetFileName: string;
begin
  Result := FFileName;
end;

function TcwDFMResource.GetUnitIndex: Integer;
begin
  Result := FUnitIndex;
end;

function TcwDFMResource.HasProperty(const aPropName: string): Boolean;
var
  i: Integer;
begin
  Result := False;
  for i := 0 to Components.Count - 1 do
  begin
    Result := Components[i].HasProperty(aPropName);
    if Result then
      Exit;
  end;
end;

function TcwDFMResource.HasPropertyWithIdentifier(const aIdentName: string): Boolean;
var
  i: Integer;
begin
  Result := False;
  for i := 0 to Components.Count - 1 do
  begin
    Result := Components[i].HasPropertyWithIdentifier(aIdentName);
    if Result then
      Exit;
  end;
end;

procedure TcwDFMResource.ReadDFM(const aUnit: TcwUnit; const aInStream: TStream);
var
  SaveSeparator: Char;
  Parser: TParser;
  FormClass: TcwClass;

  function ReadOrderModifier: Integer;
  begin
    Result := -1;
    if Parser.Token = '[' then
    begin
      Parser.NextToken;
      Parser.CheckToken(toInteger);
      Result := Parser.TokenInt;
      Parser.NextToken;
      Parser.CheckToken(']');
      Parser.NextToken;
    end;
  end;

  function ReadClass(const aParent: TcwComponent): TcwComponent;
  var
    vClassName: string;
    vObjectName: string;
  begin
    Parser.CheckToken(toSymbol);
    vClassName := Parser.TokenString;
    vObjectName := '';
    if Parser.NextToken = ':' then
    begin
      Parser.NextToken;
      Parser.CheckToken(toSymbol);
      vObjectName := vClassName;
      vClassName := Parser.TokenString;
      Parser.NextToken;
    end;

    if aParent = nil then
    begin
      Result := Components.AddComponent(vObjectName, vClassName);
      FormClass := aUnit.Classes.Find(vClassName);
    end else
      Result := aParent.ChildComponents.AddComponent(vObjectName, vClassName);

    Result.Position := ReadOrderModifier;

    if (vObjectName <> '') and Assigned(FormClass) then
      FormClass.FindSymbol(AUnit, FormClass, vObjectName, True,
        UsedByInfo(FFileIndex, False, 1, Parser.SourceLine));
  end;

  function ReadProperty(const aComponent: TcwComponent): TcwComponentProperty; forward;

  procedure ReadValue(const aComponent: TcwComponent; const aProperty: TcwComponentProperty);
    {var
      Order: Integer;}

    function CombineWideString: WideString;
    begin
      Result := Parser.TokenWideString;
      while Parser.NextToken = '+' do
      begin
        Parser.NextToken;
        if not (Parser.Token in [toString, toWString]) then
          Parser.CheckToken(toString);
        Result := Result + Parser.TokenWideString;
      end;
    end;

  var
    vReferencedClass: TcwSymbol;
    vSymbolName: string;
    vUsedSymbolName: string;
    p: Integer;
    AUsedByInfo: TcwUsedByInfo;
  begin
    if Parser.Token in [toString, toWString] then
    begin
      if aProperty <> nil then
      begin
        aProperty.ValueType := dptString;
        aProperty.Value := CombineWideString;
      end
      else
        CombineWideString;
    end
    else
    begin
      case Parser.Token of
        toSymbol:
          begin
            vSymbolName := Parser.TokenComponentIdent;
            p := Pos('.', vSymbolName);
            AUsedByInfo := UsedByInfo(FFileIndex, False, 1, Parser.SourceLine);
            if p > 0 then
            begin
              vReferencedClass := aUnit.FindReferencedSymbol(FormClass, Copy(vSymbolName, 1, p - 1),
                True, AUsedByInfo);
              vUsedSymbolName := Copy(vSymbolName, p + 1, Length(vSymbolName));
              if (vReferencedClass <> nil) and (vReferencedClass.TypeIntf <> nil) then
                vReferencedClass.TypeIntf.FindSymbol(AUnit, FormClass,
                  vUsedSymbolName, True, AUsedByInfo);
            end else
            begin
              vUsedSymbolName := vSymbolName;
              if FormClass <> nil then
                FormClass.FindSymbol(AUnit, FormClass, vSymbolName, True,
                  AUsedByInfo);
            end;
            if aProperty <> nil then
            begin
              aProperty.ValueType := dptIdentifier;
              aProperty.Value := vSymbolName;
              aComponent.Properties.AddIdentifierName(vSymbolName, aProperty);
            end;
          end;
        toInteger:
          begin
            if aProperty <> nil then
            begin
              aProperty.ValueType := dptInteger;
              aProperty.Value := Parser.TokenInt;
            end;
          end;
        toFloat:
          begin
            if aProperty <> nil then
            begin
              aProperty.ValueType := dptFloat;
              aProperty.Value := Parser.TokenFloat;
            end;
            {case Parser.FloatType of
              's', 'S': Writer.WriteSingle(Parser.TokenFloat);
              'c', 'C': Writer.WriteCurrency(Parser.TokenFloat / 10000);
              'd', 'D': Writer.WriteDate(Parser.TokenFloat);
            else
              Writer.WriteFloat(Parser.TokenFloat);
            end;}
          end;
        '[':
          begin
            Parser.NextToken;
            //Writer.WriteValue(vaSet);
            if Parser.Token <> ']' then
              while True do
              begin
                if Parser.Token <> toInteger then
                  Parser.CheckToken(toSymbol);
                //Writer.WriteStr(Parser.TokenString);
                if Parser.NextToken = ']' then
                  Break;
                Parser.CheckToken(',');
                Parser.NextToken;
              end;
          end;
        '(':
          begin
            Parser.NextToken;
            //Writer.WriteListBegin;
            while Parser.Token <> ')' do
              ReadValue(aComponent, nil);
            //Writer.WriteListEnd;
          end;
        '{': while Parser.Token <> '}' do
            Parser.NextToken;
        '<':
          begin
            Parser.NextToken;
            //Writer.WriteValue(vaCollection);
            while Parser.Token <> '>' do
            begin
              Parser.CheckTokenSymbol('item');
              Parser.NextToken;
              //Order := ReadOrderModifier;
              //if Order <> -1 then Writer.WriteInteger(Order);
              //Writer.WriteListBegin;
              while not Parser.TokenSymbolIs('end') do
                ReadProperty(aComponent);
              //Writer.WriteListEnd;
              Parser.NextToken;
            end;
            //Writer.WriteListEnd;
          end;
        else
          Parser.Error('Invalid property');
      end;
      Parser.NextToken;
    end;
  end;

  function ReadProperty(const aComponent: TcwComponent): TcwComponentProperty;
  var
    PropName: string;
  begin
    Result := nil;
    if Parser.Token = toSymbol then
    begin
      PropName := Parser.TokenString;
      Result := aComponent.Properties.AddProperty(PropName);
      Parser.NextToken;
      while Parser.Token = '.' do
      begin
        Parser.NextToken;
        Parser.CheckToken(toSymbol);
        PropName := PropName + '.' + Parser.TokenString;
        Parser.NextToken;
      end;
      //Writer.WriteStr(PropName);
      if Parser.Token = '=' then
      begin
        Parser.NextToken;
        ReadValue(aComponent, Result);
      end;

    end
    else
      Parser.NextToken;

  end;

  procedure ReadObject(const aParent: TcwComponent);
  var
    vClass: TcwComponent;
    vIsInherited: Boolean;
    vIsInline: Boolean;
  begin
    vIsInherited := False;
    vIsInline := False;
    if Parser.TokenSymbolIs('INHERITED') then
      vIsInherited := True
    else
      if Parser.TokenSymbolIs('INLINE') then
        vIsInline := True
      else
        if Parser.TokenSymbolIs('OBJECT') = FALSE then
        begin
          Parser.NextToken;
          Exit;
        end;
    Parser.NextToken;
    vClass := ReadClass(aParent);
    vClass.IsInherited := vIsInherited;
    vClass.IsInline := vIsInline;

    while not Parser.TokenSymbolIs('END') and
      not Parser.TokenSymbolIs('OBJECT') and
      not Parser.TokenSymbolIs('INHERITED') and
      not Parser.TokenSymbolIs('INLINE') do
      ReadProperty(vClass);
    //Writer.WriteListEnd;
    while (not Parser.TokenSymbolIs('END')) and (not (Parser.Token = #0)) do
      ReadObject(vClass);
    //Writer.WriteListEnd;
    Parser.NextToken;
  end;

begin
  Parser := TParser.Create(aInStream);
  SaveSeparator := DecimalSeparator;
  DecimalSeparator := '.';
  try
    ReadObject(nil);
  finally
    DecimalSeparator := SaveSeparator;
    Parser.Free;
  end;
end;

class function TcwDFMResource.IsBinary(AStream: TStream): Boolean;
var
  SavePosition: Integer;
  AHeader: array[0..2] of Byte;
begin
  SavePosition := AStream.Position;
  try
    AStream.Position := 0;
    if AStream.Read(AHeader, SizeOf(AHeader)) = SizeOf(AHeader) then
      Result := (AHeader[0] = $FF) and (AHeader[1] = $0A) and (AHeader[2] = $00)
    else begin
      Result := False;
      raise Exception.Create('Unable to read from stream.');
    end;
  finally
    AStream.Position := SavePosition;
  end;
end;

procedure TcwDFMResource.ReadResource(const aUnit: TcwUnit; const aFileName: string);
var
  AMemoryStream: TMemoryStream;
  AFileExists: Boolean;
  AFileStream: TFileStream;
begin
  FUnitIndex := aUnit.UnitIndex;
  FFileName := aFileName;
  AFileExists := FileExists(FFileName);
  FFileIndex := gcwProject.FileNames.Add(FFileName, AFileExists, FUnitIndex);
  if AFileExists then
  begin
    AFileStream := TFileStream.Create(aFileName, fmOpenRead or fmShareDenyNone);
    try
      if IsBinary(AFileStream) then
      begin
        AMemoryStream := TMemoryStream.Create;
        try
          ObjectResourceToText(AFileStream, AMemoryStream);
          AMemoryStream.Position := 0;
          ReadDFM(aUnit, AMemoryStream);
        finally
          AMemoryStream.Free;
        end;
      end else
        ReadDFM(aUnit, AFileStream);
    finally
      AFileStream.Free;
    end;
  end;
end;

procedure TcwDFMResource.SetUnitIndex(const aValue: Integer);
begin
  FUnitIndex := aValue;
end;

{- TcwSourceFile Category: No category }
constructor TcwSourceFile.Create(const aFileName: string; const AFileExists: Boolean;
  const AFileIndex: Integer; const aUnitIndex: Integer = -1);
begin
  inherited Create;
  FFileExists := AFileExists;
  FFullFileName := AFileName;
  FFileIndex := AFileIndex;
  FUnitIndex := AUnitIndex;
//  if FUnitIndex >= 0 then
//    Assert(FUnitIndex <= gcwProject.AllUnits.Count - 1);
end;

function TcwSourceFile.GetFileIndex: Integer;
begin
  Result := FFileIndex;
end;

function TcwSourceFile.GetFullFileName: string;
begin
  Result := FFullFileName;
end;

function TcwSourceFile.GetUnitIndex: Integer;
begin
  Result := FUnitIndex;
end;

function TcwSourceFile.GetUnitIntf: TcwUnit;
begin
  Result := gcwProject.AllUnits[UnitIndex];
end;

{- TcwFileNameList Category: No category }
constructor TcwFileNameList.Create;
begin
  inherited Create;
  FList := TObjectList.Create;
  FFileNameList := TcwHashedObjectList.Create;
end;

destructor TcwFileNameList.Destroy;
begin
  FList.Free;
  FFileNameList.Free;
  inherited;
end;

function TcwFileNameList.Add(const aFileName: string; const AFileExists: Boolean;
  AUnitIndex: Integer = -1): Integer;
var
  vFile: TcwSourceFile;
begin
  vFile := TcwSourceFile.Create(aFileName, AFileExists, FList.Count, AUnitIndex);
  Result := FList.Add(vFile);
  FFileNameList.Add(AFileName, vFile);
end;

procedure TcwFileNameList.Clear;
begin
  FList.Clear;
  FFileNameList.Clear;
end;

function TcwFileNameList.Find(AFileName: string): TcwSourceFile;
var
  AIndex: Integer;
begin
  if FFileNameList.Find(AFileName, AIndex) then
    Result := FFileNameList[AIndex] as TcwSourceFile
  else
    Result := nil;
end;

function TcwFileNameList.GetCount: Integer;
begin
  Result := FList.Count;
end;

function TcwFileNameList.GetFileNameList: TcwHashedObjectList;
begin
  Result := FFileNameList;
end;

function TcwFileNameList.GetItems(Index: Integer): TcwSourceFile;
begin
  Result := FList[Index] as TcwSourceFile;
end;

{- TcwUnitList Category: No category }
function TcwUnitList.AddUnit(const AUnit: TcwUnit): Integer;
begin
  Result := DoItemAdded(AUnit.Name, AUnit);
end;

function TcwUnitList.FindUnit(const aUnitName: string): TcwUnit;
begin
  Result := inherited Find(AUnitName) as TcwUnit;
end;

function TcwUnitList.GetSortedUnits(Index: Integer): TcwUnit;
begin
  Result := SortedItem[Index] as TcwUnit;
end;

function TcwUnitList.GetUnits(Index: Integer): TcwUnit;
begin
  Result := FList[Index] as TcwUnit;
end;

initialization

  GlobalKeyHashTable := TmwPasHash.Create;
  GlobalDirectiveHash := TmwDirectiveHash.Create;

finalization

  GlobalKeyHashTable.Free;
  GlobalDirectiveHash.Free;

end.
