(*  Copyright (c) 2002, the CodeLens project
  All rights reserved.

  Redistribution and use in source and binary forms,
  with or without modification, are permitted provided
  that the following conditions are met:

  Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.

  Redistributions in binary form must reproduce the
  above copyright notice, this list of conditions
  and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
  Neither the name of the CodeLens project nor the names
  of its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS
  AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. *)

unit ucwCodeMetricsReport;
interface
uses
  Classes, ucwClasses, ucwReport;

type
  TRPTCodeMetrics = class(TcwReport, IcwReport)
  private
    FUnitOptions : IcwSetting;
    FSourceLinesOptions : IcwSetting;
    FRoutinesOptions    : IcwSetting;
    FVariablesOptions   : IcwSetting;
    FConstantsOptions    : IcwSetting;
    FClassesOptions     : IcwSetting;
    FInterfacesOptions  : IcwSetting;
    FRecordsOptions     : IcwSetting;
    FObjectsOptions     : IcwSetting;
  public
    function GetName : string;override;

    //by default, the reports are sortable
    //the code metrics is a non-sortable report and will return false
    function GetSortable : Boolean;override;

    procedure CreateOptions;override;
    procedure CreateColumns;override;
      //run the report, so it will be filled with Data
    procedure DoExecute;override;
  end;

procedure AddRPTCodeMetrics;


implementation


function TRPTCodeMetrics.GetName : string;
begin
  Result := 'Code Metrics';
end;


//by default, the reports are sortable
//the code metrics is a non-sortable report and will return false
function TRPTCodeMetrics.GetSortable : Boolean;
begin
  Result := False;
end;



procedure TRPTCodeMetrics.CreateOptions;
begin
  FClassesOptions      := Settings.AddSetting('Classes', True, True);
  FVariablesOptions    := Settings.AddSetting('Global Variables', True, True);
  FConstantsOptions     := Settings.AddSetting('Global Constants', True, True);
  FInterfacesOptions   := Settings.AddSetting('Interfaces', True, True);
  FObjectsOptions      := Settings.AddSetting('Legacy Objects', True, True);
  FRecordsOptions      := Settings.AddSetting('Records', True, True);
  FRoutinesOptions     := Settings.AddSetting('Routines', True, True);
  FSourceLinesOptions  := Settings.AddSetting('Source Lines', True, True);
  FUnitOptions         := Settings.AddSetting('Units', True, True);



end;

procedure TRPTCodeMetrics.CreateColumns;
begin
  Columns.AddColumn('Metric',dtString);
  Columns.AddColumn('Value',dtInteger);
end;

//run the report, so it will be filled with Data
procedure TRPTCodeMetrics.DoExecute;

var i : Integer;
var vUnit   : TcwUnit;
var vUnitStats : TcwCodeStatistics;

var LibraryUnits : Integer;
var ProjectUnits : Integer;
var LibraryRoutines : Integer;
var ProjectRoutines : Integer;
var LibraryClasses : Integer;
var ProjectClasses : Integer;
var LibraryObjects : Integer;
var ProjectObjects : Integer;

var LibraryInterfaces : Integer;
var ProjectInterfaces : Integer;

var ProjectRecords    : Integer;
var LibraryRecords    : Integer;

var LibraryVariables : Integer;
var ProjectVariables : Integer;
var LibraryConstants : Integer;
var ProjectConstants : Integer;

var LibraryCodelines    : Integer;
var LibraryEmptylines   : Integer;
var ProjectCodelines    : Integer;
var ProjectEmptylines   : Integer;

var ProjectSlashCommentlines : Integer;
var ProjectBorCommentlines : Integer;
var ProjectAnsiCommentlines : Integer;
var LibrarySlashCommentlines : Integer;
var LibraryBorCommentlines : Integer;
var LibraryAnsiCommentlines : Integer;
var ProjectCommentlines     : Integer;
var LibraryCommentlines     : Integer;
begin

  LibraryUnits := 0;
  ProjectUnits := 0;
  LibraryRoutines := 0;
  ProjectRoutines := 0;
  LibraryClasses := 0;
  ProjectClasses := 0;
  LibraryObjects := 0;
  ProjectObjects := 0;

  LibraryInterfaces := 0;
  ProjectInterfaces := 0;

  ProjectRecords    := 0;
  LibraryRecords    := 0;

  LibraryVariables := 0;
  ProjectVariables := 0;
  LibraryConstants := 0;
  ProjectConstants := 0;

  LibraryCodelines    := 0;
  LibraryEmptylines   := 0;
  ProjectCodelines    := 0;
  ProjectEmptylines   := 0;

  ProjectSlashCommentlines := 0;
  ProjectBorCommentlines := 0;
  ProjectAnsiCommentlines := 0;
  LibrarySlashCommentlines := 0;
  LibraryBorCommentlines := 0;
  LibraryAnsiCommentlines := 0;

  for i := 0 to gcwProject.AllUnits.Count - 1 do
  begin
    vUnit := gcwProject.AllUnits[i];

    Assert(False);
    //TODO:
    //vUnitStats := vUnit.CodeStats;

    if (vUnit.IsLibraryUnit) then
    begin
      inc(LibraryUnits);
      inc(LibraryRoutines,vUnit.Routines.Count);
      inc(LibraryClasses,vUnit.Classes.Count);
      inc(LibraryObjects,vUnit.Objects.Count);
      inc(LibraryInterfaces,vUnit.Interfaces.Count);
      inc(LibraryRecords,vUnit.Records.Count);
      inc(LibraryVariables,vUnit.Variables.Count);
      inc(LibraryConstants,vUnit.Constants.Count);      
      inc(LibraryCodelines,vUnitStats.CodeLines);
      inc(LibrarySlashCommentlines,vUnitStats.SlashCommentLines);
      inc(LibraryAnsiCommentlines,vUnitStats.AnsiCommentLines);
      inc(LibraryBorCommentlines,vUnitStats.BorCommentLines);
      inc(LibraryEmptylines,vUnitStats.EmptyLines);
    end
  else
    begin
      inc(ProjectUnits);
      inc(ProjectRoutines,vUnit.Routines.Count);
      inc(ProjectClasses,vUnit.Classes.Count);
      inc(ProjectObjects,vUnit.Objects.Count);
      inc(ProjectInterfaces,vUnit.Interfaces.Count);
      inc(ProjectRecords,vUnit.Records.Count);
      inc(ProjectVariables,vUnit.Variables.Count);
      inc(ProjectConstants,vUnit.Constants.Count);      
      inc(ProjectCodelines,vUnitStats.CodeLines);
      inc(ProjectSlashCommentlines,vUnitStats.SlashCommentLines);
      inc(ProjectAnsiCommentlines,vUnitStats.AnsiCommentLines);
      inc(ProjectBorCommentlines,vUnitStats.BorCommentLines);
      inc(ProjectEmptylines,vUnitStats.EmptyLines);
    end;
  end;

  // units
  Data.AddTitle(FUnitOptions, 'Units');
  Data.AddRow(FUnitOptions,True, nil, nil, ['Library Units',LibraryUnits]);
  Data.AddRow(FUnitOptions,False, nil, nil, ['Project Units',ProjectUnits]);
  Data.AddRow(FUnitOptions, nil, nil, ['TOTAL UNITS',gcwProject.AllUnits.Count]);
  Data.AddEmptyLine(FUnitOptions);

  // lines of code - project
  Data.AddTitle(FSourceLinesOptions,False, 'Lines of Code - Project');

  Data.AddRow(FSourceLinesOptions,False, nil,nil, ['Project Code Lines',ProjectCodelines]);
  Data.AddRow(FSourceLinesOptions,False, nil, nil, ['Project Empty Lines',ProjectEmptylines]);

  Data.AddRow(FSourceLinesOptions,False, nil, nil,['Project Slash Comment Lines',ProjectSlashCommentlines]);
  Data.AddRow(FSourceLinesOptions,False, nil, nil,['Project Ansi Comment Lines',ProjectAnsiCommentlines]);
  Data.AddRow(FSourceLinesOptions,False, nil, nil,['Project Borland Comment Lines',ProjectBorCommentlines]);
  ProjectCommentlines := ProjectSlashCommentlines + ProjectAnsiCommentlines + ProjectBorCommentlines;
  Data.AddRow(FSourceLinesOptions,False, nil, nil,['Project Total Comment Lines',
              ProjectCommentlines]);

  Data.AddRow(FSourceLinesOptions,False, nil, nil,['TOTAL PROJECT LINES',
             ProjectCodelines +
             ProjectCommentlines +
             ProjectEmptylines]);
  Data.AddEmptyLine(FSourceLinesOptions,False);

  // lines of code - library
  Data.AddTitle(FSourceLinesOptions,True, 'Lines of Code - Library');
  Data.AddRow(FSourceLinesOptions,True,nil, nil,['Library Code Lines',LibraryCodelines]);
  Data.AddRow(FSourceLinesOptions,True,nil, nil,['Library Empty Lines',LibraryEmptylines]);
  Data.AddRow(FSourceLinesOptions,False, nil, nil,['Library Slash Comment Lines',LibrarySlashCommentlines]);
  Data.AddRow(FSourceLinesOptions,False, nil, nil,['Library Ansi Comment Lines',LibraryAnsiCommentlines]);
  Data.AddRow(FSourceLinesOptions,False, nil, nil,['Library Borland Comment Lines',LibraryBorCommentlines]);
  LibraryCommentlines := LibrarySlashCommentlines + LibraryAnsiCommentlines + LibraryBorCommentlines;
  Data.AddRow(FSourceLinesOptions,False, nil, nil,['Library Total Comment Lines',LibraryCommentlines]);

  Data.AddRow(FSourceLinesOptions,True,nil, nil,['TOTAL LIBRARY LINES',
              LibraryCodelines +
              LibraryCommentlines +
              LibraryEmptylines]);
  Data.AddEmptyLine(FSourceLinesOptions,True);

  // lines of code - total
  Data.AddTitle(FSourceLinesOptions, 'Lines of Code - Total');
  Data.AddRow(FSourceLinesOptions, nil, nil,['Code Lines',ProjectCodelines + LibraryCodelines]);
  Data.AddRow(FSourceLinesOptions,nil, nil,['Comment Lines',ProjectCommentlines + LibraryCommentlines]);
  Data.AddRow(FSourceLinesOptions,nil, nil,['Empty Lines',ProjectEmptylines + LibraryEmptylines]);
  Data.AddRow(FSourceLinesOptions,nil, nil,['TOTAL LINES',ProjectCodelines + ProjectCommentlines + ProjectEmptylines
    + LibraryCodelines + LibraryCommentlines + LibraryEmptylines]);
  Data.AddEmptyLine(FSourceLinesOptions);

  // routines
  Data.AddTitle(FRoutinesOptions, 'Routines');
  Data.AddRow(FRoutinesOptions, True, nil, nil,['Library Routines',LibraryRoutines]);
  Data.AddRow(FRoutinesOptions, False, nil, nil,['Project Routines',ProjectRoutines]);
  Data.AddRow(FRoutinesOptions,nil, nil,['TOTAL ROUTINES',ProjectRoutines + LibraryRoutines]);
  Data.AddEmptyLine(FRoutinesOptions);

  // variables
  Data.AddTitle(FVariablesOptions, 'Variables');
  Data.AddRow(FVariablesOptions, True, nil, nil,['Library Variables',LibraryVariables]);
  Data.AddRow(FVariablesOptions, False, nil, nil,['Project Variables',ProjectVariables]);
  Data.AddRow(FVariablesOptions, nil, nil,['TOTAL VARIABLES',ProjectVariables + LibraryVariables]);
  Data.AddEmptyLine(FVariablesOptions);

  // constants
  Data.AddTitle(FConstantsOptions, 'Constants');
  Data.AddRow(FConstantsOptions, True, nil, nil,['Library Constants',LibraryConstants]);
  Data.AddRow(FConstantsOptions, False, nil, nil,['Project Constants',ProjectConstants]);
  Data.AddRow(FConstantsOptions, nil, nil,['TOTAL CONSTANTS',ProjectConstants + LibraryConstants]);
  Data.AddEmptyLine(FConstantsOptions);

  // classes
  Data.AddTitle(FClassesOptions, 'Classes');
  Data.AddRow(FClassesOptions, True, nil, nil,['Library Classes',LibraryClasses]);
  Data.AddRow(FClassesOptions, False, nil, nil,['Project Classes',ProjectClasses]);
  Data.AddRow(FClassesOptions,nil, nil,['TOTAL CLASSES',ProjectClasses + LibraryClasses]);
  Data.AddEmptyLine(FClassesOptions);

  // interfaces
  Data.AddTitle(FInterfacesOptions, 'Interfaces');
  Data.AddRow(FInterfacesOptions, True, nil, nil,['Library Interfaces',LibraryInterfaces]);
  Data.AddRow(FInterfacesOptions, False, nil, nil,['Project Interfaces',ProjectInterfaces]);
  Data.AddRow(FInterfacesOptions,nil, nil,['TOTAL INTERFACES',ProjectInterfaces + LibraryInterfaces]);
  Data.AddEmptyLine(FInterfacesOptions);

  // records
  Data.AddTitle(FRecordsOptions, 'Records');
  Data.AddRow(FRecordsOptions, True, nil, nil,['Library Records',LibraryRecords]);
  Data.AddRow(FRecordsOptions, False, nil, nil,['Project Records',ProjectRecords]);
  Data.AddRow(FRecordsOptions,nil, nil,['TOTAL RECORDS',ProjectRecords + LibraryRecords]);
  Data.AddEmptyLine(FRecordsOptions);

  // objects
  Data.AddTitle(FObjectsOptions, 'Objects');
  Data.AddRow(FObjectsOptions, True, nil, nil,['Library Objects',LibraryObjects]);
  Data.AddRow(FObjectsOptions, False, nil, nil,['Project Objects',ProjectObjects]);
  Data.AddRow(FObjectsOptions, nil, nil,['TOTAL OBJECTS',ProjectObjects + LibraryObjects]);


end;


procedure AddRPTCodeMetrics;
begin
  gcwProject.Reports.AddReport(TRPTCodeMetrics.Create);
end;

end.
