(*  Copyright (c) 2002, the CodeLens project
  All rights reserved.

  Redistribution and use in source and binary forms,
  with or without modification, are permitted provided
  that the following conditions are met:

  Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.

  Redistributions in binary form must reproduce the
  above copyright notice, this list of conditions
  and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
  Neither the name of the CodeLens project nor the names
  of its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS
  AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. *)

unit ucwCodingConventions;

interface
uses
  Classes, ucwClasses;

type
  TcwCodingConventions = class(TInterfacedObject,IcwCodingConventions)
  private
    FFileName : string;
    FComponentPrefixes : IcwPrefixList;
    FFileSettings      : IcwFileSettings;
    FNamingConventions : IcwNamingConventions;
    FIndentationConventions : IcwIndentationConventions;
  public
    constructor Create(const aFileName : string);
    destructor Destroy;override;
    //component prefixes options
    function GetComponentPrefixes : IcwPrefixList;
    property ComponentPrefixes : IcwPrefixList read GetComponentPrefixes;

    function GetNamingConventions : IcwNamingConventions;
    property NamingConventions : IcwNamingConventions read GetNamingConventions;

    //indentation conventions
    function GetIndentationConventions : IcwIndentationConventions;
    property IndentationConventions : IcwIndentationConventions read GetIndentationConventions;

    procedure Save;
    procedure SaveToFile(const aFileSettings : IcwFileSettings);
    procedure Load;
    procedure LoadFromFile(const aFileSettings : IcwFileSettings);

    //load defaults
    procedure LoadDefault;

    procedure LoadDefaultComponentPrefixes;
    procedure LoadDefaultSystemSymbols;
    procedure LoadDefaultNamingConventions;
    procedure LoadDefaultIndentationConventions;
  end;

implementation
uses
  ucwPrefixList, ucwFileSettings, ucwNamingConventions, ucwIndentationConventions;

constructor TcwCodingConventions.Create(const aFileName : string);
begin
  inherited Create;
  FFileName := aFileName;
  FComponentPrefixes := TcwPrefixList.Create;
  FFileSettings      := TcwFileSettings.Create(aFileName);
  FNamingConventions := TcwNamingConventions.Create;
  FIndentationConventions := TcwIndentationConventions.Create; 
end;

destructor  TcwCodingConventions.Destroy;
begin
  inherited Destroy;
end;

//component prefixes options
function    TcwCodingConventions.GetComponentPrefixes : IcwPrefixList;
begin
  Result := FComponentPrefixes;
end;

function TcwCodingConventions.GetNamingConventions : IcwNamingConventions;
begin
  Result := FNamingConventions;
end;

function TcwCodingConventions.GetIndentationConventions : IcwIndentationConventions;
begin
  Result := FIndentationConventions;
end;


procedure TcwCodingConventions.Save;
begin
  SaveToFile(FFileSettings);
end;

procedure TcwCodingConventions.SaveToFile(const aFileSettings : IcwFileSettings);
begin
  FComponentPrefixes.Save(aFileSettings);
  FNamingConventions.Save(aFileSettings);
  FIndentationConventions.Save(aFileSettings);
end;

procedure TcwCodingConventions.Load;
begin
  LoadFromFile(FFileSettings);
end;

procedure TcwCodingConventions.LoadFromFile(const aFileSettings : IcwFileSettings);
begin
  FComponentPrefixes.Load(aFileSettings);
  if FComponentPrefixes.Count = 0 then
   LoadDefaultComponentPrefixes;

  FNamingConventions.Load(aFileSettings);
  FIndentationConventions.Load(aFileSettings);
end;

//load defaults
procedure TcwCodingConventions.LoadDefault;
begin
  LoadDefaultComponentPrefixes;
  LoadDefaultSystemSymbols;
  LoadDefaultNamingConventions;
  LoadDefaultIndentationConventions;
end;

procedure TcwCodingConventions.LoadDefaultComponentPrefixes;
begin
  FComponentPrefixes.LoadDefaultPrefixes;
end;

procedure TcwCodingConventions.LoadDefaultSystemSymbols;
begin
end;

procedure TcwCodingConventions.LoadDefaultNamingConventions;
begin
  FNamingConventions.LoadDefaults;
end;

procedure TcwCodingConventions.LoadDefaultIndentationConventions;
begin
  FIndentationConventions.LoadDefaults;
end;

end.
