(*  Copyright (c) 2002, the CodeLens project
  All rights reserved.

  Redistribution and use in source and binary forms,
  with or without modification, are permitted provided
  that the following conditions are met:

  Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.

  Redistributions in binary form must reproduce the
  above copyright notice, this list of conditions
  and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
  Neither the name of the CodeLens project nor the names
  of its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS
  AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. *)

unit ucwDocument;

interface

uses
  SysUtils,
  ucwPersistIntf;

type
  TDocument = class
  private
    FFileName: TFileName;
    FContents: IclPersist;
    FIsDirty: boolean;

    function IsWritable: boolean;

  protected
    procedure SetFileName(Value: TFileName);
    function GetCanSave: boolean;

  public
    constructor Create(aPersist: IclPersist);
    destructor Destroy; override;

    property FileName: TFileName
      read FFileName
      write SetFileName;
    property IsDirty: boolean
      read FIsDirty
      write FIsDirty;
    property CanSave: boolean
      read GetCanSave;

    procedure New;
    function  Load : Boolean;
    procedure Download(const aURL: string);
    procedure Save;
  end;

implementation

uses
  Classes,
  ucwNet;

// ------------------------------------------------------------------------------------------

constructor TDocument.Create(aPersist: IclPersist);
begin
  inherited Create;

  FContents := aPersist;
  New;
end;

destructor TDocument.Destroy;
begin
  inherited;
end;

function TDocument.IsWritable: boolean;
begin
  // init result
  Result := TRUE;

  // new file is always writable
  if (FFileName = '') then
    Exit;

  // get read-only attributes
  Result := not FileIsReadOnly(FFileName);
end;

procedure TDocument.SetFileName(Value: TFileName);
begin
  if (Value <> FFileName) then
    FFileName := Value;
end;

function TDocument.GetCanSave: boolean;
begin
  Result := IsWritable;
end;

procedure TDocument.New;
begin
  FContents.Clear;

  FFileName := '';
  FIsDirty := FALSE;
end;

function  TDocument.Load : Boolean;
begin
  Result := False;
  // check input
  if (FFileName = '') then
    Exit;

  Result := FContents.Load(FFileName);
  FIsDirty := FALSE;
end;

procedure TDocument.Download(const aURL: string);
var
  s: string;
begin
  // check input
  if (aURL = '') then
    Exit;

  s := GetRequest(aURL);
  //FContents.AsXML := s;
  FIsDirty := FALSE;
end;

procedure TDocument.Save;
begin
  // check input
  if (FFileName = '') then
    Exit;

  FContents.Save(FFileName);
  FIsDirty := FALSE;
end;

// ------------------------------------------------------------------------------------------

end.
