(*  Copyright (c) 2002, the CodeLens project
  All rights reserved.

  Redistribution and use in source and binary forms,
  with or without modification, are permitted provided
  that the following conditions are met:

  Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.

  Redistributions in binary form must reproduce the
  above copyright notice, this list of conditions
  and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
  Neither the name of the CodeLens project nor the names
  of its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS
  AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. *)

unit ucwFileSettings;

interface

uses
  Classes,
  ucwClasses,
  ucwIniOptions;

type
  TcwFileSettings = class(TInterfacedObject, IcwFileSettings)
  private
    FIniOptions: TOptions;
  protected
    function GetFileName: string;
  public
    constructor Create(const aFileName: string);
    destructor Destroy; override;
    function GetOption(const aSection: string; const aOption: string;
      const aDefaultValue: Variant): Variant;
    procedure SetOption(const aSection: string; const aOption: string; const aNewValue: Variant);
    procedure ReadSection(const aSection: string; const aOptionStrings: IcwStringList);
    //write a aSection with value pairs : each entry has a name and a value fater the '=' cgaracter
    procedure ReadSectionValues(const aSection: string; const aOptionStrings: IcwStringList);
    procedure SaveSection(const aSection: string; const aOptionStrings: IcwStringList);
    function Load(const aLocation: widestring): Boolean;
    procedure Save(const aLocation: widestring);
    property FileName: string read GetFileName;
  end;

implementation

uses SysUtils;

constructor TcwFileSettings.Create(const aFileName: string);
begin
  inherited Create;
  FIniOptions := TOptions.Create(aFileName);
end;

destructor TcwFileSettings.Destroy;
begin
  FreeAndNil(FIniOptions);
  inherited;
end;

function TcwFileSettings.GetFileName: string;
begin
  Result := FIniOptions.IniFileName;
end;

function TcwFileSettings.GetOption(const aSection: string; const aOption: string; const aDefaultValue:
  Variant): Variant;
begin
  Result := FIniOptions.GetOption(aSection, aOption, aDefaultValue);
end;

procedure TcwFileSettings.SetOption(const aSection: string; const aOption: string; const aNewValue: Variant);
begin
  FIniOptions.SetOption(aSection, aOption, aNewValue);
end;

procedure TcwFileSettings.ReadSection(const aSection: string; const aOptionStrings: IcwStringList);
var
  sList: TStringList;
begin
  sList := TStringList.Create;
  try
    FIniOptions.ReadSection(aSection, sList);
    aOptionStrings.Text := sList.Text;
  finally
    sList.Free;
  end;
end;

//write a aSection with value pairs : each entry has a name and a value fater the '=' cgaracter
procedure TcwFileSettings.ReadSectionValues(const aSection: string; const aOptionStrings: IcwStringList);
var
  sList: TStringList;
begin
  sList := TStringList.Create;
  try
    FIniOptions.ReadSectionValues(aSection, sList);
    aOptionStrings.Text := sList.Text;
  finally
    sList.Free;
  end;
end;

procedure TcwFileSettings.SaveSection(const aSection: string; const aOptionStrings: IcwStringList);
var
  sList: TStringList;
begin
  sList := TStringList.Create;
  try
    sList.Text := aOptionStrings.Text;
    FIniOptions.SaveTStringOptions(aSection, sList);
  finally
    sList.Free;
  end;
end;

function TcwFileSettings.Load(const aLocation: widestring): Boolean;
begin
  Result := True;
end;

procedure TcwFileSettings.Save(const aLocation: widestring);
begin

end;

end.
