(*  Copyright (c) 2002, the CodeLens project
  All rights reserved.

  Redistribution and use in source and binary forms,
  with or without modification, are permitted provided
  that the following conditions are met:

  Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.

  Redistributions in binary form must reproduce the
  above copyright notice, this list of conditions
  and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
  Neither the name of the CodeLens project nor the names
  of its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS
  AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. *)

unit ucwIndentationConventions;
interface
uses
  ucwClasses, Classes;

type
  TcwIndentationConventions = class(TInterfacedObject, IcwIndentationConventions)
  private
    FSpacesPerIndent : Integer;
    //Extra Indentations Before
    FIndentBeforeBegin : Boolean;
    FIndentBeforeTry : Boolean;
    FIndentBeforeElseInTryBlock : Boolean;
    FIndentBeforeElseInCaseBlock : Boolean;
    //Indentation exceptions
    FNeverIndentElseIf : Boolean;
    FIndentComments    : Boolean;
    FIndentDirectives  : Boolean;
  public
    constructor Create;
    function  GetSpacesPerIndent : Integer;
    procedure SetSpacesPerIndent(const aValue : Integer);
    property  SpacesPerIndent : Integer read GetSpacesPerIndent write SetSpacesPerIndent;
    //Extra Indentations Before
    function  GetIndentBeforeBegin : Boolean;
    procedure SetIndentBeforeBegin(const aValue : Boolean);
    property  IndentBeforeBegin : Boolean read GetIndentBeforeBegin write SetIndentBeforeBegin;

    function  GetIndentBeforeTry : Boolean;
    procedure SetIndentBeforeTry(const aValue : Boolean);
    property  IndentBeforeTry : Boolean read GetIndentBeforeTry write SetIndentBeforeTry;

    function  GetIndentBeforeElseInTryBlock : Boolean;
    procedure SetIndentBeforeElseInTryBlock(const aValue : Boolean);
    property  IndentBeforeElseInTryBlock : Boolean read GetIndentBeforeElseInTryBlock write SetIndentBeforeElseInTryBlock;

    function  GetIndentBeforeElseInCaseBlock : Boolean;
    procedure SetIndentBeforeElseInCaseBlock(const aValue : Boolean);
    property  IndentBeforeElseInCaseBlock : Boolean read GetIndentBeforeElseInCaseBlock write SetIndentBeforeElseInCaseBlock;

    //Indentation exceptions
    function  GetNeverIndentElseIf : Boolean;
    procedure SetNeverIndentElseIf(const aValue : Boolean);
    property  NeverIndentElseIf : Boolean read GetNeverIndentElseIf write SetNeverIndentElseIf;

    function  GetIndentComments : Boolean;
    procedure SetIndentComments(const aValue : Boolean);
    property  IndentComments : Boolean read GetIndentComments write SetIndentComments;

    function  GetIndentDirectives : Boolean;
    procedure SetIndentDirectives(const aValue : Boolean);
    property  IndentDirectives : Boolean read GetIndentDirectives write SetIndentDirectives;

    procedure Save(const aFile : IcwFileSettings);
    procedure Load(const aFile : IcwFileSettings);
    procedure LoadDefaults;

  end;

implementation


constructor TcwIndentationConventions.Create;
begin
  inherited Create;
  LoadDefaults;
end;


function  TcwIndentationConventions.GetSpacesPerIndent : Integer;
begin
  Result := FSpacesPerIndent;
end;

procedure TcwIndentationConventions.SetSpacesPerIndent(const aValue : Integer);
begin
  FSpacesPerIndent := aValue;
end;

//Extra Indentations Before
function  TcwIndentationConventions.GetIndentBeforeBegin : Boolean;
begin
  Result := FIndentBeforeBegin;
end;

procedure TcwIndentationConventions.SetIndentBeforeBegin(const aValue : Boolean);
begin
  FIndentBeforeBegin := aValue;
end;

function  TcwIndentationConventions.GetIndentBeforeTry : Boolean;
begin
  Result := FIndentBeforeTry;
end;

procedure TcwIndentationConventions.SetIndentBeforeTry(const aValue : Boolean);
begin
  FIndentBeforeTry := aValue;
end;

function  TcwIndentationConventions.GetIndentBeforeElseInTryBlock : Boolean;
begin
  Result := FIndentBeforeElseInTryBlock;
end;

procedure TcwIndentationConventions.SetIndentBeforeElseInTryBlock(const aValue : Boolean);
begin
  FIndentBeforeElseInTryBlock := aValue;
end;

function  TcwIndentationConventions.GetIndentBeforeElseInCaseBlock : Boolean;
begin
  Result := FIndentBeforeElseInCaseBlock;
end;

procedure TcwIndentationConventions.SetIndentBeforeElseInCaseBlock(const aValue : Boolean);
begin
  FIndentBeforeElseInCaseBlock := aValue;
end;

//Indentation exceptions
function  TcwIndentationConventions.GetNeverIndentElseIf : Boolean;
begin
  Result := FNeverIndentElseIf;
end;

procedure TcwIndentationConventions.SetNeverIndentElseIf(const aValue : Boolean);
begin
  FNeverIndentElseIf := aValue;
end;

function  TcwIndentationConventions.GetIndentComments : Boolean;
begin
  Result := FIndentComments;
end;

procedure TcwIndentationConventions.SetIndentComments(const aValue : Boolean);
begin
  FIndentComments := aValue;
end;

function  TcwIndentationConventions.GetIndentDirectives : Boolean;
begin
  Result := FIndentDirectives;
end;

procedure TcwIndentationConventions.SetIndentDirectives(const aValue : Boolean);
begin
  FIndentDirectives := aValue;
end;

procedure TcwIndentationConventions.Save(const aFile : IcwFileSettings);
begin
  aFile.SetOption('IndentationConventions','SpacesPerIndent',FSpacesPerIndent);
  //Extra Indentations Before
  aFile.SetOption('IndentationConventions','IndentBeforeBegin',FIndentBeforeBegin);
  aFile.SetOption('IndentationConventions','IndentBeforeTry',FIndentBeforeTry);
  aFile.SetOption('IndentationConventions','IndentBeforeElseInTryBlock',FIndentBeforeElseInTryBlock);
  aFile.SetOption('IndentationConventions','IndentBeforeElseInCaseBlock',FIndentBeforeElseInCaseBlock);
  //Indentation exceptions
  aFile.SetOption('IndentationConventions','NeverIndentElseIf',FNeverIndentElseIf);
  aFile.SetOption('IndentationConventions','IndentComments',FIndentComments);
  aFile.SetOption('IndentationConventions','IndentDirectives',FIndentDirectives);
end;


procedure TcwIndentationConventions.LoadDefaults;
begin
  FSpacesPerIndent := 2;
  //Extra Indentations Before
  FIndentBeforeBegin := False;
  FIndentBeforeTry   := False;
  FIndentBeforeElseInTryBlock := False;
  FIndentBeforeElseInCaseBlock := False;
  //Indentation exceptions
  FNeverIndentElseIf := False;
  FIndentComments    := True;
  FIndentDirectives  := False;
end;

procedure TcwIndentationConventions.Load(const aFile : IcwFileSettings);
begin
  FSpacesPerIndent := aFile.GetOption('IndentationConventions','SpacesPerIndent',FSpacesPerIndent);
  //Extra Indentations Before
  FIndentBeforeBegin := aFile.GetOption('IndentationConventions','IndentBeforeBegin',FIndentBeforeBegin);
  FIndentBeforeTry   := aFile.GetOption('IndentationConventions','IndentBeforeTry',FIndentBeforeTry);
  FIndentBeforeElseInTryBlock := aFile.GetOption('IndentationConventions','IndentBeforeElseInTryBlock',FIndentBeforeElseInTryBlock);
  FIndentBeforeElseInCaseBlock := aFile.GetOption('IndentationConventions','IndentBeforeElseInCaseBlock',FIndentBeforeElseInCaseBlock);
  //Indentation exceptions
  FNeverIndentElseIf := aFile.GetOption('IndentationConventions','NeverIndentElseIf',FNeverIndentElseIf);
  FIndentComments    := aFile.GetOption('IndentationConventions','IndentComments',FIndentComments);
  FIndentDirectives  := aFile.GetOption('IndentationConventions','IndentDirectives',FIndentDirectives);

end;
end.
