(*  Copyright (c) 2002, the CodeLens project
  All rights reserved.

  Redistribution and use in source and binary forms,
  with or without modification, are permitted provided
  that the following conditions are met:

  Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.

  Redistributions in binary form must reproduce the
  above copyright notice, this list of conditions
  and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
  Neither the name of the CodeLens project nor the names
  of its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS
  AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. *)

unit ucwMessageList;

interface

uses
  Classes,
  ucwQuickSort,
  ucwClasses;

type
  TcwMessageList = class(TInterfacedObject, IcwMessageList, IcwQuickSortable)
  private
    FSortBy: TMessageListSortBy;
    FList: TInterfaceList;
  protected
    function GetMessage(Index: Integer): IcwMessage;
    function GetCount: Integer;
    // IcwQuickSortable
    function Compare(Index1: Integer; Index2: Integer): Integer;
    procedure Exchange(Index1: Integer; Index2: Integer);
  public
    constructor Create;
    destructor Destroy; override;
    property Messages[Index: Integer]: IcwMessage read GetMessage; default;
    function AddMessage(const aMsg: string; const AFileIndex: Integer; const aX, aY: Integer;
      const AInterfaceSection: Boolean): IcwMessage;
    property Count: Integer read GetCount;
    procedure Clear;
    procedure Sort(ASortBy: TMessageListSortBy);
  end;

implementation

uses
  SysUtils,
  ucwMessage;

constructor TcwMessageList.Create;
begin
  inherited;
  FList := TInterfaceList.Create;
end;

destructor TcwMessageList.Destroy;
begin
  FList.Free;
  inherited;
end;

function TcwMessageList.GetMessage(Index: Integer): IcwMessage;
begin
  Result := IcwMessage(FList[Index]);
end;

function TcwMessageList.AddMessage(const aMsg: string; const AFileIndex: Integer;
  const aX, aY: Integer; const AInterfaceSection: Boolean): IcwMessage;
begin
  Result := TcwMessage.Create(aMsg, AFileIndex, aX, aY, AInterfaceSection);
  FList.Add(Result);
end;

function TcwMessageList.GetCount: Integer;
begin
  Result := FList.Count;
end;

procedure TcwMessageList.Clear;
begin
  FList.Clear;
end;

procedure TcwMessageList.Sort(ASortBy: TMessageListSortBy);
begin
  FSortBy := ASortBy;
  if Count > 0 then
    QuickSort(Self as IcwQuickSortable, 0, Count - 1);
end;

function TcwMessageList.Compare(Index1, Index2: Integer): Integer;
begin
  Result := 0;
  case FSortBy of
    mlsbMessage: Result := CompareStr(Messages[Index1].MessageText, Messages[Index2].MessageText);
    mlsbUnit: Result := CompareStr(Messages[Index1].CodeLocation.FileName,
      Messages[Index2].CodeLocation.FileName);
    mlsbInterface: Result := Ord(Messages[Index1].InterfaceSection) -
      Ord(Messages[Index2].InterfaceSection);
    else
      Assert(False);
  end;
end;

procedure TcwMessageList.Exchange(Index1, Index2: Integer);
begin
  FList.Exchange(Index1, Index2);
end;

end.
