(*  Copyright (c) 2002, the CodeLens project
  All rights reserved.

  Redistribution and use in source and binary forms,
  with or without modification, are permitted provided
  that the following conditions are met:

  Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.

  Redistributions in binary form must reproduce the
  above copyright notice, this list of conditions
  and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
  Neither the name of the CodeLens project nor the names
  of its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS
  AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. *)

unit ucwNamingConventions;

interface
uses
  Classes, ucwClasses;

type
  TcwNamingConventions = class(TInterfacedObject, IcwNamingConventions)
  private
    FLocalVariables : string;
    FGlobalVariables : string;
    FConstants : string;
    FClassFields : string;
    FClassTypes : string;
    FInterfaceTypes : string;
    FRecordTypes : string;
    FObjectTypes : string;
    FPointerTypes : string;
    FRoutineNames : string;
    FConstParameters : string;
    FVarParameters : string;
    FParameters : string;
    FProperties : string;
    FGetRoutines : string;
    FSetRoutines : string;
  public
    constructor Create;
    //variables
    function  GetLocalVariables : string;
    procedure SetLocalVariables(const aValue : string);
    property  LocalVariables : string read GetLocalVariables write SetLocalVariables;

    function  GetGlobalVariables : string;
    procedure SetGlobalVariables(const aValue : string);
    property  GlobalVariables : string read GetGlobalVariables write SetGlobalVariables;

    function  GetConstants : string;
    procedure SetConstants(const aValue : string);
    property  Constants : string read GetConstants write SetConstants;

    function  GetClassFields : string;
    procedure SetClassFields(const aValue : string);
    property  ClassFields : string read GetClassFields write SetClassFields;

    //types
    function  GetClassTypes : string;
    procedure SetClassTypes(const aValue : string);
    property  ClassTypes : string read GetClassTypes write SetClassTypes;

    function  GetInterfaceTypes : string;
    procedure SetInterfaceTypes(const aValue : string);
    property  InterfaceTypes : string read GetInterfaceTypes write SetInterfaceTypes;

    function  GetRecordTypes : string;
    procedure SetRecordTypes(const aValue : string);
    property  RecordTypes : string read GetRecordTypes write SetRecordTypes;

    function  GetObjectTypes : string;
    procedure SetObjectTypes(const aValue : string);
    property  ObjectTypes : string read GetObjectTypes write SetObjectTypes;

    function  GetPointerTypes : string;
    procedure SetPointerTypes(const aValue : string);
    property  PointerTypes : string read GetPointerTypes write SetPointerTypes;

    //routines

    function  GetRoutineNames : string;
    procedure SetRoutineNames(const aValue : string);
    property  RoutineName : string read GetRoutineNames write SetRoutineNames;

    function  GetConstParameters : string;
    procedure SetConstParameters(const aValue : string);
    property  ConstParameters : string read GetConstParameters write SetConstParameters;

    function  GetVarParameters : string;
    procedure SetVarParameters(const aValue : string);
    property  VarParameters : string read GetVarParameters write SetVarParameters;

    function  GetParameters : string;
    procedure SetParameters(const aValue : string);
    property  Parameters : string read GetParameters write SetParameters;

    //properties
    function  GetProperties : string;
    procedure SetProperties(const aValue : string);
    property  Properties : string read GetProperties write SetProperties;

    function  GetGetRoutines : string;
    procedure SetGetRoutines(const aValue : string);
    property  GetRoutines : string read GetGetRoutines write SetGetRoutines;

    function  GetSetRoutines : string;
    procedure SetSetRoutines(const aValue : string);
    property  SetRoutines : string read GetSetRoutines write SetSetRoutines;

    procedure Save(const aFile : IcwFileSettings);
    procedure Load(const aFile : IcwFileSettings);

    procedure LoadDefaults;
  end;

implementation


constructor TcwNamingConventions.Create;
begin
  inherited Create;
  LoadDefaults;
end;

function  TcwNamingConventions.GetLocalVariables : string;
begin
  Result := FLocalVariables;
end;

procedure TcwNamingConventions.SetLocalVariables(const aValue : string);
begin
  FLocalVariables := aValue;
end;

function  TcwNamingConventions.GetGlobalVariables : string;
begin
  Result := FGlobalVariables;
end;

procedure TcwNamingConventions.SetGlobalVariables(const aValue : string);
begin
  FGlobalVariables := aValue;
end;

function  TcwNamingConventions.GetConstants : string;
begin
  Result := FConstants;
end;

procedure TcwNamingConventions.SetConstants(const aValue : string);
begin
  FConstants := aValue;
end;

function  TcwNamingConventions.GetClassFields : string;
begin
  Result := FClassFields;
end;

procedure TcwNamingConventions.SetClassFields(const aValue : string);
begin
  FClassFields := aValue;
end;

//types
function  TcwNamingConventions.GetClassTypes : string;
begin
  Result := FClassTypes;
end;

procedure TcwNamingConventions.SetClassTypes(const aValue : string);
begin
  FClassTypes := aValue;
end;

function  TcwNamingConventions.GetInterfaceTypes : string;
begin
  Result := FInterfaceTypes;
end;

procedure TcwNamingConventions.SetInterfaceTypes(const aValue : string);
begin
  FInterfaceTypes := aValue;
end;

function  TcwNamingConventions.GetRecordTypes : string;
begin
  Result := FRecordTypes;
end;

procedure TcwNamingConventions.SetRecordTypes(const aValue : string);
begin
  FRecordTypes := aValue;
end;

function  TcwNamingConventions.GetObjectTypes : string;
begin
  Result := FObjectTypes;
end;

procedure TcwNamingConventions.SetObjectTypes(const aValue : string);
begin
  FObjectTypes := aValue;
end;

function  TcwNamingConventions.GetPointerTypes : string;
begin
  Result := FPointerTypes;
end;

procedure TcwNamingConventions.SetPointerTypes(const aValue : string);
begin
  FPointerTypes := aValue;
end;

//routines

function  TcwNamingConventions.GetRoutineNames : string;
begin
  Result := FRoutineNames;
end;

procedure TcwNamingConventions.SetRoutineNames(const aValue : string);
begin
  FRoutineNames := aValue;
end;

function  TcwNamingConventions.GetConstParameters : string;
begin
  Result := FConstParameters;
end;

procedure TcwNamingConventions.SetConstParameters(const aValue : string);
begin
  FConstParameters := aValue;
end;

function  TcwNamingConventions.GetVarParameters : string;
begin
  Result := FVarParameters;
end;

procedure TcwNamingConventions.SetVarParameters(const aValue : string);
begin
  FVarParameters := aValue;
end;

function  TcwNamingConventions.GetParameters : string;
begin
  Result := FParameters;
end;

procedure TcwNamingConventions.SetParameters(const aValue : string);
begin
  FParameters := aValue;
end;

//properties
function  TcwNamingConventions.GetProperties : string;
begin
  Result := FProperties;
end;

procedure TcwNamingConventions.SetProperties(const aValue : string);
begin
  FProperties := aValue;
end;

function  TcwNamingConventions.GetGetRoutines : string;
begin
  Result := FGetRoutines;
end;

procedure TcwNamingConventions.SetGetRoutines(const aValue : string);
begin
  FGetRoutines := aValue;
end;

function  TcwNamingConventions.GetSetRoutines : string;
begin
  Result := FSetRoutines;
end;

procedure TcwNamingConventions.SetSetRoutines(const aValue : string);
begin
  FSetRoutines := aValue;
end;

procedure TcwNamingConventions.Save(const aFile : IcwFileSettings);
begin
  aFile.SetOption('NamingConventions','LocalVariables',FLocalVariables);
  aFile.SetOption('NamingConventions','GlobalVariables',FGlobalVariables);
  aFile.SetOption('NamingConventions','Constants',FConstants);
  aFile.SetOption('NamingConventions','ClassFields',FClassFields);
  aFile.SetOption('NamingConventions','ClassTypes',FClassTypes);
  aFile.SetOption('NamingConventions','InterfaceTypes',FInterfaceTypes);
  aFile.SetOption('NamingConventions','RecordTypes',FRecordTypes);
  aFile.SetOption('NamingConventions','ObjectTypes',FObjectTypes);
  aFile.SetOption('NamingConventions','PointerTypes',FPointerTypes);
  aFile.SetOption('NamingConventions','RoutineNames',FRoutineNames);
  aFile.SetOption('NamingConventions','ConstParameters',FConstParameters);
  aFile.SetOption('NamingConventions','VarParameters',FVarParameters);
  aFile.SetOption('NamingConventions','Parameters',FParameters);
  aFile.SetOption('NamingConventions','Properties',FProperties);
  aFile.SetOption('NamingConventions','GetRoutines',FGetRoutines);
  aFile.SetOption('NamingConventions','SetRoutines',FSetRoutines);
end;


procedure TcwNamingConventions.LoadDefaults;
begin
  FLocalVariables  := 'v[A-Z].*';
  FGlobalVariables := 'g[A-Z].*';
  FConstants       := '[A-Z]+';
  FClassFields     := 'F[A-Z].*';
  FClassTypes      := 'T[A-Z].*';
  FInterfaceTypes  := 'I[A-Z].*';
  FRecordTypes     := 'T[A-Z].*';
  FObjectTypes     := 'T[A-Z].*';
  FPointerTypes    := 'P[A-Z].*';
  FRoutineNames    := '[A-Z].*';
  FConstParameters := 'a[A-Z].*';
  FVarParameters   := 'a[A-Z].*';
  FParameters      := '';
  FProperties      := '[A-Z].*';
  FGetRoutines     := 'Get[A-Z].*';
  FSetRoutines     := 'Set[A-Z].*';

end;

procedure TcwNamingConventions.Load(const aFile : IcwFileSettings);
begin
  FLocalVariables  := aFile.GetOption('NamingConventions','LocalVariables',FLocalVariables);
  FGlobalVariables := aFile.GetOption('NamingConventions','GlobalVariables',FGlobalVariables);
  FConstants       := aFile.GetOption('NamingConventions','Constants',FConstants);
  FClassFields     := aFile.GetOption('NamingConventions','ClassFields',FClassFields);
  FClassTypes      := aFile.GetOption('NamingConventions','ClassTypes',FClassTypes);
  FInterfaceTypes  := aFile.GetOption('NamingConventions','InterfaceTypes',FInterfaceTypes);
  FRecordTypes     := aFile.GetOption('NamingConventions','RecordTypes',FRecordTypes);
  FObjectTypes     := aFile.GetOption('NamingConventions','ObjectTypes',FObjectTypes);
  FPointerTypes    := aFile.GetOption('NamingConventions','PointerTypes',FPointerTypes);
  FRoutineNames    := aFile.GetOption('NamingConventions','RoutineNames',FRoutineNames);
  FConstParameters := aFile.GetOption('NamingConventions','ConstParameters',FConstParameters);
  FVarParameters   := aFile.GetOption('NamingConventions','VarParameters',FVarParameters);
  FParameters      := aFile.GetOption('NamingConventions','Parameters',FParameters);
  FProperties      := aFile.GetOption('NamingConventions','Properties',FProperties);
  FGetRoutines     := aFile.GetOption('NamingConventions','GetRoutines',FGetRoutines);
  FSetRoutines     := aFile.GetOption('NamingConventions','SetRoutines',FSetRoutines);

end;

end.
