(*  Copyright (c) 2002, the CodeLens project
  All rights reserved.

  Redistribution and use in source and binary forms,
  with or without modification, are permitted provided
  that the following conditions are met:

  Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.

  Redistributions in binary form must reproduce the
  above copyright notice, this list of conditions
  and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
  Neither the name of the CodeLens project nor the names
  of its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS
  AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. *)

unit ucwOptions;

interface

uses
  Classes,
  ucwClasses;

type
  TcwOptions = class(TInterfacedObject, IcwOptions)
  private
    FFileToAnalyze: string;
    FProjectSearchPath: IcwStringList;
    FDefines: IcwStringList;
    FAliases: IcwStringList;
  protected
    function GetAliases: IcwStringList;
    function GetFileToAnalyze: widestring;
    procedure SetFileToAnalyze(const aValue: widestring);
    function GetProjectSearchPath: IcwStringList;
    function GetDefines: IcwStringList;
  public
    constructor Create;
    procedure Save(const aFileSettings: IcwFileSettings);
    procedure Load(const aFileSettings: IcwFileSettings);
    procedure Clear;
    property ProjectSearchPath: IcwStringList read GetProjectSearchPath;
    property Defines: IcwStringList read GetDefines;
    property Aliases: IcwStringList read GetAliases;
  end;

implementation

uses
  ucwStringList;

constructor TcwOptions.Create;
begin
  inherited;
  FProjectSearchPath := TcwStringList.Create;
  FDefines := TcwStringList.Create;
  FAliases := TcwStringList.Create;
end;

function TcwOptions.GetFileToAnalyze: widestring;
begin
  Result := FFileToAnalyze;
end;

procedure TcwOptions.SetFileToAnalyze(const aValue: widestring);
begin
  FFileToAnalyze := aValue;
end;

function TcwOptions.GetProjectSearchPath: IcwStringList;
begin
  Result := FProjectSearchPath;
end;

function TcwOptions.GetDefines: IcwStringList;
begin
  Result := FDefines;
  if FDefines.IndexOf('CodeWrench') = -1 then
    FDefines.Add('CodeWrench');
end;

procedure TcwOptions.Save(const aFileSettings: IcwFileSettings);
begin
  aFileSettings.SetOption('general', 'filetoanalyze', FFileToAnalyze);
  aFileSettings.SaveSection('projectsearchpath', FProjectSearchPath);
  aFileSettings.SaveSection('defines', FDefines);
  aFileSettings.SaveSection('aliases', FAliases);
end;

procedure TcwOptions.Load(const aFileSettings: IcwFileSettings);
begin
  FFileToAnalyze := aFileSettings.GetOption('general', 'filetoanalyze', 'Default Value');
  aFileSettings.ReadSection('projectsearchpath', FProjectSearchPath);
  aFileSettings.ReadSection('defines', FDefines);
  aFileSettings.ReadSectionValues('aliases', FAliases);
end;

procedure TcwOptions.Clear;
begin
  FFileToAnalyze := '';
  FProjectSearchPath.Clear;
  FDefines.Clear;
  FAliases.Clear;
end;

function TcwOptions.GetAliases: IcwStringList;
begin
  Result := FAliases;
end;

end.

