(*  Copyright (c) 2002, the CodeLens project
  All rights reserved.

  Redistribution and use in source and binary forms,
  with or without modification, are permitted provided
  that the following conditions are met:

  Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.

  Redistributions in binary form must reproduce the
  above copyright notice, this list of conditions
  and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
  Neither the name of the CodeLens project nor the names
  of its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS
  AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. *)

unit ucwPasLintReport;
interface
uses
  Windows, SysUtils, Classes, ucwClasses, ucwReport;

type
  TRPTpasLint = class(TcwReport, IcwReport)
  private
    FUnusedVarsOptions: IcwSetting;
    FUnusedConstsOptions: IcwSetting;
    FUnusedRoutinesOptions: IcwSetting;
    FUnusedClassFieldsOption: IcwSetting;
    FUnusedClassPropertiesOption: IcwSetting;
    FUnusedClassRoutinesOption: IcwSetting;
    FUnusedParametersOption: IcwSetting;
    FUnusedParametersAsConstOption: IcwSetting;
    FNonOverrideDestructor: IcwSetting;
    FVirtualDestructorOption: IcwSetting;
    FVirtualRoutineWithNoDescendants: IcwSetting;
    FMoreThanOneDestructor: IcwSetting;
    FSameRoutineImplemetsTwoInterfaces: IcwSetting;
    FAncestorInterfaceNotImplemented: IcwSetting;
    FPublicFields: IcwSetting;
    FUnusedRecordFields: IcwSetting;
    FGUIDNotDefined: IcwSetting;
    FDuplicatedGUID: IcwSetting;
    FRoutineNameToReturnRoutineResult: IcwSetting;
    FCheckForWrongCase: IcwSetting;
    FComponentPrefixes: IcwSetting;
    FIndentationConventions: IcwSetting;
    FConventionLocalVariables: IcwSetting;
    FConventionGlobalVariables: IcwSetting;
    FConventionConstants: IcwSetting;
    FConventionClassFields: IcwSetting;
    FConventionClassTypes: IcwSetting;
    FConventionInterfaceTypes: IcwSetting;
    FConventionRecordTypes: IcwSetting;
    FConventionObjectTypes: IcwSetting;
    FConventionPointerTypes: IcwSetting;
    FConventionRoutineName: IcwSetting;
    FConventionConstParameters: IcwSetting;
    FConventionVarParameters: IcwSetting;
    FConventionParameters: IcwSetting;
    FConventionProperties: IcwSetting;
    FConventionGetRoutines: IcwSetting;
    FConventionSetRoutines: IcwSetting;

    function HasDescendingRoutine(const aClass: TcwClass; const aRoutine: tcwRoutine): Boolean;
    procedure CheckUnit(const aUnit: TcwUnit);
    procedure CheckClasses(const aUnit: TcwUnit; const aClassList: TcwClassList);
    procedure CheckRecords(const aUnit: TcwUnit);
    procedure CheckRoutine(const aUnit: TcwUnit; const aClass: TcwClass; const aRoutine: TcwRoutine);
    procedure CheckForComponentPrefixes(const aUnit: TcwUnit);
    procedure CheckForIndentationConventions(const aUnit: TcwUnit);
    procedure CheckRoutineConventions(const aUnit: TcwUnit; const aClass: TcwClass; const aRoutine:
      TcwRoutine);
    procedure CheckForNonConstParameters(const aUnit: TcwUnit; const aClass: TcwClass; const aRoutine:
      TcwRoutine);
    procedure CheckForUnusedVars(const aUnit: TcwUnit; const aClass: TcwClass; const aContainer:
      TcwVarContainer);
    procedure CheckForReturnParameters(const aUnit: TcwUnit; const aRoutine: TcwRoutine);
    procedure CheckForUnusedClassMembers(const aUnit: TcwUnit; const aClass: TcwClass);
    procedure CheckForUnusedParameters(const aUnit: TcwUnit; const aClass: TcwClass; const aRoutine:
      TcwRoutine);
    procedure CheckForWrongCaseOfSymbol(const aUnit: TcwUnit; const aClass: TcwClass; const aContainer:
      TcwSymbolContainer);
  public
    function GetName: string; override;

    procedure CreateColumns; override;
    procedure CreateOptions; override;
    //run the report, so it will be filled with data
    procedure DoExecute; override;
  end;

procedure AddRPTpasLint;

implementation
uses
  TypInfo, Forms;
var
  Frm: TForm;

function IsFormStoredProperty(const aPropName: string): Boolean;
var
  vPropInfo: PPropInfo;
begin
  if Pos('Form', aPropName) = 1 then
  begin
    if Frm = nil then
      Frm := TForm.Create(nil);
    vPropInfo := GetPropInfo(Frm, 'On' + Copy(aPropName, 5, Length(aPropName) - 4));
    if vPropInfo <> nil then
    begin
      Result := IsStoredProp(Frm, vPropInfo);
    end
    else
      Result := False;

  end
  else
    Result := False;
end;

function TRPTpasLint.GetName: string;
begin
  Result := 'Pascal Lint';
end;

procedure TRPTpasLint.CreateColumns;
begin
  Columns.AddColumn('Unit', dtString);
  Columns.AddColumn('Class Name', dtString);
  Columns.AddColumn('Message', dtString);
  //Columns.AddColumn('Severity', dtInteger);
end;

procedure TRPTpasLint.CreateOptions;
begin

  FAncestorInterfaceNotImplemented := Settings.AddSetting('Ancestor Interface not implemented', True, True);

  FComponentPrefixes := Settings.AddSetting('Component Prefixes convention check', True, True);
  FIndentationConventions := Settings.AddSetting('Indentation conventions check', True, False);

  FConventionLocalVariables := Settings.AddSetting('Coding conventions for local variables', False, False);
  FConventionGlobalVariables := Settings.AddSetting('Coding conventions for global variables', True, False);
  FConventionConstants := Settings.AddSetting('Coding conventions for constants', True, False);
  FConventionClassFields := Settings.AddSetting('Coding conventions for class fields', True, False);
  FConventionClassTypes := Settings.AddSetting('Coding conventions for class types', True, False);
  FConventionInterfaceTypes := Settings.AddSetting('Coding conventions for interface types', True, False);
  FConventionRecordTypes := Settings.AddSetting('Coding conventions for record types', True, False);
  FConventionObjectTypes := Settings.AddSetting('Coding conventions for object types', True, False);
  FConventionPointerTypes := Settings.AddSetting('Coding conventions for pointer types', True, False);
  FConventionRoutineName := Settings.AddSetting('Coding conventions for routine names', True, False);
  FConventionConstParameters := Settings.AddSetting('Coding conventions for const parameters', True, False);
  FConventionVarParameters := Settings.AddSetting('Coding conventions for var parameters', True, False);
  FConventionParameters := Settings.AddSetting('Coding conventions for other parameters', True, False);
  FConventionProperties := Settings.AddSetting('Coding conventions for properties', True, False);
  FConventionGetRoutines := Settings.AddSetting('Coding conventions for Get routines', True, False);
  FConventionSetRoutines := Settings.AddSetting('Coding conventions for Set routines', True, False);

  FNonOverrideDestructor := Settings.AddSetting('Destructors without override', True, True);
  FVirtualDestructorOption := Settings.AddSetting('Destructors declared as virtual', True, True);

  FPublicFields := Settings.AddSetting('Fields declared as public', True, True);

  FGUIDNotDefined := Settings.AddSetting('GUID not defined in interface', True, True);
  FDuplicatedGUID := Settings.AddSetting('GUID declaration duplicated', True, True);

  FMoreThanOneDestructor := Settings.AddSetting('More than 1 destructor', True, True);

  FUnusedParametersAsConstOption := Settings.AddSetting('Non "const" Parameters', True, False);

  FRoutineNameToReturnRoutineResult := Settings.AddSetting('Routine Name used instead of Result', True,
    True);

  FSameRoutineImplemetsTwoInterfaces := Settings.AddSetting('Same routine implementing two interfaces', True,
    False);

  FUnusedClassFieldsOption := Settings.AddSetting('Unused Class Fields', True, False);
  FUnusedRecordFields := Settings.AddSetting('Unused Record Fields', True, False);
  FUnusedClassPropertiesOption := Settings.AddSetting('Unused Class Properties', True, False);
  FUnusedClassRoutinesOption := Settings.AddSetting('Unused Class Routines', True, False);

  FUnusedConstsOptions := Settings.AddSetting('Unused Constants', True, False);
  FUnusedRoutinesOptions := Settings.AddSetting('Unused Routines', True, False);
  FUnusedParametersOption := Settings.AddSetting('Unused Routine Parameters', True, False);

  FUnusedVarsOptions := Settings.AddSetting('Unused Variables', True, False);

  FVirtualRoutineWithNoDescendants := Settings.AddSetting('Virtual routines with no descendants', True,
    False);

  FCheckForWrongCase := Settings.AddSetting('Case mismatch on symbol', True, True);
end;

function TRPTpasLint.HasDescendingRoutine(const aClass: TcwClass; const aRoutine: TcwRoutine): Boolean;
var
  i: Integer;
var
  vDescendant: TcwClass;
begin
  Result := False;
  for i := 0 to aClass.Descendants.Count - 1 do
  begin
    vDescendant := aClass.Descendants[i] as TcwClass;
    Result := (vDescendant.Routines.Find(aRoutine.name) <> nil) or
      HasDescendingRoutine(vDescendant, aRoutine);
    if Result then
      EXIT;
  end
end;

procedure TRPTpasLint.CheckForUnusedVars(const aUnit: TcwUnit; const aClass: TcwClass; const aContainer:
  TcwVarContainer);
var
  vVariable: TcwVariable;
var
  vConstant: TcwVariable;
var
  vRoutine: TcwRoutine;
var
  i: Integer;
var
  vContainerName: string;
begin
  for i := 0 to aContainer.Variables.Count - 1 do
  begin
    vVariable := aContainer.Variables[i];
    if vVariable.UsedBy.Count = 0 then
    begin
      if aClass = nil then
        vContainerName := aContainer.Name
      else
        vContainerName := aClass.Name + '.' + aContainer.Name;
      Data.AddRow(aUnit, FUnusedVarsOptions, vVariable, nil, [aUnit.Name, vContainerName,
        Format('Unused variable: %s', [vVariable.name])]);
    end;
  end;

  for i := 0 to aContainer.Constants.Count - 1 do
  begin
    vConstant := aContainer.Constants[i];
    if vConstant.UsedBy.Count = 0 then
    begin
      if aClass = nil then
        vContainerName := aContainer.Name
      else
        vContainerName := aClass.Name + '.' + aContainer.Name;
      Data.AddRow(aUnit, FUnusedConstsOptions, vConstant, nil, [aUnit.Name, vContainerName,
        Format('Unused constant: %s', [vConstant.name])]);
    end;
  end;

  for i := 0 to aContainer.Routines.Count - 1 do
  begin
    vRoutine := aContainer.Routines[i];
    if vRoutine.UsedBy.Count = 0 then
    begin
      if aClass = nil then
        vContainerName := aContainer.Name
      else
        vContainerName := aClass.Name + '.' + aContainer.Name;
      Data.AddRow(aUnit, FUnusedRoutinesOptions, vRoutine, nil, [aUnit.Name, vContainerName,
        Format('Unused routine: %s', [vRoutine.name])]);
    end;
  end;
end;

procedure TRPTpasLint.CheckForWrongCaseOfSymbol(const aUnit: TcwUnit; const aClass: TcwClass; const
  aContainer: TcwSymbolContainer);
var
  vContainerName: string;
var
  i, j: Integer;
var
  vSymbol: TcwSymbol;
var
  vUsedBy: TcwUsedByLocation;

begin
  if (aClass = nil) or (aContainer = aClass) then
    vContainerName := aContainer.Name
  else
    vContainerName := aClass.Name + '.' + aContainer.Name;
  for i := 0 to aContainer.SymbolCount - 1 do
  begin
    vSymbol := aContainer.Symbols[i];
    for j := 0 to vSymbol.UsedBy.Count - 1 do
    begin
      vUsedBy := vSymbol.UsedBy[j];
      if vUsedBy.IsWrongCase then
        Data.AddRow(aUnit, FCheckForWrongCase, vUsedBy, nil,
          [vUsedBy.UnitIntf.Name, vContainerName, Format('Case mismatch on symbol: %s', [vSymbol.Name])]);
    end;
  end;
end;

procedure TRPTpasLint.CheckForReturnParameters(const aUnit: TcwUnit; const aRoutine: TcwRoutine);
var
  vReturnParam: TcwSymbol;
begin
  if aRoutine.RoutineType <> nil then
  begin
    vReturnParam := aRoutine.RoutineType;
    if vReturnParam.UsedBy.Count > 0 then
      Data.AddRow(aUnit, FRoutineNameToReturnRoutineResult, vReturnParam.UsedBy[0], nil,
        [aUnit.Name, aRoutine.Name, Format('Usage of routine name to return routine result: %s',
          [aRoutine.name])]);
  end;
end;

procedure TRPTpasLint.CheckForUnusedClassMembers(const aUnit: TcwUnit; const aClass: TcwClass);
var
  vVariable: TcwVariable;
var
  vRoutine: TcwRoutine;
var
  i: Integer;
begin
  for i := 0 to aClass.Fields.Count - 1 do
  begin
    vVariable := aClass.Fields[i];
    if vVariable.UsedBy.Count = 0 then
    begin
      Data.AddRow(aUnit, FUnusedClassFieldsOption, vVariable, nil, [aUnit.Name, aClass.Name,
        Format('Unused field: %s', [vVariable.name])]);
    end;
  end;

  for i := 0 to aClass.Properties.Count - 1 do
  begin
    vVariable := aClass.Properties[i];
    if vVariable.UsedBy.Count = 0 then
    begin
      //check if the property comes from an Interface definition :
      if not aClass.IsSymbolFromInterface(vVariable.name) then
        Data.AddRow(aUnit, FUnusedClassPropertiesOption, vVariable, nil, [aUnit.Name, aClass.Name,
          Format('Unused property: %s', [vVariable.name])]);
    end;
  end;

  for i := 0 to aClass.Routines.Count - 1 do
  begin
    vRoutine := aClass.Routines[i];
    if (not (rdOverride in vRoutine.Directives)) and
      (not (rdVirtual in vRoutine.Directives)) and
      (not (rdMessage in vRoutine.Directives)) then
    begin
      if vRoutine.UsedBy.Count = 0 then
      begin
        //check if the routine comes from an Interface definition :
        if not aClass.IsSymbolFromInterface(vRoutine.name) and
          not IsFormStoredProperty(vRoutine.name) then
          Data.AddRow(aUnit, FUnusedClassRoutinesOption, vRoutine, nil, [aUnit.Name, aClass.Name,
            Format('Unused routine: %s', [vRoutine.name])]);
      end;
    end;
  end;
end;

procedure TRPTpasLint.CheckForUnusedParameters(const aUnit: TcwUnit; const aClass: TcwClass; const aRoutine:
  TcwRoutine);
var
  i: Integer;
var
  vParam: TcwParameter;
var
  vRoutineName: string;
begin
  if (not (rdAbstract in aRoutine.Directives)) and
    (not (rdExternal in aRoutine.Directives)) and
    (not (rdMessage in aRoutine.Directives)) and
    (not (rdVirtual in aRoutine.Directives)) and
    (not (rdOverride in aRoutine.Directives)) then
  begin
    //if an event handler, do not display "unused" messages
    if (aClass <> nil) and ((aRoutine.IsUsedFromDFM) or (IsFormStoredProperty(aRoutine.Name)) or
      aClass.IsSymbolFromInterface(aRoutine.name)) then
      EXIT;
    for i := 0 to aRoutine.Parameters.Count - 1 do
    begin
      vParam := aRoutine.Parameters[i];
      if vParam.UsedBy.Count = 0 then
      begin
        if aClass = nil then
          vRoutineName := aRoutine.Name
        else
          vRoutineName := aClass.Name + '.' + aRoutine.Name;
        Data.AddRow(aUnit, FUnusedParametersOption, vParam, nil, [aUnit.Name, vRoutineName,
          Format('Unused parameter: %s', [vParam.name])]);
      end;
    end;
  end;
end;

procedure TRPTpasLint.CheckForNonConstParameters(const aUnit: TcwUnit; const aClass: TcwClass; const
  aRoutine: TcwRoutine);
var
  i: Integer;
  vParam: TcwParameter;
  vRoutineName: string;
begin
  for i := 0 to aRoutine.Parameters.Count - 1 do
  begin
    vParam := aRoutine.Parameters[i];
    if not (vParam.ParamKind in [pkVar, pkConst, pkIn, pkOut]) and
      (vParam.ParamType <> nil) then
    begin
      if (vParam.ParamType.TypeKind in TStringTypes) or
        (vParam.ParamType.TypeKind in TVariantTypes) then
      begin
        if aClass = nil then
          vRoutineName := aRoutine.Name
        else
          vRoutineName := aClass.Name + '.' + aRoutine.Name;
        Data.AddRow(aUnit, FUnusedParametersAsConstOption, aRoutine, nil, [aUnit.Name, vRoutineName,
          Format('Parameter %s should be declared as const', [vParam.name])]);
      end;
    end;
  end;
end;

procedure TRPTpasLint.CheckForComponentPrefixes(const aUnit: TcwUnit);
var
  vSymbol: TcwSymbol;
var
  vType: TcwSymbol;
var
  i, j: Integer;
var
  aMembersList: TcwSymbolList;
var
  vPrefixList: IcwPrefixList;
var
  vPrefix: string;
begin
  if aUnit.DFMResource <> nil then
    if (aUnit.IsLibraryUnit and FComponentPrefixes.LibrarySet) or
      (not aUnit.IsLibraryUnit and FComponentPrefixes.ProjectSet) then
    begin
      vPrefixList := gcwProject.CodingConventions.ComponentPrefixes;
      for i := 0 to aUnit.Classes.Count - 1 do
      begin
        Assert(False); //TODO: aMembersList is never created
        aUnit.Classes[i].GetMembersWithVisibility(aMembersList, [vkPublished]);
        try
          for j := 0 to aMembersList.Count - 1 do
          begin
            vSymbol := aMembersList[j];
            vType := vSymbol.TypeIntf;
            if (vType <> nil) then
            begin
              if not vPrefixList.ComponentPrefixComplies(vType.Name, vSymbol, vPrefix) then
                Data.AddRow(vSymbol, nil, [aUnit.Name, vSymbol.Name, 'Component prefix mismatch - ' +
                  vPrefix]);
            end;
          end;
        finally
          aMembersList.Free;
        end;
      end;
    end;
end;

procedure TRPTpasLint.CheckForIndentationConventions(const aUnit: TcwUnit);
var
  i: Integer;
var
  vErrorList: TcwCodingErrorsList;
var
  vCodeError: TcwCodeError;
begin
  if (aUnit.IsLibraryUnit and FIndentationConventions.LibrarySet) or
    (not aUnit.IsLibraryUnit and FIndentationConventions.ProjectSet) then
  begin
    vErrorList := aUnit.CodingErrorsList;
    for i := 0 to vErrorList.Count - 1 do
    begin
      vCodeError := vErrorList[i];
      if vCodeError.ErrorKind = ekBadIndentation then
      begin
        Data.AddRow(vCodeError, nil, [aUnit.Name, '', 'Bad indentation']);
      end;
    end;
  end;
end;

procedure TRPTpasLint.CheckUnit(const aUnit: TcwUnit);
var
  i: Integer;
var
  vConventions: IcwNamingConventions;
var
  vVariable: TcwVariable;

begin
  CheckForComponentPrefixes(aUnit);
  CheckForIndentationConventions(aUnit);

  vConventions := gcwProject.CodingConventions.NamingConventions;
  for i := 0 to aUnit.Variables.Count - 1 do
  begin
    vVariable := aUnit.Variables[i];
    if not gcwProject.RegularExpressions.Match(vConventions.GlobalVariables, vVariable.Name) then
      Data.AddRow(aUnit, FConventionGlobalVariables, vVariable, nil,
        [aUnit.Name, '', 'Global variable naming error - ' + vConventions.GlobalVariables]);
  end;

  for i := 0 to aUnit.Constants.Count - 1 do
  begin
    vVariable := aUnit.Constants[i];
    if not gcwProject.RegularExpressions.Match(vConventions.Constants, vVariable.Name) then
      Data.AddRow(aUnit, FConventionConstants, vVariable, nil,
        [aUnit.Name, '', 'Constant naming error - ' + vConventions.Constants]);
  end;

end;

procedure TRPTpasLint.CheckClasses(const aUnit: TcwUnit; const aClassList: TcwClassList);
var
  i, j, k, l: Integer;
var
  vClass: TcwClass;
var
  DestructorsCount: integer;
var
  vRoutine: TcwRoutine;
var
  vList: TStringList;
var
  vInterface: TcwInterface;
var
  vImplInterface: TcwInterface;
var
  bFound: Boolean;
var
  vField: TcwVariable;
var
  vProperty: TcwProperty;
var
  vConventions: IcwNamingConventions;
var
  vSymbol: TcwSymbol;
begin
  for i := 0 to aClassList.Count - 1 do
  begin
    vClass := aClassList[i];
    DestructorsCount := 0;
    for j := 0 to vClass.Routines.Count - 1 do
    begin
      vRoutine := vClass.Routines[j];
      //check for destructors errors

      if vRoutine.RoutineKind = rkDestructor then
      begin
        inc(DestructorsCount);
        if not (rdOverride in vRoutine.Directives) then
          Data.AddRow(aUnit, FNonOverrideDestructor, vRoutine, nil, [aUnit.Name, vClass.Name,
            Format('Destructor must be declared as override: %s', [vRoutine.name])]);
        if (vClass.InheritsFromClass <> nil) and
          ((rdVirtual in vRoutine.Directives) or
          (rdDynamic in vRoutine.Directives)) then
          Data.AddRow(aUnit, FVirtualDestructorOption, vRoutine, nil, [aUnit.Name, vClass.Name,
            Format('Destructor should not be declared as virtual: %s', [vRoutine.name])]);
      end;

      //check for virtual routines errors :
      if (rdVirtual in vRoutine.Directives) or
        (rdDynamic in vRoutine.Directives) then
      begin
        if not HasDescendingRoutine(vClass, vRoutine) then
          Data.AddRow(aUnit, FVirtualRoutineWithNoDescendants, vRoutine, nil, [aUnit.Name, vClass.Name,
            Format('Virtual routine has no descendants: %s', [vRoutine.name])]);
      end;

      CheckRoutine(aUnit, vClass, vRoutine);
    end;
    CheckForWrongCaseOfSymbol(aUnit, vClass, vClass);

    vConventions := gcwProject.CodingConventions.NamingConventions;

    if not gcwProject.RegularExpressions.Match(vConventions.ClassTypes, vClass.Name) then
      Data.AddRow(aUnit, FConventionClassTypes, vClass, nil,
        [aUnit.Name, '', 'Class type naming error - ' + vConventions.ClassTypes]);

    if DestructorsCount > 1 then
      Data.AddRow(aUnit, FMoreThanOneDestructor, vClass, nil, [aUnit.Name, vClass.Name,
        Format('Too many destructors declared (%d)', [DestructorsCount])]);

    vList := TStringList.Create;
    try
      vList.Sorted := True;

      for j := 0 to vClass.Interfaces.Count - 1 do
      begin
        vInterface := vClass.Interfaces[j];

        //check to see if any of the implemented interfaces have the same method,
        //this can lead to ambigous interface implementation (one method of the class will implement)
        //to methods from two interfaces
        for k := 0 to vInterface.Routines.Count - 1 do
        begin
          vRoutine := vInterface.Routines[k];
          //if the routine has been implemented in the class itself
          //and if the routine has not already been reported
          if (vClass.Routines.Find(vRoutine.Name) <> nil) and (vList.indexOf(vRoutine.Name) < 0) then
          begin
            for l := 0 to vClass.Interfaces.Count - 1 do
            begin
              vImplInterface := vClass.Interfaces[l];
              if vImplInterface <> vInterface then
              begin
                if vImplInterface.Routines.Find(vRoutine.Name) <> nil then
                begin
                  vList.Add(vRoutine.Name);
                  Data.AddRow(aUnit, FSameRoutineImplemetsTwoInterfaces, vRoutine, nil,
                    [aUnit.Name, vClass.Name,
                      Format('Ambiguous routine naming : %s is declared in %s and %s',
                      [vRoutine.Name, vInterface.name, vImplInterface.Name])]);
                end;
              end;
            end;
          end;
        end;

        //check to see if the class implements all interafces that in interface might be inheriting from
        for k := 0 to vInterface.Interfaces.Count - 1 do
        begin
          vImplInterface := vInterface.Interfaces[k];
          bFound := False;
          for l := 0 to vClass.Interfaces.Count - 1 do
          begin
            if vImplInterface = vClass.Interfaces[l] then
            begin
              bFound := True;
              BREAK;
            end;
          end;
          if not bFound then
            Data.AddRow(aUnit, FAncestorInterfaceNotImplemented, vClass, nil,
              [aUnit.Name, vClass.Name,
                Format('Class does not implement interface %s, from which %s is descending',
                [vImplInterface.Name, vInterface.name])]);
        end;
      end;
    finally
      vList.Free;
    end;

    for j := 0 to vClass.Fields.Count - 1 do
    begin
      vField := vClass.Fields[j];
      if vField.Visibilty = vkPublic then
        Data.AddRow(aUnit, FPublicFields, vField, nil,
          [aUnit.Name, vClass.Name, Format('Field is declared as public: %s', [vField.name])]);

      if (aUnit.DFMResource = nil) or (vField.Visibilty <> vkPublished) then
      begin
        if not gcwProject.RegularExpressions.Match(vConventions.ClassFields, vField.Name) then
          Data.AddRow(aUnit, FConventionClassFields, vField, nil,
            [aUnit.Name, vClass.Name, 'Class field naming error - ' + vConventions.ClassFields]);
      end;
    end;

    for j := 0 to vClass.Properties.Count - 1 do
    begin
      vProperty := vClass.Properties[j];
      if not gcwProject.RegularExpressions.Match(vConventions.Properties, vProperty.Name) then
        Data.AddRow(aUnit, FConventionProperties, vProperty, nil,
          [aUnit.Name, vClass.Name, 'Class property naming error - ' + vConventions.Properties]);

      vSymbol := vProperty.ReadAccess;

      if (vSymbol <> nil) and (vSymbol is TcwRoutine) then
      begin
        vRoutine := vSymbol as TcwRoutine;
        if not gcwProject.RegularExpressions.Match(vConventions.GetRoutines, vRoutine.Name) then
          Data.AddRow(aUnit, FConventionGetRoutines, vSymbol, nil,
            [aUnit.Name, vClass.Name, 'Property Get routine naming error - ' + vConventions.GetRoutines]);

      end;

      vSymbol := vProperty.WriteAccess;
      if (vSymbol <> nil) and (vSymbol is TcwRoutine) then
      begin
        vRoutine := vSymbol as TcwRoutine;
        if not gcwProject.RegularExpressions.Match(vConventions.SetRoutines, vRoutine.Name) then
          Data.AddRow(aUnit, FConventionSetRoutines, vSymbol, nil,
            [aUnit.Name, vClass.Name, 'Property Set routine naming error - ' + vConventions.SetRoutines]);

      end;
    end;

    CheckForUnusedClassMembers(aUnit, vClass);
  end;
end;

procedure TRPTpasLint.CheckRecords(const aUnit: TcwUnit);
var
  vVariable: TcwVariable;
var
  vRecord: TcwRecord;
var
  i, j: Integer;
var
  vConventions: IcwNamingConventions;
begin

  for i := 0 to aUnit.Records.Count - 1 do
  begin
    vRecord := aUnit.Records[i];

    for j := 0 to vRecord.Fields.Count - 1 do
    begin
      vVariable := vRecord.Fields[j];
      if vVariable.UsedBy.Count = 0 then
      begin
        Data.AddRow(aUnit, FUnusedRecordFields, vVariable, nil,
          [aUnit.Name, vRecord.Name, Format('Unused field: %s', [vVariable.name])]);
      end;
    end;
    CheckForWrongCaseOfSymbol(aUnit, nil, vRecord);

    vConventions := gcwProject.CodingConventions.NamingConventions;

    if not gcwProject.RegularExpressions.Match(vConventions.RecordTypes, vRecord.Name) then
      Data.AddRow(aUnit, FConventionRecordTypes, vRecord, nil,
        [aUnit.Name, '', 'Record type naming error - ' + vConventions.RecordTypes]);

  end;
end;

procedure TRPTpasLint.CheckRoutineConventions(const aUnit: TcwUnit; const aClass: TcwClass; const aRoutine:
  TcwRoutine);
var
  i: Integer;
var
  vParam: TcwParameter;
var
  vConventions: IcwNamingConventions;
var
  vVariable: TcwVariable;
begin
  if (not (rdAbstract in aRoutine.Directives)) and
    (not (rdExternal in aRoutine.Directives)) and
    (not (rdMessage in aRoutine.Directives)) and
    (not (rdOverride in aRoutine.Directives)) then
  begin
    //if an event handler, do not display "unused" messages
    if (aClass <> nil) and ((aRoutine.IsUsedFromDFM) or (IsFormStoredProperty(aRoutine.Name))) then
      EXIT;

    vConventions := gcwProject.CodingConventions.NamingConventions;
    if not gcwProject.RegularExpressions.Match(vConventions.RoutineName, aRoutine.Name) then
      Data.AddRow(aUnit, FConventionRoutineName, aRoutine, nil,
        [aUnit.Name, aRoutine.Name, 'Routine naming error - ' + vConventions.RoutineName]);

    for i := 0 to aRoutine.Variables.Count - 1 do
    begin
      vVariable := aRoutine.Variables[i];
      if not gcwProject.RegularExpressions.Match(vConventions.LocalVariables, vVariable.Name) then
        Data.AddRow(aUnit, FConventionLocalVariables, vVariable, nil,
          [aUnit.Name, aRoutine.Name, 'Local variable naming error - ' + vConventions.LocalVariables]);
    end;

    for i := 0 to aRoutine.Constants.Count - 1 do
    begin
      vVariable := aRoutine.Constants[i];
      if not gcwProject.RegularExpressions.Match(vConventions.Constants, vVariable.Name) then
        Data.AddRow(aUnit, FConventionConstants, vVariable, nil,
          [aUnit.Name, aRoutine.Name, 'Constant naming error - ' + vConventions.Constants]);
    end;

    for i := 0 to aRoutine.Parameters.Count - 1 do
    begin
      vParam := aRoutine.Parameters[i];
      if (vParam.ParamKind in [pkVar, pkOut]) then
      begin
        if not gcwProject.RegularExpressions.Match(vConventions.VarParameters, vParam.Name) then
          Data.AddRow(aUnit, FConventionVarParameters, vParam, nil,
            [aUnit.Name, aRoutine.Name, 'Var parameter naming error - ' + vConventions.VarParameters]);
      end
      else if (vParam.ParamKind in [pkConst, pkIn]) then
      begin
        if not gcwProject.RegularExpressions.Match(vConventions.ConstParameters, vParam.Name) then
          Data.AddRow(aUnit, FConventionConstParameters, vParam, nil,
            [aUnit.Name, aRoutine.Name, 'Const parameter naming error - ' + vConventions.ConstParameters]);

      end
      else
      begin
        if not gcwProject.RegularExpressions.Match(vConventions.Parameters, vParam.Name) then
          Data.AddRow(aUnit, FConventionParameters, vParam, nil,
            [aUnit.Name, aRoutine.Name, 'Parameter naming error - ' + vConventions.Parameters]);

      end;

    end;
  end;
end;

procedure TRPTpasLint.CheckRoutine(const aUnit: TcwUnit; const aClass: TcwClass; const aRoutine: TcwRoutine);
begin
  CheckForNonConstParameters(aUnit, aClass, aRoutine);
  CheckForUnusedParameters(aUnit, aClass, aRoutine);
  CheckForUnusedVars(aUnit, aClass, aRoutine);
  CheckForReturnParameters(aUnit, aRoutine);
  CheckForWrongCaseOfSymbol(aUnit, aClass, aRoutine);
  CheckRoutineConventions(aUnit, aClass, aRoutine);
end;

//run the report, so it will be filled with Data

procedure TRPTpasLint.DoExecute;

var
  i, j, k: Integer;
var
  vUnit: TcwUnit;
var
  vInterface: TcwInterface;
var
  GUIDList: TStringList;
var
  vRoutine: TcwRoutine;
var
  Index: Integer;
begin
  GUIDList := TStringList.Create;
  try
    GUIDList.Sorted := True;
    for i := 0 to gcwProject.AllUnits.Count - 1 do
    begin
      vUnit := gcwProject.AllUnits[i];
      CheckUnit(vUnit);

      CheckClasses(vUnit, vUnit.Classes);
      CheckClasses(vUnit, vUnit.Objects);
      CheckRecords(vUnit);

      for j := 0 to vUnit.Interfaces.Count - 1 do
      begin
        vInterface := vUnit.Interfaces[j];
        CheckForWrongCaseOfSymbol(vUnit, vInterface, vInterface);
        if vInterface.GUID = '' then
          Data.AddRow(vUnit, FGUIDNotDefined, vInterface, nil,
            [vUnit.Name, vInterface.Name, 'Interface has no GUID defined'])
        else
        begin
          if GUIDList.Find(vInterface.GUID, Index) then
          begin
            Data.AddRow(vUnit, FDuplicatedGUID, vInterface, nil, [vUnit.Name, vInterface.Name,
              Format('Duplicate GUID defined in %s and %s',
                [vInterface.Name, TcwInterface(Pointer(GUIDList.Objects[Index])).Name])]);
          end
          else
            GUIDList.AddObject(vInterface.GUID, Pointer(vInterface));
        end;

        for k := 0 to vInterface.Routines.Count - 1 do
        begin
          vRoutine := vInterface.Routines[k];
          CheckForNonConstParameters(vUnit, vInterface, vRoutine);
        end;
      end;

      for j := 0 to vUnit.Routines.Count - 1 do
      begin
        vRoutine := vUnit.Routines[j];
        CheckRoutine(vUnit, nil, vRoutine);
      end;
      CheckForUnusedVars(vUnit, nil, vUnit);
      CheckForWrongCaseOfSymbol(vUnit, nil, vUnit);
    end;
  finally
    GUIDList.Free;
  end;

end;

procedure AddRPTpasLint;
begin
  gcwProject.Reports.AddReport(TRPTpasLint.Create);
end;

initialization
finalization
  FreeAndNil(Frm);
end.

