(*  Copyright (c) 2002, the CodeLens project
  All rights reserved.

  Redistribution and use in source and binary forms,
  with or without modification, are permitted provided
  that the following conditions are met:

  Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.

  Redistributions in binary form must reproduce the
  above copyright notice, this list of conditions
  and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
  Neither the name of the CodeLens project nor the names
  of its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS
  AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. *)

unit ucwPrefixList;

interface

uses
  SysUtils,
  Classes,
  ucwClasses;

type
  TcwPrefixList = class(TInterfacedObject, IcwPrefixList)
  private
    FList: TInterfaceList;
    //FSortedList: IcwSortedInterfaceList;
    FEnforcePrefixOnUnreferecend: Boolean;
  public
    constructor Create;
    destructor Destroy; override;
    function GetItems(Index: Integer): IcwComponentPrefix;
    property Items[Index: Integer]: IcwComponentPrefix read GetItems; default;

    function GetCount: Integer;
    property Count: Integer read GetCount;

    procedure Clear;

    procedure LoadDefaultPrefixes;

    function Add(const aPrefix: string; const aComponentClass: string): IcwComponentPrefix;
    procedure Delete(Index: Integer);
    procedure DeletePrefix(const aPrefix: IcwComponentPrefix);

    function FindComponentClass(const aComponentClass: string): IcwComponentPrefix;
    function ComponentPrefixComplies(const aComponentClass: string; const aSymbol: TcwSymbol;
      var aExpectedPrefix: string): Boolean;

    //if true, will enforce the component prefix requirements even on components
    //that are not referenced in the source code (like labels, panels etc. that are only visual)
    function GetEnforcePrefixOnUnreferecend: Boolean;
    procedure SetEnforcePrefixOnUnreferecend(aValue: Boolean);
    property EnforcePrefixOnUnreferecend: Boolean read GetEnforcePrefixOnUnreferecend write
      SetEnforcePrefixOnUnreferecend;

    procedure Save(const aFile: IcwFileSettings);
    procedure Load(const aFile: IcwFileSettings);
  end;

implementation

uses
  ucwComponentPrefix,
  ucwStringList,
  ucwRegExpr;

constructor TcwPrefixList.Create;
begin
  inherited Create;
  FList := TInterfaceList.Create;
  //FSortedList := TcwSortedInterfaceList.Create;
  Clear;
end;

destructor TcwPrefixList.Destroy;
begin
  FList.Free;
  inherited Destroy;
end;

function TcwPrefixList.GetItems(Index: Integer): IcwComponentPrefix;
begin
  Result := IcwComponentPrefix(FList[Index]);
end;

function TcwPrefixList.GetCount: Integer;
begin
  Result := FList.Count;
end;

function TcwPrefixList.Add(const aPrefix: string; const aComponentClass: string): IcwComponentPrefix;
begin
  Result := TcwComponentPrefix.Create(aPrefix, aComponentClass);
  FList.Add(Result);
  //FSortedList.Add(Result.ComponentClass, Result);
end;

procedure TcwPrefixList.Delete(Index: Integer);
var
  vPrefix: IcwComponentPrefix;
  AIndex: Integer;
begin
Assert(False); //TODO
//  FSortedList.Sorted := true;
//  vPrefix := IcwComponentPrefix(FList[Index]);
//  if FSortedList.Find(vPrefix.ComponentClass, AIndex) then
//  begin
//    FList.Delete(Index);
//    FSortedList.Delete(AIndex);
//  end;
end;

procedure TcwPrefixList.DeletePrefix(const aPrefix: IcwComponentPrefix);
var
  i: Integer;
var
  Index: Integer;
begin
  //TODO:
  Assert(False);
  for i := 0 to Count - 1 do
  begin
    if FList[i] = aPrefix then
    begin
//      if FSortedList.Find(aPrefix.ComponentClass, Index) then
//      begin
//        FList.Delete(i);
//        FSortedList.Delete(Index);
//        BREAK;
//      end;
    end;
  end;

end;

function TcwPrefixList.FindComponentClass(const aComponentClass: string): IcwComponentPrefix;
var
  Index: Integer;
begin
  //TODO:
  Assert(False);
//  FSortedList.Sorted := true;
//  if FSortedList.Find(aComponentClass, Index) then
//    Result := FSortedList[Index] as IcwComponentPrefix
//  else
//    Result := nil;
end;

function TcwPrefixList.ComponentPrefixComplies(const aComponentClass: string;
  const aSymbol: TcwSymbol; var aExpectedPrefix: string): Boolean;
var
  vPrefix: IcwComponentPrefix;
  i: Integer;
  vLocation: TcwUsedByLocation;
begin
  Result := True;
  vPrefix := FindComponentClass(aComponentClass);
  if vPrefix <> nil then
  begin
    Result := gcwProject.RegularExpressions.Match(vPrefix.Prefix, aSymbol.Name);
    if not Result then
    begin
      aExpectedPrefix := vPrefix.Prefix;
      if FEnforcePrefixOnUnreferecend = False then
      begin
        for i := 0 to aSymbol.UsedBy.Count - 1 do
        begin
          vLocation := aSymbol.UsedBy[i];
          if (aSymbol.UnitIndex <> vLocation.UnitIndex) or
            (aSymbol.FileIndex = vLocation.FileIndex) then
          begin
            EXIT;
          end;
        end;
        //all the references are from the unit's DFM file
        Result := True;
      end;
    end;
  end;
end;

//if true, will enforce the component prefix requirements even on components
//that are not referenced in the source code (like labels, panels etc. that are only visual)

function TcwPrefixList.GetEnforcePrefixOnUnreferecend: Boolean;
begin
  Result := FEnforcePrefixOnUnreferecend;
end;

procedure TcwPrefixList.SetEnforcePrefixOnUnreferecend(aValue: Boolean);
begin
  FEnforcePrefixOnUnreferecend := aValue;
end;

procedure TcwPrefixList.Clear;
begin
  FList.Clear;
//  FSortedList.Clear;
//  FSortedList.Sorted := False;
  FEnforcePrefixOnUnreferecend := false;
end;

procedure TcwPrefixList.Save(const aFile: IcwFileSettings);
var
  i: Integer;
  aStringList: IcwStringList;
  vPrefix: IcwComponentPrefix;
  s: string;
begin

  aFile.SetOption('Options', 'EnforcePrefixOnUnreferecend', FEnforcePrefixOnUnreferecend);

  aStringList := TcwStringList.Create;
  for i := 0 to Count - 1 do
  begin
    vPrefix := Items[i];
    s := vPrefix.Prefix + '=' + vPrefix.ComponentClass;
    aStringList.Add(s);
  end;
  aFile.SaveSection('Prefixes', aStringList);

end;

procedure TcwPrefixList.Load(const aFile: IcwFileSettings);
var
  i: Integer;
  aStringList: IcwStringList;
  vPrefix: string;
  vComponentClass: string;
  s: string;
begin
  Clear;
  FEnforcePrefixOnUnreferecend := aFile.GetOption('Options', 'EnforcePrefixOnUnreferecend',
    FEnforcePrefixOnUnreferecend);

  aStringList := TcwStringList.Create;
  aFile.ReadSectionValues('Prefixes', aStringList);
  for i := 0 to aStringList.Count - 1 do
  begin
    s := aStringList[i];
    vPrefix := Trim(Copy(s, 1, Pos('=', s) - 1));
    vComponentClass := Trim(Copy(s, Pos('=', s) + 1, Length(s)));
    if (vPrefix <> '') and (vComponentClass <> '') then
      Add(vPrefix, vComponentClass);
  end;
end;

procedure TcwPrefixList.LoadDefaultPrefixes;
begin
  Clear;
  //global options
  EnforcePrefixOnUnreferecend := False;

  //add list of defaults : see http://www.econos.de/delphi/cs.htm

  //standard Tab
  Add('mm[A-Z].+', 'TMainMenu');
  Add('pm[A-Z].+', 'TPopupMenu');
  Add('mmi[A-Z].+', 'TMainMenuItem');
  Add('pmi[A-Z].+', 'TPopupMenuItem');
  Add('lbl[A-Z].+', 'TLabel');
  Add('edt[A-Z].+', 'TEdit');
  Add('mem[A-Z].+', 'TMemo');
  Add('btn[A-Z].+', 'TButton');
  Add('chk[A-Z].+', 'TCheckBox');
  Add('rb[A-Z].+', 'TRadioButton');
  Add('lb[A-Z].+', 'TListBox');
  Add('cb[A-Z].+', 'TComboBox');
  Add('scb[A-Z].+', 'TScrollBar');
  Add('gb[A-Z].+', 'TGroupBox');
  Add('rg[A-Z].+', 'TRadioGroup');
  Add('pnl[A-Z].+', 'TPanel');
  Add('cl[A-Z].+', 'TCommandList');

  //Additional Tab
  Add('bbtn[A-Z].+', 'TBitBtn');
  Add('sb[A-Z].+', 'TSpeedButton');
  Add('me[A-Z].+', 'TMaskEdit');
  Add('sg[A-Z].+', 'TStringGrid');
  Add('dg[A-Z].+', 'TDrawGrid');
  Add('img[A-Z].+', 'TImage');
  Add('shp[A-Z].+', 'TShape');
  Add('bvl[A-Z].+', 'TBevel');
  Add('sbx[A-Z].+', 'TScrollBox');
  Add('clb[A-Z].+', 'TCheckListbox');
  Add('spl[A-Z].+', 'TSplitter');
  Add('stx[A-Z].+', 'TStaticText');
  Add('cht[A-Z].+', 'TChart');

  //Win32 Tab
  Add('tbc[A-Z].+', 'TTabControl');
  Add('pgc[A-Z].+', 'TPageControl');
  Add('il[A-Z].+', 'TImageList');
  Add('re[A-Z].+', 'TRichEdit');
  Add('tbr[A-Z].+', 'TTrackBar');
  Add('prb[A-Z].+', 'TProgressBar');
  Add('ud[A-Z].+', 'TUpDown');
  Add('hk[A-Z].+', 'THotKey');
  Add('ani[A-Z].+', 'TAnimate');
  Add('dtp[A-Z].+', 'TDateTimePicker');
  Add('tv[A-Z].+', 'TTreeView');
  Add('lv[A-Z].+', 'TListView');
  Add('hdr[A-Z].+', 'THeaderControl');
  Add('stb[A-Z].+', 'TStatusBar');
  Add('tlb[A-Z].+', 'TToolBar');
  Add('clb[A-Z].+', 'TCoolBar');

  //System Tab
  Add('tm[A-Z].+', 'TTimer');
  Add('pb[A-Z].+', 'TPaintBox');
  Add('mp[A-Z].+', 'TMediaPlayer');
  Add('olec[A-Z].+', 'TOleContainer');
  Add('ddcc[A-Z].+', 'TDDEClientConv');
  Add('ddci[A-Z].+', 'TDDEClientItem');
  Add('ddsc[A-Z].+', 'TDDEServerConv');
  Add('ddsi[A-Z].+', 'TDDEServerItem');

  //Internet Tab
  Add('csk[A-Z].+', 'TClientSocket');
  Add('ssk[A-Z].+', 'TServerSocket');
  Add('wbd[A-Z].+', 'TWebDispatcher');
  Add('pp[A-Z].+', 'TPageProducer');
  Add('tp[A-Z].+', 'TQueryTableProducer');
  Add('dstp[A-Z].+', 'TDataSetTableProducer');
  Add('nmdt[A-Z].+', 'TNMDayTime');
  Add('nec[A-Z].+', 'TNMEcho');
  Add('nf[A-Z].+', 'TNMFinger');
  Add('nftp[A-Z].+', 'TNMFtp');
  Add('nhttp[A-Z].+', 'TNMHttp');
  Add('nMsg[A-Z].+', 'TNMMsg');
  Add('nmsg[A-Z].+', 'TNMMSGServ');
  Add('nntp[A-Z].+', 'TNMNNTP');
  Add('npop[A-Z].+', 'TNMPop3');
  Add('nuup[A-Z].+', 'TNMUUProcessor');
  Add('smtp[A-Z].+', 'TNMSMTP');
  Add('nst[A-Z].+', 'TNMStrm');
  Add('nsts[A-Z].+', 'TNMStrmServ');
  Add('ntm[A-Z].+', 'TNMTime');
  Add('nudp[A-Z].+', 'TNMUdp');
  Add('psk[A-Z].+', 'TPowerSock');
  Add('ngs[A-Z].+', 'TNMGeneralServer');
  Add('html[A-Z].+', 'THtml');
  Add('url[A-Z].+', 'TNMUrl');
  Add('sml[A-Z].+', 'TSimpleMail');

  //Data Access Tab
  Add('ds[A-Z].+', 'TDataSource');
  Add('tbl[A-Z].+', 'TTable');
  Add('qry[A-Z].+', 'TQuery');
  Add('sp[A-Z].+', 'TStoredProc');
  Add('db[A-Z].+', 'TDataBase');
  Add('ssn[A-Z].+', 'TSession');
  Add('bm[A-Z].+', 'TBatchMove');
  Add('usql[A-Z].+', 'TUpdateSQL');

  //Data Controls Tab
  Add('dbg[A-Z].+', 'TDBGrid');
  Add('dbn[A-Z].+', 'TDBNavigator');
  Add('dbt[A-Z].+', 'TDBText');
  Add('dbe[A-Z].+', 'TDBEdit');
  Add('dbm[A-Z].+', 'TDBMemo');
  Add('dbi[A-Z].+', 'TDBImage');
  Add('dblb[A-Z].+', 'TDBListBox');
  Add('dbcb[A-Z].+', 'TDBComboBox');
  Add('dbch[A-Z].+', 'TDBCheckBox');
  Add('dbrg[A-Z].+', 'TDBRadioGroup');
  Add('dbll[A-Z].+', 'TDBLookupListBox');
  Add('dblc[A-Z].+', 'TDBLookupComboBox');
  Add('dbre[A-Z].+', 'TDBRichEdit');
  Add('dbcg[A-Z].+', 'TDBCtrlGrid');
  Add('dbch[A-Z].+', 'TDBChart');

  //Decision Cube Tab
  Add('dcb[A-Z].+', 'TDecisionCube');
  Add('dcq[A-Z].+', 'TDecisionQuery');
  Add('dcs[A-Z].+', 'TDecisionSource');
  Add('dcp[A-Z].+', 'TDecisionPivot');
  Add('dcg[A-Z].+', 'TDecisionGrid');
  Add('dcgr[A-Z].+', 'TDecisionGraph');

  //QReport Tab
  Add('qr[A-Z].+', 'TQuickReport');
  Add('qrsd[A-Z].+', 'TQRSubDetail');
  Add('qrb[A-Z].+', 'TQRBand');
  Add('qrcb[A-Z].+', 'TQRChildBand');
  Add('qrg[A-Z].+', 'TQRGroup');
  Add('qrl[A-Z].+', 'TQRLabel');
  Add('qrt[A-Z].+', 'TQRText');
  Add('qre[A-Z].+', 'TQRExpr');
  Add('qrs[A-Z].+', 'TQRSysData');
  Add('qrm[A-Z].+', 'TQRMemo');
  Add('qrrt[A-Z].+', 'TQRRichText');
  Add('qrdr[A-Z].+', 'TQRDBRichText');
  Add('qrsh[A-Z].+', 'TQRShape');
  Add('qri[A-Z].+', 'TQRImage');
  Add('qrdi[A-Z].+', 'TQRDBMImage');
  Add('qrcr[A-Z].+', 'TQRCompositeReport');
  Add('qrp[A-Z].+', 'TQRPreview');
  Add('qrch[A-Z].+', 'TQRChart');

  //Dialogs Tab
  Add('ofd[A-Z].+', 'TOpenDialog');
  Add('sfd[A-Z].+', 'TSaveDialog');
  Add('opd[A-Z].+', 'TOpenPictureDialog');
  Add('spd[A-Z].+', 'TSavePictureDialog');
  Add('fntd[A-Z].+', 'TFontDialog');
  Add('clrd[A-Z].+', 'TColorDialog');
  Add('prtd[A-Z].+', 'TPrintDialog');
  Add('prsd[A-Z].+', 'TPrintSetupDialog');
  Add('fndd[A-Z].+', 'TFindDialog');
  Add('rpld[A-Z].+', 'TReplaceDialog');

  //Win31 Tab
  Add('dbll[A-Z].+', 'TDBLookupList');
  Add('dblc[A-Z].+', 'TDBLookupCombo');
  Add('ts[A-Z].+', 'TTabSet');
  Add('ol[A-Z].+', 'TOutline');
  Add('tnb[A-Z].+', 'TTabbedNoteBook');
  Add('nb[A-Z].+', 'TNoteBook');
  Add('hdr[A-Z].+', 'THeader');
  Add('flb[A-Z].+', 'TFileListBox');
  Add('dlb[A-Z].+', 'TDirectoryListBox');
  Add('dcb[A-Z].+', 'TDriveComboBox');
  Add('fcb[A-Z].+', 'TFilterComboBox');

  //Samples Tab
  Add('gg[A-Z].+', 'TGauge');
  Add('cg[A-Z].+', 'TColorGrid');
  Add('spb[A-Z].+', 'TSpinButton');
  Add('spe[A-Z].+', 'TSpinEdit');
  Add('dol[A-Z].+', 'TDirectoryOutline');
  Add('cal[A-Z].+', 'TCalendar');
  Add('ibea[A-Z].+', 'TIBEventAlerter');

  //ActiveX Tab
  Add('cfx[A-Z].+', 'TChartFX');
  Add('vsp[A-Z].+', 'TVSSpell');
  Add('f1b[A-Z].+', 'TF1Book');
  Add('vtc[A-Z].+', 'TVTChart');
  Add('grp[A-Z].+', 'TGraph');

  //Midas Tab
  Add('prv[A-Z].+', 'TProvider');
  Add('cds[A-Z].+', 'TClientDataSet');
  Add('qcds[A-Z].+', 'TQueryClientDataSet');
  Add('dcom[A-Z].+', 'TDCOMConnection');
  Add('olee[A-Z].+', 'TOleEnterpriseConnection');
  Add('sck[A-Z].+', 'TSocketConnection');
  Add('rms[A-Z].+', 'TRemoteServer');
  Add('mid[A-Z].+', 'TMidasConnection');
end;

end.

