(*  Copyright (c) 2002, the CodeLens project
  All rights reserved.

  Redistribution and use in source and binary forms,
  with or without modification, are permitted provided
  that the following conditions are met:

  Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.

  Redistributions in binary form must reproduce the
  above copyright notice, this list of conditions
  and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
  Neither the name of the CodeLens project nor the names
  of its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS
  AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. *)

unit ucwPreprocessor;

interface

uses
  SysUtils,
  Classes,
  mwPasLexTypes,
  mwSimplePasPar;

type
  TPreProcessor = class(TmwSimplePasPar)
  private
    FCurrentUnit: string;
    FUsedUnits: TStringList;

  protected
    function GetUnitCount: integer;

    // Program files
    procedure MainUsedUnitName; override;
    procedure MainUsedUnitExpression; override;
    procedure UsedUnitName; override;

    // Packages
    procedure ContainsStatement; override;
    procedure ContainsExpression; override;

  public
    constructor Create(ADefines: TmwAssignAbleHash);
    destructor Destroy; override;

    property UnitCount: Integer read GetUnitCount;
  end;

function PreProcessUsedUnits(const aFilename: string; sl: TStrings = nil): integer;

implementation

uses
  ucwClasses;

constructor TPreProcessor.Create(ADefines: TmwAssignAbleHash);
begin
  inherited Create;
  FCurrentUnit := '';
  FUsedUnits := TStringList.Create;
  Lexer.DefinedList.Assign(aDefines);
end;

destructor TPreProcessor.Destroy;
begin
  FUsedUnits.Free;
  inherited;
end;

function TPreProcessor.GetUnitCount: integer;
begin
  Result := FUsedUnits.Count;
end;

procedure TPreProcessor.MainUsedUnitName;
begin
  FCurrentUnit := Lexer.Token;
  FUsedUnits.Values[FCurrentUnit] := FCurrentUnit;
  inherited;
end;

procedure TPreProcessor.MainUsedUnitExpression;
begin
  FUsedUnits.Values[FCurrentUnit] := Lexer.Token;
  inherited;
end;

procedure TPreProcessor.UsedUnitName;
begin
  FCurrentUnit := Lexer.Token;
  FUsedUnits.Values[FCurrentUnit] := FCurrentUnit;
  inherited;
end;

procedure TPreProcessor.ContainsStatement;
begin
  FCurrentUnit := Lexer.Token;
  FUsedUnits.Values[FCurrentUnit] := FCurrentUnit;
  inherited;
end;

procedure TPreProcessor.ContainsExpression;
begin
  FUsedUnits.Values[FCurrentUnit] := Lexer.Token;
  inherited;
end;

// ------------------------------------------------------------------------------------------

function PreProcessUsedUnits(const aFilename: string; sl: TStrings = nil): integer;
var
  APreProcessor: TPreProcessor;
  vDefines: TmwAssignAbleHash;
  sCurrentDir: string;
  stmFile: TMemoryStream;
begin
  // init result
  Result := 0;

  // check input
  if not FileExists(aFilename) then
    EXIT;

  // save current directory
  sCurrentDir := GetCurrentDir;
  try
    // move to directory of file
    SetCurrentDir(ExtractFilePath(aFilename));

    vDefines := TmwAssignAbleHash.Create(gcwProject.CWOptions.Defines.Text);

  //TODO add codewrench define

    try
      APreProcessor := TPreProcessor.Create(vDefines);
      try
        stmFile := TMemoryStream.Create;
        try
          stmFile.LoadFromFile(aFilename);
          APreProcessor.Run(aFilename, stmFile);
          Result := APreProcessor.UnitCount;
          if Assigned(sl) then
            sl.Assign(APreProcessor.FUsedUnits);
        finally
          stmFile.Free;
        end;
      finally
        APreProcessor.Free;
      end;
    finally
      vDefines.Free;
    end;
  finally
    // restore current directory
    SetCurrentDir(sCurrentDir);
  end;
end;

// ------------------------------------------------------------------------------------------

end.

