(*  Copyright (c) 2002, the CodeLens project
  All rights reserved.

  Redistribution and use in source and binary forms,
  with or without modification, are permitted provided
  that the following conditions are met:

  Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.

  Redistributions in binary form must reproduce the
  above copyright notice, this list of conditions
  and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
  Neither the name of the CodeLens project nor the names
  of its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS
  AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. *)

unit ucwRefactorOptions;

interface

uses
  Classes, ucwClasses;

type
  TcwRefactorOptions = class(TInterfacedObject, IcwRefactorOptions)
  private
    FRefactoringsStringList: TStringList;
    FFolderComparisonExe: string;
    FFolderComparisonExeParams: string;
  public
    constructor Create;
    destructor Destroy; override;

    function GetFolderComparisonExe: string;
    procedure SetFolderComparisonExe(const Value: String);
    property FolderComparisonExe: string read GetFolderComparisonExe write SetFolderComparisonExe;

    function GetFolderComparisonExeParams: string;
    procedure SetFolderComparisonExeParams(const Value: String);
    property FolderComparisonExeParams: string read GetFolderComparisonExeParams write
      SetFolderComparisonExeParams;

    function GetRefactorings: TStrings;
    procedure SetRefactorings(const Value: TStrings);
    property Refactorings: TStrings read GetRefactorings write SetRefactorings;

    procedure Save(const aFile: IcwFileSettings);
    procedure Load(const aFile: IcwFileSettings);
    procedure LoadDefaults;
  end;

implementation

uses
  SysUtils;

{ TcwRefactorOptions }

constructor TcwRefactorOptions.Create;
begin
  inherited Create;
  FRefactoringsStringList := TStringList.Create;
  LoadDefaults;
end;

destructor TcwRefactorOptions.Destroy;
begin
  FRefactoringsStringList.Free;
  inherited;
end;

function TcwRefactorOptions.GetFolderComparisonExe: string;
begin
  Result := FFolderComparisonExe;
end;

function TcwRefactorOptions.GetFolderComparisonExeParams: string;
begin
  Result := FFolderComparisonExeParams;
end;

procedure TcwRefactorOptions.LoadDefaults;
begin
  FFolderComparisonExe := '';
  FFolderComparisonExeParams := '';
end;

procedure TcwRefactorOptions.Load(const aFile: IcwFileSettings);
var
  Index: Integer;
  ACount: Integer;
begin
  FFolderComparisonExe := aFile.GetOption('FolderComparison', 'FolderComparisonExe',
    FFolderComparisonExe);
  FFolderComparisonExeParams := aFile.GetOption('FolderComparison', 'FolderComparisonExeParams',
    FFolderComparisonExeParams);
  FRefactoringsStringList.Clear;
  ACount := 0;
  ACount := aFile.GetOption('Refactorings', 'Count', ACount);
  for Index := 0 to ACount - 1 do
    FRefactoringsStringList.Add(aFile.GetOption('Refactorings', IntToStr(Index), ''));
end;

procedure TcwRefactorOptions.Save(const aFile: IcwFileSettings);
var
  Index: Integer;
begin
  aFile.SetOption('FolderComparison', 'FolderComparisonExe', FFolderComparisonExe);
  aFile.SetOption('FolderComparison', 'FolderComparisonExeParams', FFolderComparisonExeParams);
  aFile.SetOption('Refactorings', 'Count', FRefactoringsStringList.Count);
  for Index := 0 to FRefactoringsStringList.Count - 1 do
    aFile.SetOption('Refactorings', IntToStr(Index), FRefactoringsStringList[Index]);
end;

procedure TcwRefactorOptions.SetFolderComparisonExe(const Value: String);
begin
  FFolderComparisonExe := Value;
end;

procedure TcwRefactorOptions.SetFolderComparisonExeParams(const Value: String);
begin
  FFolderComparisonExeParams := Value;
end;

function TcwRefactorOptions.GetRefactorings: TStrings;
begin
  Result := FRefactoringsStringList;
end;

procedure TcwRefactorOptions.SetRefactorings(const Value: TStrings);
begin
  FRefactoringsStringList.Assign(Value);
end;

end.
