(*  Copyright (c) 2002, the CodeLens project
  All rights reserved.

  Redistribution and use in source and binary forms,
  with or without modification, are permitted provided
  that the following conditions are met:

  Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.

  Redistributions in binary form must reproduce the
  above copyright notice, this list of conditions
  and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
  Neither the name of the CodeLens project nor the names
  of its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS
  AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. *)

unit ucwReportData;

interface
uses
  SysUtils, Classes, Graphics, ucwClasses, Variants;

type
  PVarArray = ^TVarArray;
  TVarArray = array of Variant;

  TcwReportData = class(TINterfacedObject, IcwReportData)
  private
    FList: TList;

    procedure InternalAddRow(const aCodeLocation : TcwCodeLocation; const aDetail : TcwUsedByList;
      const aValues : array of Variant);

  public
    procedure AddTitle(const aValue: variant);overload;
    procedure AddTitle(const aSetting : IcwSetting;const aValue: variant);overload;
    procedure AddTitle(const aSetting : IcwSetting;const aIsLibraryUnit : Boolean;const aValue: variant);overload;


    procedure AddRow(const aCodeLocation : TcwCodeLocation;const aDetails : TcwUsedByList;const aValues : array of Variant);overload;
    procedure AddRow(const aUnit : TcwUnit; const aSetting : IcwSetting;const aCodeLocation : TcwCodeLocation;const aDetails : TcwUsedByList;const aValues : array of Variant);overload;
    procedure AddRow(const aSetting : IcwSetting;const aIsLibraryUnit : Boolean;const aCodeLocation : TcwCodeLocation;const aDetails : TcwUsedByList;const aValues : array of Variant);overload;
    procedure AddRow(const aSetting : IcwSetting;const aCodeLocation : TcwCodeLocation;const aDetails : TcwUsedByList; const aValues : array of Variant);overload;

    procedure AddSummary(const aValues : array of Variant);

    procedure AddEmptyLine;overload;
    procedure AddEmptyLine(const aSetting : IcwSetting);overload;
    procedure AddEmptyLine(const aSetting : IcwSetting;const aIsLibraryUnit : Boolean);overload;

    procedure Clear;
    procedure SortBy(const aAsc: Boolean; const aColumn: Integer);

    function GetValues(Row : Integer; Column : Integer) : Variant;
    property aValues [Row : Integer; Column : Integer] : Variant read GetValues; default;

    function GetDetails(Index : Integer) : TcwUsedByList;
    property Details[Index : Integer] : TcwUsedByList read GetDetails;

    function GetCount : Integer;
    property Count : Integer read GetCount;

    //will call the show source code the the specific row
    procedure ExecRow(const aRow : Integer);

    constructor Create;
    destructor Destroy; override;
  end;

implementation


procedure TcwReportData.InternalAddRow(const aCodeLocation : TcwCodeLocation;const aDetail : TcwUsedByList; const aValues : array of Variant);
var
  P: PVarArray;
  I: Integer;
begin
  New(P);
  SetLength(P^, High(aValues) - Low(aValues) + 3);
  P^[0] := Integer(aDetail);
  P^[1] := Integer(aCodeLocation);
  for I := Low(aValues) to High(aValues) do
     P^[I+2] := aValues[I];
  FList.Add(P);
end;

procedure TcwReportData.AddTitle(const aValue: variant);
begin
  InternalAddRow(nil, nil, [aValue, null]);
end;

procedure TcwReportData.AddTitle(const aSetting : IcwSetting;const aValue: variant);
begin
  if (aSetting.ProjectSet or aSetting.LibrarySet) then
    AddTitle(aValue);
end;


procedure TcwReportData.AddTitle(const aSetting : IcwSetting;const aIsLibraryUnit : Boolean;const aValue: variant);
begin
  if (aSetting.ProjectSet and not aIsLibraryUnit) or
     (aSetting.LibrarySet  and aIsLibraryUnit) then
       AddTitle(aValue);
end;

procedure TcwReportData.AddRow(const aCodeLocation : TcwCodeLocation;const aDetails : TcwUsedByList; const aValues : array of Variant);
begin
  InternalAddRow(aCodeLocation, aDetails, aValues);
end;

procedure TcwReportData.AddRow(const aUnit : TcwUnit; const aSetting : IcwSetting;const aCodeLocation : TcwCodeLocation;const aDetails : TcwUsedByList;const aValues : array of Variant);
begin
  if (aSetting.ProjectSet and not aUnit.IsLibraryUnit) or
     (aSetting.LibrarySet  and aUnit.IsLibraryUnit) then
      AddRow(aCodeLocation,aDetails, aValues);

end;

procedure TcwReportData.AddRow(const aSetting : IcwSetting;const aIsLibraryUnit : Boolean;const aCodeLocation : TcwCodeLocation;
                              const aDetails : TcwUsedByList;const aValues : array of Variant);
begin
  if (aSetting.ProjectSet and not aIsLibraryUnit) or
     (aSetting.LibrarySet  and aIsLibraryUnit) then
      AddRow(aCodeLocation,aDetails, aValues);
end;

procedure TcwReportData.AddRow(const aSetting : IcwSetting;const aCodeLocation : TcwCodeLocation;
                               const aDetails : TcwUsedByList;const aValues : array of Variant);
begin
  if (aSetting.ProjectSet or aSetting.LibrarySet) then
    AddRow(aCodeLocation,aDetails, aValues);
end;

procedure TcwReportData.AddSummary(const aValues : array of Variant);
begin
  InternalAddRow(nil, nil, aValues);
end;

procedure TcwReportData.AddEmptyLine;
begin
  InternalAddRow(nil,nil,[null, null]);
end;

procedure TcwReportData.AddEmptyLine(const aSetting : IcwSetting);
begin
  if (aSetting.ProjectSet or aSetting.LibrarySet) then
    AddEmptyLine;
end;

procedure TcwReportData.AddEmptyLine(const aSetting : IcwSetting;const aIsLibraryUnit : Boolean);
begin
  if (aSetting.ProjectSet and not aIsLibraryUnit) or
     (aSetting.LibrarySet  and aIsLibraryUnit) then
    AddEmptyLine;
end;

function TcwReportData.GetValues(Row : Integer; Column : Integer) : Variant;
var
  P: PVarArray;
begin
  P := FList[Row];
  Result := P^[Column+1];
end;


function TcwReportData.GetDetails(Index : Integer) : TcwUsedByList;
var
  P: PVarArray;
begin
  P := FList[Index];
  Result := TcwUsedByList(Integer(P^[0]));
end;


function TcwReportData.GetCount : Integer;
begin
  Result := FList.Count;
end;

constructor TcwReportData.Create;
begin
  inherited Create;
  FList := TList.Create;
end;

destructor TcwReportData.Destroy;
begin
  Clear;
  FreeAndNil(FList);
  inherited Destroy;
end;

procedure TcwReportData.Clear;
var
  I: Integer;
  P: PVarArray;
begin
  for I := FList.Count - 1 downto 0 do
  begin
    P := FList[I];
    Dispose(P);
  end;
  FList.Clear;
end;

    //will call the show source code the the specific row
procedure TcwReportData.ExecRow(const aRow : Integer);
var P: PVarArray;
begin
  P := FList[aRow];
  gcwProject.ShowLocation(TcwCodeLocation(Integer(P^[1])));
end;


function cwCompareVar(const aLeft, aRight: Variant): Integer;
begin
  if ((VarType(aLeft) = varOleStr) and (VarType(aRight) = varOleStr)) or
     ((VarType(aLeft) = varString) and (VarType(aRight) = varString)) then
    Result := AnsiCompareText(aLeft, aRight) else
  begin
    if aLeft < aRight then  Result := -1
    else if aLeft > aRight then  Result := 1
    else Result := 0;
  end;
end;

var
  SortColumn: Integer;

function ListCompare(Item1, Item2: Pointer): Integer;
begin
  Result := cwCompareVar(PVarArray(Item1)^[SortColumn], PVarArray(Item2)^[SortColumn]);
  if Result = 0 then
  begin
    if Integer(Item1) < Integer(Item2) then  Result := -1
    else if Integer(Item1) > Integer(Item2) then  Result := 1
    else Result := 0;
  end;
end;

function ListCompareDesc(Item1, Item2: Pointer): Integer;
begin
  Result := - cwCompareVar(PVarArray(Item1)^[SortColumn], PVarArray(Item2)^[SortColumn]);
  if Result = 0 then
  begin
    if Integer(Item1) > Integer(Item2) then  Result := -1
    else if Integer(Item1) < Integer(Item2) then  Result := 1
    else Result := 0;
  end;

end;

procedure TcwReportData.SortBy(const aAsc: Boolean; const aColumn: Integer);
var
  nSortColumn: Integer;
begin
  nSortColumn := aColumn + 2;
  SortColumn := nSortColumn;

  if aAsc then
     FList.Sort(ListCompare)
  else
     FList.Sort(ListCompareDesc);
end;

end.
