(*  Copyright (c) 2002, the CodeLens project
  All rights reserved.

  Redistribution and use in source and binary forms,
  with or without modification, are permitted provided
  that the following conditions are met:

  Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.

  Redistributions in binary form must reproduce the
  above copyright notice, this list of conditions
  and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
  Neither the name of the CodeLens project nor the names
  of its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS
  AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. *)

unit ucwReportView;

interface
uses
  SysUtils, Classes, Controls, Graphics, Forms, VirtualTrees, ucwClasses, Variants,
  ExtCtrls, Math, ucwStringTreeEx;

type
  TVTReportViewer = class
  private
    FData       : IcwReportData;
    FDetailData : TcwUsedByList;
    FReport     : IcwReport;
    FTree       : TVirtualStringTreeEx;
    FDetailsTree : TVirtualStringTreeEx;
    FSplitter    : TSplitter;

    procedure OnGetText(Sender: TBaseVirtualTree; Node: PVirtualNode; Column: TColumnIndex;
                            TextType: TVSTTextType; var Text: WideString);
    procedure OnDetailGetText(Sender: TBaseVirtualTree; Node: PVirtualNode;
                        Column: TColumnIndex; TextType: TVSTTextType; var Text: WideString);

    procedure OnHeaderClick(Sender: TVTHeader; Column: TColumnIndex; Button: TMouseButton; Shift: TShiftState; X,
       Y: Integer);

    procedure OnDblClick(Sender: TObject);
    procedure OnChange(Sender: TBaseVirtualTree; Node: PVirtualNode);
    procedure OnDetailDblClick(Sender: TObject);


    procedure OnInitChildren(Sender: TBaseVirtualTree; Node: PVirtualNode;
                             var ChildCount: Cardinal);
    procedure OnDetailInitChildren(Sender: TBaseVirtualTree; Node: PVirtualNode;
                             var ChildCount: Cardinal);

    function  CreateVirtualTreeView (const aParent : TWinControl; aTop : Integer;aHeight : Integer;aAlign : TAlign) : TVirtualStringTreeEx;

    procedure AssignMainEventHandlers(const aTree : TVirtualStringTreeEx);

    procedure AssignDetailEventHandlers(const aTree : TVirtualStringTreeEx);
    procedure AddColumn(aTree : TVirtualStringTreeEx; const aCaption : string;
                        aAlignment : TAlignment; aWidth : Integer);

    procedure CreateDetailColumns(const aTree : TVirtualStringTreeEx);

  public
    destructor Destroy;override;
    procedure CreateReportVirtualTreeView(const aParent : TWinControl; aReport : IcwReport);
    procedure UpdateReportVirtualTreeView(const aParent : TWinControl; aReport : IcwReport);
    property  Data    : IcwReportData read FData;
    property  Report    : IcwReport read FReport;
  end;


implementation

destructor TVTReportViewer.Destroy;
begin
  if FReport <> nil then
  begin
    FReport.Visible := FALSE;
    gcwCodeWrenchOptions.SetOption('Reports',FReport.Name, FTree.Height);
  end;

  FreeAndNil(FTree);
  FreeAndNil(FDetailsTree);
  FreeAndNil(FSplitter);
  inherited Destroy;
end;

procedure TVTReportViewer.OnGetText(Sender: TBaseVirtualTree; Node: PVirtualNode;
  Column: TColumnIndex; TextType: TVSTTextType; var Text: WideString);
begin
  try
    if FData.Count > Integer(Node.Index) then
      Text := VarToStr(FData[Node.Index, Column+1]) else
        Text := '';
  except
    //ShowMessage('');
  end;
end;


procedure TVTReportViewer.OnDetailGetText(Sender: TBaseVirtualTree; Node: PVirtualNode;
  Column: TColumnIndex; TextType: TVSTTextType; var Text: WideString);
var vCodeLocation : TcwUsedByLocation;
begin
  try
    if (FDetailData <> nil) and (FDetailData.Count > Integer(Node.Index)) then
    begin
      vCodeLocation := FDetailData[Node.Index];
      case Column of
        0 : Text := vCodeLocation.UnitIntf.Name;
        1 : Text := Format('%d:%d',[vCodeLocation.X,vCodeLocation.Y]);
        2 : if vCodeLocation.IsWrongCase then
              Text := '  X  ' else
                Text := '';
      end;
    end else
       Text := '';
  except
  end;
end;


procedure TVTReportViewer.OnInitChildren(Sender: TBaseVirtualTree; Node: PVirtualNode;
                             var ChildCount: Cardinal);
begin
  ChildCount := FData.Count;
end;

procedure TVTReportViewer.OnDetailInitChildren(Sender: TBaseVirtualTree; Node: PVirtualNode;
                             var ChildCount: Cardinal);
begin
  if FDetailData <> nil then
    ChildCount := 0 else
      ChildCount := FDetailData.Count;
end;

procedure TVTReportViewer.OnDblClick(Sender: TObject);
var vNode : PVirtualNode;
begin
  vNode := FTree.FocusedNode;
  if vNode <> nil then
  begin
    FData.ExecRow(vNode.Index);
  end;
end;


procedure TVTReportViewer.OnChange(Sender: TBaseVirtualTree; Node: PVirtualNode);
begin
  if FDetailsTree = nil then EXIT;
  FDetailsTree.RootNodeCount := 0;
  if Node <> nil then
  begin
    FDetailData := FData.Details[Node.Index];
    if FDetailData <> nil then
      FDetailsTree.RootNodeCount := FDetailData.Count;
  end;
  FDetailsTree.Invalidate;
end;

procedure TVTReportViewer.OnDetailDblClick(Sender: TObject);
var vCodeLocation : TcwUsedByLocation;
var vNode : PVirtualNode;
begin
  vNode := FDetailsTree.FocusedNode;
  if vNode <> nil then
  begin
    vCodeLocation := FDetailData[vNode.Index];
    gcwProject.ShowLocation(vCodeLocation);
  end;
end;


procedure TVTReportViewer.OnHeaderClick(Sender: TVTHeader; Column: TColumnIndex; Button: TMouseButton; Shift: TShiftState; X,
    Y: Integer);
var
  bAsc: Boolean;
begin
  if (Sender.SortColumn = Column) {and (Sender.SortDirection = sdAscending)} then
  begin
    if (Sender.SortDirection = sdDescending) then
    begin
      bAsc := True;
      Sender.SortDirection := sdAscending;
    end else
    begin
      bAsc := False;
      Sender.SortDirection := sdDescending;
    end;
  end
  else
  begin
    bAsc := True;
    Sender.SortColumn := Column;
    Sender.SortDirection := sdDescending;
  end;
  FData.SortBy(bAsc, Column);

  Sender.Treeview.Invalidate;
end;

function TVTReportViewer.CreateVirtualTreeView (const aParent : TWinControl; aTop : Integer;aHeight : Integer;aAlign : TAlign) : TVirtualStringTreeEx;
begin
  Result := TVirtualStringTreeEx.Create(aParent);
  Result.Parent := aParent;
  Result.Top := aTop;
  Result.Height := aHeight;
  Result.Align := aAlign;
  Result.AutoScrollDelay := 500;
  Result.BevelInner := bvNone;
  Result.BevelKind := bkFlat;
  Result.BorderStyle := bsNone;
  Result.ButtonFillMode := fmShaded;
  Result.CheckImageKind := ckXP;
  Result.Colors.FocusedSelectionColor := clActiveCaption;
  Result.Colors.FocusedSelectionBorderColor := clActiveCaption;
  Result.Colors.GridLineColor := clBtnShadow;
  Result.Colors.UnfocusedSelectionBorderColor := clBtnShadow;
  Result.DrawSelectionMode := smBlendedRectangle;
  Result.Header.Height := 20;
  Result.Header.Options := [{hoAutoResize,} hoColumnResize, hoDrag, hoShowSortGlyphs, hoVisible];
  Result.Header.SortColumn := 0;
  Result.Header.Style := hsThickButtons;
  Result.HintAnimation := hatFade;
  Result.HintMode      := hmHint;
  Result.IncrementalSearchDirection := sdForward;
  Result.TreeOptions.AnimationOptions := [toAnimatedToggle];
  Result.TreeOptions.AutoOptions := [];
  Result.TreeOptions.MiscOptions := [toInitOnSave, toReportMode, toWheelPanning];
  Result.TreeOptions.PaintOptions := [toShowHorzGridLines, toShowVertGridLines, toShowDropmark, toShowRoot, toThemeAware];
  Result.TreeOptions.SelectionOptions := [toFullRowSelect];
  Result.AutoWidth := True;
end;

procedure TVTReportViewer.AssignMainEventHandlers(const aTree : TVirtualStringTreeEx);
begin
  aTree.OnGetText      := Self.OnGetText;
  aTree.OnDblClick     := Self.OnDblClick;
  aTree.OnChange       := Self.OnChange;
  aTree.OnInitChildren := Self.OnInitChildren;

end;


procedure TVTReportViewer.AssignDetailEventHandlers(const aTree : TVirtualStringTreeEx);
begin
  aTree.OnGetText := Self.OnDetailGetText;
  aTree.OnDblClick  := Self.OnDetailDblClick;
  aTree.OnInitChildren := Self.OnDetailInitChildren;
end;


procedure TVTReportViewer.AddColumn(aTree : TVirtualStringTreeEx; const aCaption : string;
                                   aAlignment : TAlignment; aWidth : Integer);
var AColumn : TVirtualTreeColumn;
begin
  AColumn := aTree.Header.Columns.Add;
  AColumn.Alignment := aAlignment;
  AColumn.Color   := clWhite;
  AColumn.Options := [coAllowClick, coEnabled, coParentBidiMode, coResizable, coShowDropMark, coVisible];
  AColumn.Width   := aWidth;
  AColumn.Text    := aCaption;

end;

procedure TVTReportViewer.CreateDetailColumns(const aTree : TVirtualStringTreeEx); 
begin 
  aTree.Header.Columns.BeginUpdate; 
  try 
    AddColumn(aTree,'Unit', taLeftJustify, 100); 
    AddColumn(aTree,'Location', taCenter, 80); 
    AddColumn(aTree,'Wrong Case', taCenter, 80); 
  finally 
    aTree.Header.Columns.EndUpdate; 
  end; 
end; 

procedure TVTReportViewer.CreateReportVirtualTreeView(const aParent : TWinControl; aReport : IcwReport);

var i       : Integer;
var cwColumn : IcwColumn;
var vAlignment : TAlignment;
var vWidth     : Integer;
var vHeight    : Integer;
begin
  FReport := aReport;
  FDetailData := nil;
  if FReport.HasDetails then
  begin
    vHeight := gcwCodeWrenchOptions.GetOption('Reports',FReport.Name, 300);
    FTree := CreateVirtualTreeView(aParent, 0, Min(vHeight, aParent.Height - 80), alTop);
    AssignMainEventHandlers(FTree);

    FSplitter := TSplitter.Create(aParent);
    FSplitter.Parent := aParent;
    FSplitter.Align := alTop;
    FSplitter.Visible := True;
    FSplitter.Top := FTree.Top + FTree.Height + 10;
    FSplitter.Cursor := crVSplit;
    FSplitter.Height := 6;
    FDetailsTree := CreateVirtualTreeView(aParent, FSplitter.Top + 10, 350, alClient);
    AssignDetailEventHandlers(FDetailsTree);
    CreateDetailColumns(FDetailsTree);
  end else
  begin
    FTree := CreateVirtualTreeView(aParent, 0, 350, alClient);
    AssignMainEventHandlers(FTree);
  end;


  FData := aReport.Data;


  FTree.Header.Columns.BeginUpdate; 
  try 
    for i := 0 to aReport.Columns.Count - 1 do
    begin 
      cwColumn := aReport.Columns[i]; 
      if cwColumn.DataType = dtString then 
      begin 
        vWidth   := 80; 
        vAlignment := taLeftJustify; 
      end else 
      begin 
        vAlignment := taRightJustify; 
         if cwColumn.DataType = dtBoolean then 
            vWidth   :=  20 
          else 
            vWidth   :=  80; 
      end; 
      AddColumn(FTree,cwColumn.Name, vAlignment, vWidth); 
    end; 
    FTree.Header.AutoSizeIndex := 0; 
  finally 
    FTree.Header.Columns.EndUpdate; 
  end; 


end;

procedure TVTReportViewer.UpdateReportVirtualTreeView(const aParent : TWinControl;aReport : IcwReport);
var vTree : TVirtualStringTreeEx;
var i : Integer;
begin
  FDetailData := nil;
  for i := 0 to aParent.ControlCount - 1 do
  if aParent.Controls[i] = FTree then
  begin
    vTree := FTree;
    vTree.RootNodeCount := aReport.Data.Count;
    if aReport.Sortable then
      vTree.OnHeaderClick := Self.OnHeaderClick else
        vTree.OnHeaderClick := nil;
    vTree.Invalidate;
  end;
end;

end.
