(*  Copyright (c) 2002, the CodeLens project
  All rights reserved.

  Redistribution and use in source and binary forms,
  with or without modification, are permitted provided
  that the following conditions are met:

  Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.

  Redistributions in binary form must reproduce the
  above copyright notice, this list of conditions
  and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
  Neither the name of the CodeLens project nor the names
  of its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS
  AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. *)

unit ucwReports;

interface
uses
  SysUtils, Classes, ucwClasses;

type
  TcwReports = class(TInterfacedObject, IcwReports)
  private
    FReports : TInterfaceList;
  public
    constructor Create;
    destructor Destroy;override;

    function GetReports(Index : Integer) : IcwReport;
    property Reports[Index : Integer] : IcwReport read GetReports;default;

    function GetCount : Integer;
    property Count : Integer read GetCount;

    function GetReportByName(const aReportName : string) : IcwReport;

    procedure Clear;

    //add a reporting interface
    //returns the index
    function  AddReport(const aReport : IcwReport) : Integer;

    //remove a report from the list of installed reports
    function RemoveReport(const aReport : IcwReport) : Boolean;

  end;

implementation
uses
  ucwReport;
constructor TcwReports.Create;
begin
  inherited;
  FReports := TInterfaceList.Create;
end;


destructor TcwReports.Destroy;
begin
  FReports.Free;
  inherited;
end;

function TcwReports.GetReports(Index : Integer) : IcwReport;
begin
  Result := IcwReport(FReports[Index]);
end;

function TcwReports.GetCount : Integer;
begin
  Result := FReports.Count;
end;

function TcwReports.GetReportByName(const aReportName : string) : IcwReport;
var i : Integer;
begin
  Result := nil;
  for i := 0 to Count - 1 do
    if SameText(Reports[i].Name, aReportName) then
    begin
      Result := Reports[i];
      EXIT;
    end;
end;
procedure TcwReports.Clear;
var
  i: integer;
begin
  for i:=0 to Count-1 do
    Reports[i].Clear;
end;

//add a reporting interface
//returns the index
function  TcwReports.AddReport(const aReport : IcwReport) : Integer;
begin
  Result := FReports.Add(aReport);
end;

//returns a newly created reporting interface
function TcwReports.RemoveReport(const aReport : IcwReport) : Boolean;

begin
  Result := FReports.Remove(aReport) >= 0;
end;

end.
