(*  Copyright (c) 2002, the CodeLens project
  All rights reserved.

  Redistribution and use in source and binary forms,
  with or without modification, are permitted provided
  that the following conditions are met:

  Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.

  Redistributions in binary form must reproduce the
  above copyright notice, this list of conditions
  and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
  Neither the name of the CodeLens project nor the names
  of its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS
  AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. *)

unit ucwSettings;

interface
uses
  Classes, ucwClasses, ucwSettingsCategory;

type
  TcwSettings = class(TcwSettingsCategory, IcwSettings, IcwSettingsCategory)
  private
    procedure LoadCategory(const aFileSettings : IcwFileSettings;const aCategory : IcwSettingsCategory);
    procedure SaveCategory(const aFileSettings : IcwFileSettings;const aCategory : IcwSettingsCategory);
  public
    //save setings
    procedure SaveSettings(const aFileSettings : IcwFileSettings);
    //load settings from options file
    procedure LoadSettings(const aFileSettings : IcwFileSettings);
  end;

implementation

procedure TcwSettings.SaveCategory(const aFileSettings : IcwFileSettings;const aCategory : IcwSettingsCategory);
var i : Integer;
var Setting : IcwSetting;
begin
  for i := 0 to aCategory.Count - 1 do
  begin
    Setting := aCategory[i];
    aFileSettings.SetOption(aCategory.Name,'Project' + Setting.Name, Setting.ProjectSet);
    aFileSettings.SetOption(aCategory.Name, 'Library' + Setting.Name, Setting.LibrarySet);
  end;
  for i := 0 to aCategory.CategoryCount - 1 do
    SaveCategory(aFileSettings, aCategory.Categories[i]);
end;

procedure TcwSettings.LoadCategory(const aFileSettings : IcwFileSettings;const aCategory : IcwSettingsCategory);
var i : Integer;
var Setting : IcwSetting;
begin
  for i := 0 to aCategory.Count - 1 do
  begin
    Setting := aCategory[i];
    Setting.ProjectSet := aFileSettings.GetOption(aCategory.Name,'Project' + Setting.Name, Setting.ProjectSet);
    Setting.LibrarySet := aFileSettings.GetOption(aCategory.Name, 'Library' + Setting.Name, Setting.LibrarySet);
  end;
  for i := 0 to aCategory.CategoryCount - 1 do
    LoadCategory(aFileSettings,aCategory.Categories[i]);
end;

//save setings
procedure TcwSettings.SaveSettings(const aFileSettings : IcwFileSettings);
begin
  SaveCategory(aFileSettings, self);
end;

//load settings from options file
procedure TcwSettings.LoadSettings(const aFileSettings : IcwFileSettings);
begin
  LoadCategory(aFileSettings, Self);
end;

end.
