(*  Copyright (c) 2002, the CodeLens project
  All rights reserved.

  Redistribution and use in source and binary forms,
  with or without modification, are permitted provided
  that the following conditions are met:

  Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.

  Redistributions in binary form must reproduce the
  above copyright notice, this list of conditions
  and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
  Neither the name of the CodeLens project nor the names
  of its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS
  AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. *)

unit ucwSettingsCategory;

interface
uses
  Classes, ucwClasses;

type
  TcwSettingsCategory = class(TINterfacedObject, IcwSettingsCategory)
  private
    FName : string;
    FList : TInterfaceList;
    FSubCategories : TInterfaceList;
  public
    constructor Create(const aName : string);
    destructor Destroy;override;

    function GetName : string;
    property Name : string read GetName;

    //all settings for this category
    function GetItems(Index : Integer) : IcwSetting;
    property Items[Index : Integer] : IcwSetting read GetItems; default;

    function GetCount : Integer;
    property Count : Integer read GetCount;



    function AddSetting(const aName: string;
                          const aDefaultProject : Boolean;
                          const aDefaultLibrary : Boolean) : IcwSetting;
    //settings by sub-category
    function GetCategories(Index : Integer) : IcwSettingsCategory;
    property Categories[Index : Integer] : IcwSettingsCategory read GetCategories;

    function GetCategoryCount : Integer;
    property CategoryCount : Integer read GetCategoryCount;

    function AddCategory(const aName : string) : IcwSettingsCategory;

   end;

implementation
uses
  ucwSetting;

constructor TcwSettingsCategory.Create(const aName : string);
begin
  inherited Create;
  FList := TInterfaceList.Create;
  FSubCategories := TInterfaceList.Create;
  FName := aName;
end;


destructor TcwSettingsCategory.Destroy;
begin
  FList.Free;
  FSubCategories.Free;
  inherited Destroy;
end;

function TcwSettingsCategory.GetName : string;
begin
  Result := FName;
end;

//all settings for this category
function TcwSettingsCategory.GetItems(Index : Integer) : IcwSetting;
begin
  Result := IcwSetting(FList[Index]);
end;

function TcwSettingsCategory.GetCount : Integer;
begin
  Result := FList.Count;
end;



function TcwSettingsCategory.AddSetting(const aName: string;
                                        const aDefaultProject : Boolean;
                                        const aDefaultLibrary : Boolean) : IcwSetting;
begin
  Result := TcwSetting.Create(aName, FName, aDefaultProject,aDefaultLibrary);
  FList.Add(Result);

end;

//settings by sub-category
function TcwSettingsCategory.GetCategories(Index : Integer) : IcwSettingsCategory;
begin
  Result := IcwSettingsCategory(FSubCategories[Index]);
end;

function TcwSettingsCategory.GetCategoryCount : Integer;
begin
  Result := FSubCategories.Count;
end;

function TcwSettingsCategory.AddCategory(const aName : string) : IcwSettingsCategory;
begin
  Result := TcwSettingsCategory.Create(aName);
  FSubCategories.Add(Result);
end;

end.
