(*  Copyright (c) 2002, the CodeLens project
  All rights reserved.

  Redistribution and use in source and binary forms,
  with or without modification, are permitted provided
  that the following conditions are met:

  Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.

  Redistributions in binary form must reproduce the
  above copyright notice, this list of conditions
  and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
  Neither the name of the CodeLens project nor the names
  of its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS
  AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. *)

unit ucwStringList;

interface

uses
  Classes,
  ucwClasses;

type
  TcwStringList = class(TInterfacedObject, IcwStringList)
  private
    FStrings: TStringList;
  protected
    function GetText: string;
    procedure SetText(const aValue: string);
    function GetCount: Integer;
    function GetString(Index: Integer): string;
  public
    constructor Create;
    destructor Destroy; override;
    function Add(const aValue: string): Integer;
    procedure SaveToFile(const aFileName: string);
    procedure CopyToClipboard;
    procedure Clear;
    function IndexOf(AString: string): Integer;
    property Strings[Index: Integer]: string read GetString; default;
    property Count: Integer read GetCount;
    property Text: string read GetText write SetText;
  end;

implementation

uses
  ClipBrd;

constructor TcwStringList.Create;
begin
  inherited;
  FStrings := TStringList.Create;
end;

destructor TcwStringList.Destroy;
begin
  FStrings.Free;
  inherited;
end;

function TcwStringList.GetString(Index: Integer): string;
begin
  Result := FStrings[Index];
end;

function TcwStringList.Add(const aValue: string): Integer;
begin
  Result := FStrings.Add(aValue);
end;

function TcwStringList.GetCount: Integer;
begin
  Result := FStrings.Count;
end;

function TcwStringList.GetText: string;
begin
  Result := FStrings.Text;
end;

procedure TcwStringList.SetText(const aValue: string);
begin
  FStrings.Text := aValue;
end;

procedure TcwStringList.SaveToFile(const aFileName: string);
begin
  FStrings.SaveToFile(aFileName);
end;

procedure TcwStringList.CopyToClipboard;
begin
  Clipboard.SetTextBuf(PChar(FStrings.Text));
end;

procedure TcwStringList.Clear;
begin
  FStrings.Clear;
end;

function TcwStringList.IndexOf(AString: string): Integer;
begin
  Result := FStrings.IndexOf(AString);
end;

end.
