(*  Copyright (c) 2002, the CodeLens project
  All rights reserved.

  Redistribution and use in source and binary forms,
  with or without modification, are permitted provided
  that the following conditions are met:

  Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.

  Redistributions in binary form must reproduce the
  above copyright notice, this list of conditions
  and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
  Neither the name of the CodeLens project nor the names
  of its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS
  AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. *)

unit ucwStringLiteralsReport;
interface
uses
  Classes, ucwClasses, ucwReport;

type
  TRPTStringLiterals = class(TcwReport, IcwReport)
  private
    FOptions : IcwSetting;
  public
    function GetName : string;override;
    procedure CreateColumns;override;
    procedure CreateOptions;override;
      //run the report, so it will be filled with Data
    procedure DoExecute;override;
  end;

procedure AddRPTStringLiterals;


implementation


function TRPTStringLiterals.GetName : string;
begin
  Result := 'String Literals';
end;


procedure TRPTStringLiterals.CreateColumns;
begin
  Columns.AddColumn('Unit',dtString);
  Columns.AddColumn('String',dtString);
  Columns.AddColumn('Line',dtInteger);
end;


procedure TRPTStringLiterals.CreateOptions;
begin
  FOptions := Settings.AddSetting('Report strings', True, True);
end;

//run the report, so it will be filled with Data
procedure TRPTStringLiterals.DoExecute;
var i, j : Integer;
var vUnit   : TcwUnit;
var vLiteral  : TcwSymbol;
begin
  for i := 0 to gcwProject.AllUnits.Count - 1 do
  begin
    vUnit := gcwProject.AllUnits[i];

    for j := 0 to vUnit.StringLiterals.Count - 1 do
    begin
      vLiteral := vUnit.StringLiterals[j];
      Data.AddRow(vUnit, FOptions , vLiteral, nil, [vUnit.Name, vLiteral.Name, vLiteral.Y]);
    end;
  end;
end;


procedure AddRPTStringLiterals;
begin
  gcwProject.Reports.AddReport(TRPTStringLiterals.Create);
end;
end.
