(*  Copyright (c) 2002, the CodeLens project
  All rights reserved.

  Redistribution and use in source and binary forms,
  with or without modification, are permitted provided
  that the following conditions are met:

  Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.

  Redistributions in binary form must reproduce the
  above copyright notice, this list of conditions
  and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
  Neither the name of the CodeLens project nor the names
  of its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS
  AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. *)

//TVirtualStringTreeEx - implemented by Alex Danvy
unit ucwStringTreeEx;

interface
uses
  VirtualTrees;

type
TVirtualStringTreeEx = class(TVirtualStringTree)
  private
    FSizing: Boolean;
    FColumnsWidth: array of Single;
    FAutoWidth: boolean;
    procedure SetAutoWidth(const aValue: boolean);
  protected
    procedure DoColumnResize(Column: TColumnIndex); override;
    procedure Resize; override;
  published
    property AutoWidth: boolean read FAutoWidth write SetAutoWidth;
  end;

implementation

uses
  Classes;

type
  TCollectionHack = class(TCollection);

{ TVirtualStringTreeEx }

procedure TVirtualStringTreeEx.DoColumnResize(Column: TColumnIndex);
var
  I, iWidth, iStart, iEnd, iCount, iColCount, iDelta: Integer;
  D2, Carry: Single;
begin
  if FSizing or (TCollectionHack(Header.Columns).UpdateCount > 0) then
    Exit;
  if Column >= 0 then
    inherited;
  iColCount := Header.Columns.Count;
  if iColCount = 0 then
    Exit;
  if Length(FColumnsWidth) <> iColCount then
    SetLength(FColumnsWidth, iColCount);
  iWidth := ClientWidth;
  BeginUpdate;
  try
      for I := iColCount - 1 downto 0 do
      begin
        iDelta := Header.Columns[I].Width;
        Dec(iWidth, iDelta);
        if Round(FColumnsWidth[I]) <> iDelta then
          FColumnsWidth[I]:= iDelta;
      end;
      if iWidth = 0 then
        Exit;
      FSizing := True;
      try
        if Column = iColCount - 1 then
        begin
          iStart := 0;
          iEnd := Column - 1;
        end
        else
        begin
          iStart := Column + 1;
          iEnd := iColCount - 1;
        end;
        iCount := iEnd - iStart + 1;
        if iCount > 0 then
        begin
          D2 := iWidth / iCount;
          for I := iStart to iEnd do
            with Header.Columns[I] do
              if Round(FColumnsWidth[I] + D2) < MinWidth then
              begin
                Dec(iCount);
                Inc(iWidth, Width - MinWidth);
                FColumnsWidth[I] := MinWidth;
              end;
        end;
        if iCount > 0 then
        begin
          D2 := iWidth / iCount;
          Carry:= 0;
          for I:= iStart to iEnd - 1 do
            with Header.Columns[I] do
              if Round(FColumnsWidth[I] + D2) >= MinWidth then
              begin
                FColumnsWidth[I] := FColumnsWidth[I] + D2 + Carry;
                iDelta := Round(FColumnsWidth[I]);
                Carry := FColumnsWidth[I] - iDelta;
                Dec(iWidth, iDelta - Width);
                FColumnsWidth[I] := iDelta;
              end;
        end;
        if (iWidth <> 0) and (iEnd >= 0) then
          with Header.Columns[iEnd] do
          begin
            if Width + iWidth > MinWidth then
              FColumnsWidth[iEnd] := Width + iWidth;
          end;
        for I := 0 to iColCount - 1 do
          Header.Columns[I].Width := Trunc(FColumnsWidth[I]);
      finally
        FSizing := False;
      end;
  finally
    EndUpdate;
  end;
end;

procedure TVirtualStringTreeEx.Resize;
begin
  inherited;
  if AutoWidth then
    DoColumnResize(-1);
end;

procedure TVirtualStringTreeEx.SetAutoWidth(const aValue: boolean);
begin
  if aValue <> FAutoWidth then
  begin
    FAutoWidth := aValue;
    Resize;
  end;
end;


end.
