(*  Copyright (c) 2002, the CodeLens project
  All rights reserved.

  Redistribution and use in source and binary forms,
  with or without modification, are permitted provided
  that the following conditions are met:

  Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.

  Redistributions in binary form must reproduce the
  above copyright notice, this list of conditions
  and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
  Neither the name of the CodeLens project nor the names
  of its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS
  AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. *)

unit ucwSystemOptions;

interface

uses
  Classes,
  ucwClasses;

type
  TcwSystemOptions = class(TInterfacedObject, IcwSystemOptions)
  private
    FRefactorOptions: IcwRefactorOptions;
    FLibrarySearchPath: IcwStringList;
    function GetRefactorOptions: IcwRefactorOptions;
  protected
    function GetLibrarySearchPath: IcwStringList;
  public
    constructor Create;
    procedure Save(const aFile: IcwFileSettings);
    procedure Load(const aFile: IcwFileSettings);
    procedure LoadDefaults;
    property LibrarySearchPath: IcwStringList read GetLibrarySearchPath;
    property RefactorOptions: IcwRefactorOptions read GetRefactorOptions;
  end;

implementation

uses
  SysUtils,
  ucwRefactorOptions,
  ucwStringList;

{ TcwSystemOptions }

constructor TcwSystemOptions.Create;
begin
  inherited Create;
  FLibrarySearchPath := TcwStringList.Create;
  FRefactorOptions := TcwRefactorOptions.Create;
  LoadDefaults;
end;

function TcwSystemOptions.GetLibrarySearchPath: IcwStringList;
begin
  Result := FLibrarySearchPath;
end;

procedure TcwSystemOptions.LoadDefaults;
begin
  FLibrarySearchPath.Clear;
  FRefactorOptions.LoadDefaults;
end;

procedure TcwSystemOptions.Load(const aFile: IcwFileSettings);
begin
  aFile.ReadSection('librarysearchpath', FLibrarySearchPath);
  FRefactorOptions.Load(aFile);
end;

procedure TcwSystemOptions.Save(const aFile: IcwFileSettings);
begin
  aFile.SaveSection('librarysearchpath', FLibrarySearchPath);
  FRefactorOptions.Save(aFile);
end;

function TcwSystemOptions.GetRefactorOptions: IcwRefactorOptions;
begin
  Result := FRefactorOptions;
end;

end.
