(*  Copyright (c) 2002, the CodeLens project
  All rights reserved.

  Redistribution and use in source and binary forms,
  with or without modification, are permitted provided
  that the following conditions are met:

  Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.

  Redistributions in binary form must reproduce the
  above copyright notice, this list of conditions
  and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
  Neither the name of the CodeLens project nor the names
  of its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS
  AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. *)

unit ucwSystemSymbolsReport;

interface
uses
  Classes, ucwClasses, ucwReport;

type
  TRPTSystemSymbols = class(TcwReport, IcwReport)
  private
    FOptions : IcwSetting;
  public
    function  GetName : string;override;
    procedure CreateColumns;override;
    procedure CreateOptions;override;
    function GetHasDetails : Boolean;override;

      //run the report, so it will be filled with Data
    procedure DoExecute;override;
  end;

procedure AddSystemSymolsReport;


implementation


function TRPTSystemSymbols.GetName : string;
begin
  Result := 'System symbols';
end;


procedure TRPTSystemSymbols.CreateColumns;
begin
  Columns.AddColumn('Symbol',dtString);
  Columns.AddColumn('Type',dtString);
  Columns.AddColumn('Used Count',dtInteger);
end;

procedure TRPTSystemSymbols.CreateOptions;
begin
  FOptions := Settings.AddSetting('Report Classes', True, True);
  FOptions := Settings.AddSetting('Report Legacy Objects', True, True);
  FOptions := Settings.AddSetting('Report Interfaces', True, True);
  FOptions := Settings.AddSetting('Report Symbols', True, True);
end;

function TRPTSystemSymbols.GetHasDetails : Boolean;
begin
  Result := True;
end;

//run the report, so it will be filled with Data
procedure TRPTSystemSymbols.DoExecute;
var i        : Integer;
var vSymbol : TcwSymbol;
begin
  for i := 0 to gcwProject.UnparsedClasses.Count - 1 do
  begin
    vSymbol := gcwProject.UnparsedClasses[i];
    if vSymbol.UsedBy.Count > 0 then
      Data.AddRow(vSymbol, vSymbol.UsedBy, [vSymbol.Name, 'Class', vSymbol.UsedBy.Count]);
  end;

  for i := 0 to gcwProject.UnparsedObjects.Count - 1 do
  begin
    vSymbol := gcwProject.UnparsedObjects[i];
    if vSymbol.UsedBy.Count > 0 then
      Data.AddRow(vSymbol, vSymbol.UsedBy, [vSymbol.Name, 'Object', vSymbol.UsedBy.Count]);
  end;

  for i := 0 to gcwProject.UnparsedInterfaces.Count - 1 do
  begin
    vSymbol := gcwProject.UnparsedInterfaces[i];
    if vSymbol.UsedBy.Count > 0 then
      Data.AddRow(vSymbol, vSymbol.UsedBy, [vSymbol.Name, 'Interface', vSymbol.UsedBy.Count]);
  end;

  for i := 0 to gcwProject.UnparsedSymbols.Count - 1 do
  begin
    vSymbol := gcwProject.UnparsedSymbols[i];
    if vSymbol.UsedBy.Count > 0 then
      Data.AddRow(vSymbol, vSymbol.UsedBy, [vSymbol.Name, 'Symbol', vSymbol.UsedBy.Count]);
  end;

end;


procedure AddSystemSymolsReport;
begin
  gcwProject.Reports.AddReport(TRPTSystemSymbols.Create);
end;

end.
