(*  Copyright (c) 2002, the CodeLens project
  All rights reserved.

  Redistribution and use in source and binary forms,
  with or without modification, are permitted provided
  that the following conditions are met:

  Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.

  Redistributions in binary form must reproduce the
  above copyright notice, this list of conditions
  and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
  Neither the name of the CodeLens project nor the names
  of its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS
  AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. *)

unit ucwUsedUnitsReport;

interface
uses
  Classes, ucwClasses, ucwReport;

type
  TRPTUsedUnits = class(TcwReport, IcwReport)
  private
    FOptions : IcwSetting;
  public
    function GetName : string;override;
    procedure CreateColumns;override;
    procedure CreateOptions;override;
    function GetHasDetails : Boolean;override;
      //run the report, so it will be filled with Data
    procedure DoExecute;override;
  end;

procedure AddRPTUsedUnits;


implementation


function TRPTUsedUnits.GetName : string;
begin
  Result := 'Used Units';
end;


procedure TRPTUsedUnits.CreateColumns;
begin
  Columns.AddColumn('Full Path',dtString);
  Columns.AddColumn('Unit',dtString);
  Columns.AddColumn('Used Count',dtInteger);
  Columns.AddColumn('Code Lines',dtInteger);
  Columns.AddColumn('Comment Lines',dtInteger);
  Columns.AddColumn('Empty Lines',dtInteger);
  Columns.AddColumn('Total Lines',dtInteger);
  Columns.AddColumn('Parsed [Y/N]',dtBoolean);
  Columns.AddColumn('Project Unit [Y/N]',dtBoolean);
end;
procedure TRPTUsedUnits.CreateOptions;
begin
  FOptions := Settings.AddSetting('Report Units', True, True);
end;

function TRPTUsedUnits.GetHasDetails : Boolean;
begin
  Result := True;
end;

//run the report, so it will be filled with Data
procedure TRPTUsedUnits.DoExecute;

const BoolStr : array[Boolean] of string = ('N','Y');

var i : Integer;
var vUnit   : TcwUnit;
var vUnitStats : TcwCodeStatistics;
begin
  for i := 0 to gcwProject.AllUnits.Count - 1 do
  begin
    vUnit := gcwProject.AllUnits[i];
    Assert(False);
    //TODO: vUnitStats := vUnit.CodeStats;
    Data.AddRow(vUnit, FOptions, vUnit, vUnit.UsedBy, [vUnit.FullFileName, vUnit.Name, vUnit.UsedBy.Count,
                vUnitStats.CodeLines, vUnitStats.CommentLines, vUnitStats.EmptyLines, vUnitStats.TotalLines,
                BoolStr[vUnit.Parsed = psrFull],
                BoolStr[not vUnit.IsLibraryUnit]]);
  end;
end;


procedure AddRPTUsedUnits;
begin
  gcwProject.Reports.AddReport(TRPTUsedUnits.Create);
end;

end.
