(*  Copyright (c) 2002, the CodeLens project
  All rights reserved.

  Redistribution and use in source and binary forms,
  with or without modification, are permitted provided
  that the following conditions are met:

  Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.

  Redistributions in binary form must reproduce the
  above copyright notice, this list of conditions
  and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
  Neither the name of the CodeLens project nor the names
  of its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS
  AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. *)

unit ucwVarParser;

interface

uses
  Classes,
  ucwClasses,
  ucwUnitParser,
  mwSimplePasPar;

type
  TVarParser = class(TUnitParser)
  protected
    function ParseConstantDeclaration(const aContainer: TcwVarContainer): TcwVariable;
    procedure ParseConstSection(const aContainer: TcwVarContainer);
    function ParseResourceDeclaration(const aContainer: TcwVarContainer): TcwVariable;
    procedure ParseVarNameList(const aContainer: TcwVarContainer; IsThreadVar: Boolean);
    procedure ParseVarDeclaration(const aContainer: TcwVarContainer; IsThreadVar: Boolean);
    procedure ParseVarSection(const aContainer: TcwVarContainer);
  public
  end;

implementation

uses
  mwPasLexTypes,
  mwSimplePasParTypes;

function TVarParser.ParseResourceDeclaration(const AContainer: TcwVarContainer): TcwVariable;
begin
  Result := AContainer.Constants.AddVariable(Lexer.Token, UnitVisibility, True,
    FileIndex, Lexer.PosXY.X + 1, Lexer.PosXY.Y + 1);
  Expected(ptIdentifier);
  Expected(ptEqual);
  ParseCharString;
end;

function TVarParser.ParseConstantDeclaration(const aContainer: TcwVarContainer): TcwVariable;
begin
  Result := aContainer.Constants.AddVariable(Lexer.Token, UnitVisibility, True,
    FileIndex, Lexer.PosXY.X + 1, Lexer.PosXY.Y + 1);
  ConstantName;
  case TokenID of
    ptEqual: ConstantEqual;
    ptColon: begin
      Expected(ptColon);
      Result.TypeIntf := ParseTypeKind(aContainer, Lexer.Token, Lexer.PosXY, False);
      Expected(ptEqual);
      ConstantValueTyped;
    end;
    else
      SynError(InvalidConstantDeclaration);
  end;
end;

procedure TVarParser.ParseConstSection(const aContainer: TcwVarContainer);
begin
  IncIndentLevel;
  case TokenID of
    ptConst: begin
      NextToken;
      while (TokenID = ptIdentifier) do
      begin
        ParseConstantDeclaration(aContainer);
        HintDirectives;
        SEMICOLON;
      end;
    end;
    ptResourceString: begin
      NextToken;
      while (TokenID = ptIdentifier) do
      begin
        ParseResourceDeclaration(AContainer);
        HintDirectives;
        SEMICOLON;
      end;
    end;
    else
      SynError(InvalidConstSection);
  end;
  DecIndentLevel;
end;

procedure TVarParser.ParseVarNameList(const aContainer: TcwVarContainer; IsThreadVar: Boolean);
var
  vVariable: TcwVariable;
begin
  vVariable := aContainer.Variables.AddVariable(Lexer.Token, UnitIntf.Visibilty, False,
    FileIndex, Lexer.PosXY.X + 1, Lexer.PosXY.Y + 1);
  vVariable.IsThreadVar := IsThreadVar;
  VarName;
  while TokenID = ptComma do
  begin
    NextToken;
    vVariable := aContainer.Variables.AddVariable(Lexer.Token, UnitIntf.Visibilty, False,
      FileIndex, Lexer.PosXY.X + 1, Lexer.PosXY.Y + 1);
    vVariable.IsThreadVar := IsThreadVar;
    VarName;
  end;
end;

procedure TVarParser.ParseVarDeclaration(const aContainer: TcwVarContainer; IsThreadVar: Boolean);
var
  Cnt: Integer;
  i: Integer;
  vType: TcwSymbol;
begin
  Cnt := aContainer.Variables.Count;
  ParseVarNameList(aContainer, IsThreadVar);
  Expected(ptColon);
  vType := ParseTypeKind(aContainer, Lexer.Token, Lexer.PosXY, False);
  if vType <> nil then
    for i := Cnt to aContainer.Variables.Count - 1 do
      aContainer.Variables[i].TypeIntf := vType;

  HintDirectives;
  case GenID of
    ptAbsolute: VarAbsolute;
    ptEqual: VarEqual;
  end;
end;

procedure TVarParser.ParseVarSection(const aContainer: TcwVarContainer);
var
  IsThreadVar: Boolean;
begin
  IsThreadVar := False;
  case TokenID of
    ptThreadVar: begin
      IsThreadVar := True;
      NextToken;
    end;
    ptVar: NextToken;
    else
      SynError(InvalidVarSection);
  end;
  while TokenID = ptIdentifier do
  begin
    ParseVarDeclaration(aContainer, IsThreadVar);
    SEMICOLON;
  end;
end;

end.
